// pb_data module for retrieving pushbutton data from the 
// Xilinx Multimedia board
// Based on Push Button Scan Data In (PB_SCAN_DATA_IN.v)
// by Xilinx, Inc. June 2001.
//
// Modifications to base design made by Blair Fort for
// ECE532 - University of Toronto.  Winter 2004.
//

module volume_buttons
(
reset,
// reset_interrupt,
PB_CLOCK_P,
PB_DATA_P,

// interrupt,
// pb_status,
// startup

mic_volume,
song_volume
);

input 			reset;				// system reset
// input 			reset_interrupt;	// interrupt reset
input 			PB_CLOCK_P;			// data shift clock sourced by the CPLD
input 			PB_DATA_P;			// serial push button data from CPLD

// output			interrupt;			// 1 - when there is new data, 0 - after reset or reset_interrupt
// output [9:0]	pb_status;			// The status of the pushbuttons
// output			startup;				// 1 - tells the CPLD to be fully operational, 0 - CPLD off

output [2:0]	mic_volume;		// volume for mic (bits ? of pcm register)
output [2:0]	song_volume;	// volume for song (bits ? of pcm register) 

//parameter C_BASEADDR = 32'hffffff00;
//parameter C_HIGHADDR = 32'hffffffff;

// wire				interrupt;
// wire	[9:0]		pb_status;

// wire				startup; 

wire 				reset;
// wire				reset_interrupt;
wire 				pb_data;
wire 				pb_shift_clock;

// wire				pb_reset_interrupt;		
// reg				interrupt_reg;
reg [9:0] 		data_shift_reg;

reg [2:0]	mic_volume;		// volume for mic (bits ? of pcm register)
reg [2:0]	song_volume;	// volume for song (bits ? of pcm register) 

// create the internal signals from the ports
assign pb_shift_clock 		= PB_CLOCK_P;
assign pb_data 				= PB_DATA_P;
// assign pb_reset_interrupt	= reset | reset_interrupt;

//  turn the CPLD on
// assign startup				= 1'b1;

// latch the data transmitted by the CPLD 
always @ (posedge reset or posedge pb_shift_clock) begin
	if (reset) begin
		data_shift_reg[9:0] <= 10'h000;
		end
	else begin
		data_shift_reg[9:1] <= data_shift_reg[8:0];
		data_shift_reg[0] <= pb_data;
		end
	end

// if data is transmitted by the CPLD set the interrupt high
/* always @ (posedge pb_reset_interrupt or posedge pb_shift_clock) begin
	if (pb_reset_interrupt) begin
		interrupt_reg <= 1'b0;
		end
	else begin
		interrupt_reg <= 1'b1;
		end
	end
*/

// Route the pb data register and interrupt to the output
// assign	pb_status	= data_shift_reg;
// assign	interrupt	= interrupt_reg;

// decode the pb data in the data register
always @ (posedge reset or posedge pb_shift_clock) 
   begin
	if (reset) 		// reset to nothing selected
		begin		
		mic_volume	<= 3'b000;
		song_volume	<= 3'b100;
		end

	else  
		begin
 		if( data_shift_reg[8])	// change song_vol if song button chosen
			song_volume <= data_shift_reg[2:0];
		else	if(data_shift_reg[9])	// change mic_vol if mic button chosen
		   mic_volume <= data_shift_reg[2:0];
		   
		   	/*case (data_shift_reg[7:0])
				8'h80: mic_volume <= 3'b111; // lowest volume
				8'h40: mic_volume <= 3'b110;
				8'h20: mic_volume <= 3'b101;
				8'h10: mic_volume <= 3'b100;
				8'h08: mic_volume <= 3'b011;
				8'h04: mic_volume <= 3'b010;
				8'h02: mic_volume <= 3'b001;
				8'h01: mic_volume <= 3'b000; // highest volume
				default: mic_volume <= 3'b000;
			endcase */

		end
   end

	   
endmodule // pb_data



