// pb_data module for retrieving pushbutton data from the 
// Xilinx Multimedia board
// Based on Push Button Scan Data In (PB_SCAN_DATA_IN.v)
// by Xilinx, Inc. June 2001.
//
// Modifications to base design made by Blair Fort for
// ECE532 - University of Toronto.  Winter 2004.
//

module volume_buttons
(
clock,
reset_b,
PB_CLOCK_P,
PB_DATA_P,

mic_volume,
song_volume,
startup


// reset_interrupt,
// interrupt,
// pb_status,


);



input clock;		// system clock 27 MHz
input reset_b;		// system reset
input PB_CLOCK_P;	// data shift clock sourced by the CPLD
input PB_DATA_P;	// serial push button data from CPLD
output	startup;	 // 1 - tells the CPLD to be fully operational, 0 - CPLD off
output [2:0] mic_volume;   // volume for mic (bits ? of pcm register)
output [2:0] song_volume;  // volume for song (bits ? of pcm register) 




// input 			reset_interrupt;	// interrupt reset
// output interrupt;	// 1 - when there is new data, 0 - after reset or reset_interrupt
// output [9:0]	pb_status;			// The status of the pushbuttons

// wire				interrupt;
// wire	[9:0]		pb_status;
// wire				reset_interrupt;

wire clock;
wire reset_b;
wire pb_data;
wire pb_shift_clock;

reg [9:0] data_shift_reg;
reg [2:0] mic_volume;	// volume for mic (bits ? of pcm register)
reg [2:0] song_volume;	// volume for song (bits ? of pcm register) 

reg [9:0] 	data_reg;
reg [7:0] 	data_bit_count;
reg		latch_data;


// wire				pb_reset_interrupt;		
// reg				interrupt_reg;

// create the internal signals from the ports
assign pb_shift_clock = PB_CLOCK_P;
assign pb_data = PB_DATA_P;


// assign pb_reset_interrupt	= reset | reset_interrupt;

//  turn the CPLD on
assign startup	= 1'b1;

// latch the data transmitted by the CPLD 
always @ (negedge reset_b or posedge pb_shift_clock) begin
	if (!reset_b) begin
		data_shift_reg[9:0] <= 10'h000;
		end
	else begin
		data_shift_reg[9:1] <= data_shift_reg[8:0];
		data_shift_reg[0] <= pb_data;
		end
	end


// create a bit counter
always @ (negedge reset_b or posedge pb_shift_clock or posedge latch_data) begin
//	if (!reset_b | latch_data) begin	  //this caused an error
	if (!reset_b) begin
		data_bit_count <= 8'h00;
		end
	else if (latch_data)
		data_bit_count <= 8'h00;
	else begin
		data_bit_count <= data_bit_count +1;
		end
	end

// create a signal to transfer the data to a latch
always @ (negedge reset_b or posedge clock) begin
	if (!reset_b) begin
		latch_data <= 1'b0;
		end
	else if (data_bit_count == 8'h0A) begin
		latch_data <= 1'b1;
		end
	else begin
		latch_data <= 1'b0;
		end
	end


// load the received data into a data register
always @ (negedge reset_b or posedge latch_data) begin
	if (!reset_b) begin
		data_reg[9:0]	<= 10'h000;	// clear data registers on power up
		end
	else begin
		data_reg[9:0]	<= data_shift_reg[9:0];	// latch the data from the shift register
		end
	end



// if data is transmitted by the CPLD set the interrupt high
/* always @ (posedge pb_reset_interrupt or posedge pb_shift_clock) begin
	if (pb_reset_interrupt) begin
		interrupt_reg <= 1'b0;
		end
	else begin
		interrupt_reg <= 1'b1;
		end
	end
*/

// Route the pb data register and interrupt to the output
// assign	pb_status	= data_shift_reg;
// assign	interrupt	= interrupt_reg;


// decode the pb data in the data register
always @ (negedge reset_b or posedge clock) 
   begin
	if (!reset_b) 		// reset to nothing selected
		begin		
		mic_volume	<= 3'b000;
		song_volume	<= 3'b100;
		end

	else  
		begin
 		if( data_reg[8])	// change song_vol if song button chosen
         	   song_volume <= data_reg[2:0];
		else if(data_reg[9])	// change mic_vol if mic button chosen
		   mic_volume <= data_reg[2:0];
		   
		   	/*case (data_reg[7:0])
				8'h80: mic_volume <= 3'b111; // lowest volume
				8'h40: mic_volume <= 3'b110;
				8'h20: mic_volume <= 3'b101;
				8'h10: mic_volume <= 3'b100;
				8'h08: mic_volume <= 3'b011;
				8'h04: mic_volume <= 3'b010;
				8'h02: mic_volume <= 3'b001;
				8'h01: mic_volume <= 3'b000; // highest volume
				default: mic_volume <= 3'b000;
			endcase */

		end
   end

	   
endmodule // pb_data



