/* HTAB = 4 */
/****************************************************************************
 * madlld.c -- A simple program decoding an mpeg audio stream to 16-bit		*
 * PCM from stdin to stdout. This program is just a simple sample			*
 * demonstrating how the low-level libmad API can be used.					*
 *--------------------------------------------------------------------------*
 * (c) 2001, 2002 Bertrand Petit											*
 *																			*
 * Redistribution and use in source and binary forms, with or without		*
 * modification, are permitted provided that the following conditions		*
 * are met:																	*
 *																			*
 * 1. Redistributions of source code must retain the above copyright		*
 *    notice, this list of conditions and the following disclaimer.			*
 *																			*
 * 2. Redistributions in binary form must reproduce the above				*
 *    copyright notice, this list of conditions and the following			*
 *    disclaimer in the documentation and/or other materials provided		*
 *    with the distribution.												*
 * 																			*
 * 3. Neither the name of the author nor the names of its contributors		*
 *    may be used to endorse or promote products derived from this			*
 *    software without specific prior written permission.					*
 * 																			*
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''		*
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED		*
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A			*
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR		*
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,				*
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT			*
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF			*
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND		*
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,		*
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT		*
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF		*
 * SUCH DAMAGE.																*
 *																			*
 ****************************************************************************/

/*
 * $Name: v1_0p1 $
 * $Date: 2002/01/08 01:06:56 $
 * $Revision: 1.9 $
 */

/****************************************************************************
 * Includes																	*
 ****************************************************************************/
//#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <mad.h>
#include <xuartlite.h>      /* For serial IO? */
#include <xbasic_types.h>
#include <xtmrctr.h>
#include <xtmrctr_l.h>		/* to stop counter before initializing it */
#include <xintc.h>
#include <xintc_l.h>		/* to disable interrupts before setting up interrupt controller */

#define printf xil_printf

// -----------------------------------
// Karaoke Machine
#include <xgpio.h>          /* layer 1 GPIO device driver                   */

#define LED1    0x8          /* bit 0 of GPIO is connected to the LED1 */
#define SWITCH1 0x4			/* bit 1 is connected to sw1 */
#define LED0    0x2          /* bit 2 of GPIO is connected to the LED0 */
#define SWITCH0 0x1			/* bit 3 is connected to sw0 */

#define MIC 0x7
#define SONG 0x7
// -----------------------------------

/* Usha -- added */
/*  MICROBLAZE */
/* The assumption is that the first word of the data memory will contain the file size */
/* (in bytes), followed by the file itself. */
/**************/

extern char sound_data[];
extern int sound_data_size;


/****************************************************************************
 * Global variables.														*
 ****************************************************************************/
const char	ProgName[10] = "madlld";

/****************************************************************************
 * Return an error string associated with a mad error code.					*
 ****************************************************************************/
/* Mad version 0.14.2b introduced the mad_stream_errorstr() function.
 * For previous library versions a replacement is provided below.
 */
#if (MAD_VERSION_MAJOR>=1) || \
    ((MAD_VERSION_MAJOR==0) && \
     (((MAD_VERSION_MINOR==14) && \
       (MAD_VERSION_PATCH>=2)) || \
      (MAD_VERSION_MINOR>14)))
#define MadErrorString(x) mad_stream_errorstr(x)
#else
static const char *MadErrorString(const struct mad_stream *Stream)
{
	switch(Stream->error)
	{
		/* Generic unrecoverable errors. */
		case MAD_ERROR_BUFLEN:
			return("input buffer too small (or EOF)");
		case MAD_ERROR_BUFPTR:
			return("invalid (null) buffer pointer");
		case MAD_ERROR_NOMEM:
			return("not enough memory");

		/* Frame header related unrecoverable errors. */
		case MAD_ERROR_LOSTSYNC:
			return("lost synchronization");
		case MAD_ERROR_BADLAYER:
			return("reserved header layer value");
		case MAD_ERROR_BADBITRATE:
			return("forbidden bitrate value");
		case MAD_ERROR_BADSAMPLERATE:
			return("reserved sample frequency value");
		case MAD_ERROR_BADEMPHASIS:
			return("reserved emphasis value");

		/* Recoverable errors */
		case MAD_ERROR_BADCRC:
			return("CRC check failed");
		case MAD_ERROR_BADBITALLOC:
			return("forbidden bit allocation value");
		case MAD_ERROR_BADSCALEFACTOR:
			return("bad scalefactor index");
		case MAD_ERROR_BADFRAMELEN:
			return("bad frame length");
		case MAD_ERROR_BADBIGVALUES:
			return("bad big_values count");
		case MAD_ERROR_BADBLOCKTYPE:
			return("reserved block_type");
		case MAD_ERROR_BADSCFSI:
			return("bad scalefactor selection info");
		case MAD_ERROR_BADDATAPTR:
			return("bad main_data_begin pointer");
		case MAD_ERROR_BADPART3LEN:
			return("bad audio data length");
		case MAD_ERROR_BADHUFFTABLE:
			return("bad Huffman table select");
		case MAD_ERROR_BADHUFFDATA:
			return("Huffman data overrun");
		case MAD_ERROR_BADSTEREO:
			return("incompatible block_type for JS");

		/* Unknown error. This swich may be out of sync with libmad's
		 * defined error codes.
		 */
		default:
			return("Unknown error code");
	}
}
#endif

/****************************************************************************
 * Converts a sample from mad's fixed point number format to an unsigned	*
 * short (16 bits).															*
 ****************************************************************************/
static unsigned short MadFixedToUshort(mad_fixed_t Fixed)
{
	/* A fixed point number is formed of the following bit pattern:
	 *
	 * SWWWFFFFFFFFFFFFFFFFFFFFFFFFFFFF
	 * MSB                          LSB
	 * S ==> Sign (0 is positive, 1 is negative)
	 * W ==> Whole part bits
	 * F ==> Fractional part bits
	 *
	 * This pattern contains MAD_F_FRACBITS fractional bits, one
	 * should alway use this macro when working on the bits of a fixed
	 * point number. It is not guaranteed to be constant over the
	 * different platforms supported by libmad.
	 *
	 * The unsigned short value is formed by the least significant
	 * whole part bit, followed by the 15 most significant fractional
	 * part bits. Warning: this is a quick and dirty way to compute
	 * the 16-bit number, madplay includes much better algorithms.
	 */
	Fixed=Fixed>>(MAD_F_FRACBITS-15);
	return((unsigned short)Fixed);
}

/****************************************************************************
 * Print human readable informations about an audio MPEG frame.				*
 ****************************************************************************/
/* Usha -- changed
static int PrintFrameInfo(FILE *fp, struct mad_header *Header)
*/
static int PrintFrameInfo(struct mad_header *Header)
{
	const char	*Layer,
				*Mode,
				*Emphasis;

	/* Convert the layer number to it's printed representation. */
	switch(Header->layer)
	{
		case MAD_LAYER_I:
			Layer="I";
			break;
		case MAD_LAYER_II:
			Layer="II";
			break;
		case MAD_LAYER_III:
			Layer="III";
			break;
		default:
			Layer="(unexpected layer value)";
			break;
	}

	/* Convert the audio mode to it's printed representation. */
	switch(Header->mode)
	{
		case MAD_MODE_SINGLE_CHANNEL:
			Mode="single channel";
			break;
		case MAD_MODE_DUAL_CHANNEL:
			Mode="dual channel";
			break;
		case MAD_MODE_JOINT_STEREO:
			Mode="joint (MS/intensity) stereo";
			break;
		case MAD_MODE_STEREO:
			Mode="normal LR stereo";
			break;
		default:
			Mode="(unexpected mode value)";
			break;
	}

	/* Convert the emphasis to it's printed representation. */
	switch(Header->emphasis)
	{
		case MAD_EMPHASIS_NONE:
			Emphasis="no";
			break;
		case MAD_EMPHASIS_50_15_US:
			Emphasis="50/15 us";
			break;
		case MAD_EMPHASIS_CCITT_J_17:
			Emphasis="CCITT J.17";
			break;
		default:
			Emphasis="(unexpected emphasis value)";
			break;
	}

	/* Usha -- changed */
	/*	fprintf(fp,"%s: %lu kb/s audio mpeg layer %s stream %s crc, " */
	/* *************
	printf("%s: %u kb/s audio mpeg layer %s stream %s crc, "
			"%s with %s emphasis at %u Hz sample rate\n",
			ProgName,Header->bitrate,Layer,
			Header->flags&MAD_FLAG_PROTECTION?"with":"without",
			Mode,Emphasis,Header->samplerate);
	**************/
	/* Usha -- changed */
	/*	return(ferror(fp)); */
	return(0);
}

/****************************************************************************
 * Main decoding loop. This is where mad is used.							*
 ****************************************************************************/
#define INPUT_BUFFER_SIZE	(5*8192)
#define OUTPUT_BUFFER_SIZE	8192 /* Must be an integer multiple of 4. */
/* Usha -- changed
static int MpegAudioDecoder(FILE *InputFp, FILE *OutputFp)
*/

unsigned char *InputPtr = (unsigned char *) 0xfa800004;
unsigned short *OutputPtr = (unsigned short *) 0xfc800000;




#ifndef NO_PLAY
extern unsigned short *PlayPcmPtr;
#include "xparameters.h"
#endif


#define STOP while(1)

char NextLine[1024];
long NextLineTime, PreviousLineTime;
int count, no_more_lyrics;
char new_time[128];
int FileSize;

/* definitions used in setting up the counter */

#define CLOCK_MHZ       100000000 
#define CLOCKS_PER_MS   (CLOCK_MHZ / 1000)
#define TIMER_COUNTER_0	0

/* test lyrics related stuff */

#undef IGNORE_TIMED_LYRICS
#undef TESTING_TIMED_LYRICS
#define ONLY_PRINT_LYRICS

#ifndef IGNORE_TIMED_LYRICS
/* Global variables used by timer, interrupt controller, and lyrics
 * handling stuff
 */

XIntc InterruptController;
XTmrCtr TimerCounter;

#endif

char *lyricsMemPointer; 

//function declarations
void TimerCounterHandler(void *CallBackRef, Xuint8 TimerCounterNumber);
void initializeLyrics();


//int main(void)
void main(void)
{
	struct mad_stream	Stream;
	struct mad_frame	Frame;
	struct mad_synth	Synth;
	//mad_timer_t			Timer;
/*  Usha -- commented out
	unsigned char		InputBuffer[INPUT_BUFFER_SIZE],
						OutputBuffer[OUTPUT_BUFFER_SIZE],
						*OutputPtr=OutputBuffer;
	const unsigned char	*OutputBufferEnd=OutputBuffer+OUTPUT_BUFFER_SIZE;
*/

	
	int					Status=0,
						i;
	unsigned long		FrameCount=0;

	/* Usha -- added 3 lines */
	int first_time = 1;
	int *prev_Frame_ptr = 0;
	//	char *InputPtr = (char *)(data_buffer + sizeof(int));
	//	int FileSize = data_buffer[0];
	//char *InputPtr = (char *)(FILE_BASEADDR + sizeof(int));
	
	/* Usha -- CHANGE FOR NOW ****************
	char *InputPtr = sound_data;
	int FileSize = sound_data_size;
	*/
	

// ----------------------------------
	// Karaoke Machine
	XGpio Gpio;
	XStatus GpioStatus;

	XGpio Gpio_micvol;
	XGpio Gpio_songvol;
	

	Xuint32 curr_mic_vol;
	Xuint32 curr_song_vol;

	int loop_count = 0;
	int restarted, paused;

	FileSize = *((int *)0xfa800000);

	restarted = 0;
	paused = 0;

// ----------------------------------


	// ----------------------------

	GpioStatus = XGpio_Initialize(&Gpio, XPAR_OPB_GPIO_0_DEVICE_ID);
	//printf("init for gpio 0 successful\n");
	
    if (GpioStatus != XST_SUCCESS)
    {
        printf("GPIO 0 initialization error\n\r\r");
    }


	GpioStatus = XGpio_Initialize(&Gpio_micvol, XPAR_OPB_GPIO_1_DEVICE_ID);
	//printf("init for gpio 1 successful\n");

	if (GpioStatus != XST_SUCCESS)
    {
        printf("GPIO 1 initialization error\n\r\r");
    }



	GpioStatus = XGpio_Initialize(&Gpio_songvol, XPAR_OPB_GPIO_2_DEVICE_ID);
	//printf("init for gpio 2 successful\n");
		
	if (GpioStatus != XST_SUCCESS)
    {
        printf("GPIO 2 initialization error\n\r\r");
    }


    /* Set the direction for all signals to be inputs except the LED
     * outputs
     */    
	XGpio_SetDataDirection(&Gpio, 1, SWITCH0 | SWITCH1);  // bits set to 1 are inputs
	XGpio_SetDataDirection(&Gpio_micvol, 1, MIC);
	XGpio_SetDataDirection(&Gpio_songvol, 1, SONG);

	// printf("\n\rswitches initialized\r\n");

	// ----------------------------




	/* First the structures used by libmad must be initialized. */
 	mad_stream_init(&Stream);
	mad_frame_init(&Frame);
	mad_synth_init(&Synth);
	//mad_timer_reset(&Timer);

#ifndef IGNORE_TIMED_LYRICS

	XIntc_mMasterDisable(XPAR_OPB_INTC_0_BASEADDR);

	/* Ensure timer is not running */
	XTmrCtr_mDisable(XPAR_OPB_TIMER_0_BASEADDR, TIMER_COUNTER_0);

	restarted = 0;
	initializeLyrics();

	if (!no_more_lyrics) {	/* if all lyrics are at time = 0, updateLyrics
							 * will return 1, and there won't be any need
							 * to setup and run the timer and interrupt
							 * controller */

		/* Set up and start the interrupt controller and timer that 
		 * will be used for lyrics display */

	    /* Initialize the timer counter */
	    Status = XTmrCtr_Initialize(&TimerCounter, 
	                                XPAR_OPB_TIMER_0_DEVICE_ID);
	    if (Status != XST_SUCCESS)
	    {
	        printf("Timer counter initialization error\n\r\r");
	    }

	    /* Perform a self-test */
	    Status = XTmrCtr_SelfTest(&TimerCounter, 
	                              TIMER_COUNTER_0);
	    if (Status != XST_SUCCESS)
	    {
	        printf("Timer counter self-test error\n\r");
	    }

	    /* Setup the handler for the timer counter */
	    XTmrCtr_SetHandler(&TimerCounter, 
	                       TimerCounterHandler, &TimerCounter);

	    /* Enable the interrupt of the timer counter and set it to count down */
	    XTmrCtr_SetOptions(&TimerCounter, TIMER_COUNTER_0,
	                       XTC_INT_MODE_OPTION | XTC_DOWN_COUNT_OPTION | XTC_AUTO_RELOAD_OPTION);

#ifdef TESTING_TIMED_LYRICS
//	NextLineInterval = 1000; //using different tests now
#endif


	    /* Set the reset value for the timer counter to the time until
	     * the first line needs to be printed */
	    XTmrCtr_SetResetValue(&TimerCounter, 
	                           TIMER_COUNTER_0, NextLineTime * CLOCKS_PER_MS);

		/* Set up the interrupt controller */

	    /* Initialize the interrupt controller driver */
	    Status = XIntc_Initialize(&InterruptController, XPAR_OPB_INTC_0_DEVICE_ID);
	    if (Status != XST_SUCCESS)
	    {
	        printf("Interrupt controller initialization error\n\r");
	    }

	    /* Connect the device driver handler */
	    Status = XIntc_Connect(&InterruptController,
	                           XPAR_OPB_INTC_0_OPB_TIMER_0_INTERRUPT_INTR,
	                           (XInterruptHandler)XTmrCtr_InterruptHandler,
	                           &TimerCounter);
	    if (Status != XST_SUCCESS)
	    {
	        printf("Interrupt controller connect error\n\r");
	    }

	    /* Start the interrupt controller so interrupts are enabled for all
	     * devices that cause interrupts. Specify real mode so that the timer
	     * counter can cause interrupts through the interrupt controller.
	     */
	    Status = XIntc_Start(&InterruptController, XIN_REAL_MODE);
	    if (Status != XST_SUCCESS)
	    {
	        printf("Interrupt controller start error\n\r");
	    }

	    /* Enable the interrupt for the timer counter and enable interrupts in
	     * the microblaze processor
	     */
	    XIntc_Enable(&InterruptController,
	                 XPAR_OPB_INTC_0_OPB_TIMER_0_INTERRUPT_INTR);

	    microblaze_enable_interrupts();

	    /* Start the timer counter such that it's incrementing 
	     */
	    XTmrCtr_Start(&TimerCounter, TIMER_COUNTER_0);

	} // end of the if that decides whether to setup the timer and stuff

#endif

	/* Decoding options can here be set in the options field of the
	 * Stream structure.
	 */

	/* This is the decoding loop. */
	do
	{
		/* The input bucket must be filled if it becomes empty or if
		 * it's the first execution of the loop.
		 */
	  /* Usha -- added this check */
        if (first_time) {
			mad_stream_buffer(&Stream,InputPtr,FileSize);
			Stream.error=0;
			first_time = 0;
		} else {
		  /* Usha -- commented out
			 if(Stream.buffer==NULL || Stream.error==MAD_ERROR_BUFLEN)
			{
			size_t			ReadSize,
							Remaining;
			unsigned char	*ReadStart;

		  */
			/* {1} libmad may not consume all bytes of the input
			 * buffer. If the last frame in the buffer is not wholly
			 * contained by it, then that frame's start is pointed by
			 * the next_frame member of the Stream structure. This
			 * common situation occurs when mad_frame_decode() fails,
			 * sets the stream error code to MAD_ERROR_BUFLEN, and
			 * sets the next_frame pointer to a non NULL value. (See
			 * also the comment marked {2} bellow.)
			 *
			 * When this occurs, the remaining unused bytes must be
			 * put back at the beginning of the buffer and taken in
			 * account before refilling the buffer. This means that
			 * the input buffer must be large enough to hold a whole
			 * frame at the highest observable bit-rate (currently 448
			 * kb/s). XXX=XXX Is 2016 bytes the size of the largest
			 * frame? (448000*(1152/32000))/8
			 */
			if(Stream.next_frame!=NULL)
			{
			  /* Usha -- changed
				Remaining=Stream.bufend-Stream.next_frame;
				memmove(InputBuffer,Stream.next_frame,Remaining);
				ReadStart=InputBuffer+Remaining;
				ReadSize=INPUT_BUFFER_SIZE-Remaining;
			  */
			  if ((int *)Stream.next_frame != prev_Frame_ptr) {
				prev_Frame_ptr = Stream.next_frame;
				mad_stream_buffer(&Stream, Stream.next_frame, Stream.bufend-Stream.next_frame);
				Stream.error=0;
			  } else {
				break;
			  }
			}
			else
			  /* Usha -- changed
				ReadSize=INPUT_BUFFER_SIZE,
					ReadStart=InputBuffer,
					Remaining=0;
			  */
			  break; /* end of input stream */
			
			/* Fill-in the buffer. If an error occurs print a message
			 * and leave the decoding loop. If the end of stream is
			 * reached we also leave the loop but the return status is
			 * left untouched.
			 */
			/* Usha -- commented out 
			ReadSize=fread(ReadStart,1,ReadSize,InputFp);
			if(ReadSize<=0)
			{
				if(ferror(InputFp))
				{
					fprintf(stderr,"%s: read error on bitstream (%s)\n",
							ProgName,strerror(errno));
					Status=1;
				}
				if(feof(InputFp))
					fprintf(stderr,"%s: end of input stream\n",ProgName);
				break;
			}
			*/

			/* Pipe the new buffer content to libmad's stream decoder
             * facility.
			 */
			/* Usha -- commented out
			mad_stream_buffer(&Stream,InputBuffer,ReadSize+Remaining);
			Stream.error=0;
			}
			*/
		}

		/* Decode the next mpeg frame. The streams is read from the
		 * buffer, its constituents are break down and stored the the
		 * Frame structure, ready for examination/alteration or PCM
		 * synthesis. Decoding options are carried in the Frame
		 * structure from the Stream structure.
		 *
		 * Error handling: mad_frame_decode() returns a non zero value
		 * when an error occurs. The error condition can be checked in
		 * the error member of the Stream structure. A mad error is
		 * recoverable or fatal, the error status is checked with the
		 * MAD_RECOVERABLE macro.
		 *
		 * {2} When a fatal error is encountered all decoding
		 * activities shall be stopped, except when a MAD_ERROR_BUFLEN
		 * is signaled. This condition means that the
		 * mad_frame_decode() function needs more input to achieve
		 * it's work. One shall refill the buffer and repeat the
		 * mad_frame_decode() call. Some bytes may be left unused at
		 * the end of the buffer if those bytes forms an incomplete
		 * frame. Before refilling, the remainign bytes must be moved
		 * to the begining of the buffer and used for input for the
		 * next mad_frame_decode() invocation. (See the comments marked
		 * {1} earlier for more details.)
		 *
		 * Recoverable errors are caused by malformed bit-streams, in
		 * this case one can call again mad_frame_decode() in order to
		 * skip the faulty part and re-sync to the next frame.
		 */
		if(mad_frame_decode(&Frame,&Stream)) {
			if(MAD_RECOVERABLE(Stream.error))
			{
			  /* Usha -- changed
				fprintf(stderr,"%s: recoverable frame level error (%s)\n",
						ProgName,MadErrorString(&Stream));
				fflush(stderr);
			  */
			  /* ****************
				printf("%s: recoverable frame level error (%s)\n",
						ProgName,MadErrorString(&Stream));
 			  *************** */
				continue;
			}
			else
				if(Stream.error==MAD_ERROR_BUFLEN)
					continue;
				else
				{
				  /* Usha -- changed
					fprintf(stderr,"%s: unrecoverable frame level error (%s).\n",
							ProgName,MadErrorString(&Stream));
				  */
				  /* **************
					printf("%s: unrecoverable frame level error (%s).\n",
							ProgName,MadErrorString(&Stream));
							********* */
					Status=1;
					break;
				}
		}

		/* The characteristics of the stream's first frame is printed
		 * on stderr. The first frame is representative of the entire
		 * stream.
		 */
		/* Usha -- changed 
		if(FrameCount==0)
		*/
		if (FrameCount==0) {

		  /* Usha -- changed
			if(PrintFrameInfo(stderr,&Frame.header))
		  */
			if(PrintFrameInfo(&Frame.header))
			{
				Status=1;
				break;
			}
			/* Usha -- added */
#ifndef NO_PLAY
            init_sound(Frame.header.samplerate);
#endif
		}

		/* Accounting. The computed frame duration is in the frame
		 * header structure. It is expressed as a fixed point number
		 * whole data type is mad_timer_t. It is different from the
		 * samples fixed point format and unlike it, it can't directly
		 * be added or substracted. The timer module provides several
		 * functions to operate on such numbers. Be careful there, as
		 * some functions of mad's timer module receive some of their
		 * mad_timer_t arguments by value!
		 */
		FrameCount++;
		//putnum(FrameCount);
		//mad_timer_add(&Timer,Frame.header.duration);
				
		/* Once decoded the frame is synthesized to PCM samples. No errors
		 * are reported by mad_synth_frame();
		 */
		mad_synth_frame(&Synth,&Frame);

		/* Synthesized samples must be converted from mad's fixed
		 * point number to the consumer format. Here we use unsigned
		 * 16 bit big endian integers on two channels. Integer samples
		 * are temporarily stored in a buffer that is flushed when
		 * full.
		 */
        
		for(i=0;i<Synth.pcm.length;i++)
		{
			unsigned short	Sample;
           
			/* Left channel */
 			Sample=MadFixedToUshort(Synth.pcm.samples[0][i]);

/*
			*(OutputPtr++)=Sample>>8;
			*(OutputPtr++)=Sample&0xff;
*/
			
			/* Usha -- changed
			*(OutputPtr++)=Sample&0xff;
			*(OutputPtr++)=Sample>>8;
			*/

#ifdef NO_PLAY
			*(OutputPtr++) = Sample;
#else
			play_sound((int) Sample);
#endif
			/* Right channel. If the decoded stream is monophonic then
			 * the right output channel is the same as the left one.
			 */
			if(MAD_NCHANNELS(&Frame.header)==2)
               Sample=MadFixedToUshort(Synth.pcm.samples[1][i]);


/*
			*(OutputPtr++)=Sample>>8;
			*(OutputPtr++)=Sample&0xff;
*/
			/* Usha -- changed
			*(OutputPtr++)=Sample&0xff;
			*(OutputPtr++)=Sample>>8;
			*/

#ifdef NO_PLAY
			*(OutputPtr++) = Sample;
#else
			play_sound((int) Sample);
            
#endif
			/* Flush the buffer if it is full. */
			/* Usha -- deleted
			if(OutputPtr==OutputBufferEnd)
			{
				if(fwrite(OutputBuffer,1,OUTPUT_BUFFER_SIZE,OutputFp)!=OUTPUT_BUFFER_SIZE)
				{
					fprintf(stderr,"%s: PCM write error (%s).\n",
							ProgName,strerror(errno));
					Status=2;
					break;
				}
				OutputPtr=OutputBuffer;
			}
			*/
            
		}

		// ----------------------------------------
		switch(loop_count) {
			case 0:
			// Read and decode the volume pushbuttons
			curr_mic_vol = XGpio_DiscreteRead(&Gpio_micvol, 1 ); // & MIC;
#ifndef ONLY_PRINT_LYRICS
			printf("curr_mic_vol bef manipulation: %x\r\n", curr_mic_vol);
#endif
			switch (curr_mic_vol) {
				case 7:
				// curr_mic_vol = 0x8008;  // mute mic
				curr_mic_vol = 0x0040;
				break;
				case 6:
				// curr_mic_vol = 0x0040;
				curr_mic_vol = 0x0043;
				break;
				case 5:
				// curr_mic_vol = 0x0040;
				curr_mic_vol = 0x0047;
				break;
				case 4:
				// curr_mic_vol = 0x0040;
				curr_mic_vol = 0x004A;
				break;
				case 3:
				// curr_mic_vol = 0x0040;
				curr_mic_vol = 0x0052;
				XGpio_DiscreteWrite(&Gpio, 1, ~(LED1 | LED0)); // write 0 to turn LED1 and LED0 on
				break;
				case 2:
				// curr_mic_vol = 0x0040;
				curr_mic_vol = 0x005A;
				XGpio_DiscreteWrite(&Gpio, 1, ~LED1); // write 0 to turn LED1 on
				break;
				case 1:
				// curr_mic_vol = 0x0040;
				curr_mic_vol = 0x005C;
				XGpio_DiscreteWrite(&Gpio, 1, ~LED0); // write 0 to turn LED0 on
				break;
				case 0:
				curr_mic_vol = 0x005F;
				XGpio_DiscreteWrite(&Gpio, 1, LED1 | LED0); // write 1 to turn both off
				break;
			}

//			curr_mic_vol = ((~curr_mic_vol & MIC) << 2) | 0x40 ;
#ifndef ONLY_PRINT_LYRICS
			printf("curr_mic_vol: %x\n", curr_mic_vol);
#endif
			// Change the mic volume according to the pushbuttons
			WriteAC97Reg(0x0E,curr_mic_vol);

			loop_count++;
			break;

			case 5:

			curr_song_vol = XGpio_DiscreteRead(&Gpio_songvol, 1 ); // & SONG;
#ifndef ONLY_PRINT_LYRICS
			printf("song volume before changing: %x\r\n", ReadAC97Reg(0x0E));
			printf("curr_song_vol before manipulation: %x\r\n", curr_song_vol);
#endif
//			curr_song_vol = curr_song_vol << 2;
//			curr_song_vol = curr_song_vol | 0x0040;  // turn 20db on
			switch (curr_song_vol) {
				case 7:
				// curr_song_vol = 0x0000;  // loudest
				curr_song_vol = 0x0000;
				break;
				case 6:
				// curr_song_vol = 0x1818;
				curr_song_vol = 0x0303;
				break;
				case 5:
				curr_song_vol = 0x0707;
				break;
				case 4:
				// curr_song_vol = 0x1818;
				curr_song_vol = 0x0A0A;
				break;
				case 3:
				// curr_song_vol = 0x1818;
				curr_song_vol = 0x1212;
				break;
				case 2:
				// curr_song_vol = 0x1818;
				curr_song_vol = 0x1A1A;
				break;
				case 1:
				curr_song_vol = 0x1F1F;  // softest
				// curr_song_vol = 0x001F;
				break;
				case 0:
				curr_song_vol = 0x8000;  // mute
				// curr_song_vol = 0x8008;
				break;
			}

#ifndef ONLY_PRINT_LYRICS
			printf("curr_song_vol: %x\n", curr_song_vol);
#endif
	
			// Change the song volume according to the pushbuttons
	   		WriteAC97Reg(0x18, curr_song_vol);
			loop_count++;
			break;
	
				case 10:
			// ----------------------------------------

			// Restart song if switch 1 is low
			while( (XGpio_DiscreteRead(&Gpio, 1 ) & SWITCH1) )  // wait until switch high
			{
				// printf("Restart song\n");
				first_time= 1;
				// XGpio_DiscreteWrite(&Gpio, 1, LED1); // write 1 to turn off (accidentally also turns LED0 off)
				XTmrCtr_Stop(&TimerCounter, TIMER_COUNTER_0); // stop timer so that the reset value can be changed
				restarted = 1;
			}
			// XGpio_DiscreteWrite(&Gpio, 1, ~LED1);  // write 0 to turn on (accidentally also turns LED0 on)
			if (restarted) {
				printf("\r\n\r\n");
				initializeLyrics();
				XTmrCtr_SetResetValue(&TimerCounter, 
					TIMER_COUNTER_0, NextLineTime * CLOCKS_PER_MS); // set timer reset value
				XTmrCtr_Start(&TimerCounter, TIMER_COUNTER_0); // start timer again
				restarted = 0;
			}

			// ----------------------------------------
			loop_count++;
			break;

			case 15:
			// ----------------------------------------
			// Pause if switch 0 is low
			while( (XGpio_DiscreteRead(&Gpio, 1 ) & SWITCH0) )  // wait until switch high
			{
				// printf("wait\r\n");
				//XGpio_DiscreteWrite(&Gpio, 1, LED0); // write 1 to turn off (accidentally also turns LED1 off)
				XTmrCtr_Stop(&TimerCounter, TIMER_COUNTER_0); // stop timer so that the reset value can be changed
				paused = 1;
			}
			//XGpio_DiscreteWrite(&Gpio, 1, ~LED0);  // write 0 to turn on (accidentally also turns LED1 on)
			if (paused)
			{
				XTmrCtr_Start(&TimerCounter, TIMER_COUNTER_0); // start timer again
				paused = 0;
			}
			// ----------------------------------------
	
			loop_count++;
			break;

			case 20:

			loop_count = 0;
			break;

			default:

			loop_count++;
			break;
		}
        
	} while(1);

	/* Mad is no longer used, the structures that were initialized must
     * now be cleared.
	 */
	mad_synth_finish(&Synth);
	mad_frame_finish(&Frame);
	mad_stream_finish(&Stream);

	/* If the output buffer is not empty and no error occured during
     * the last write, then flush it.
	 */
	/* Usha -- deleted
	if(OutputPtr!=OutputBuffer && Status!=2)
	{
		size_t	BufferSize=OutputPtr-OutputBuffer;

		if(fwrite(OutputBuffer,1,BufferSize,OutputFp)!=BufferSize)
		{
			fprintf(stderr,"%s: PCM write error (%s).\n",
					ProgName,strerror(errno));
			Status=2;
		}
	}
	*/

	/* Accounting report if no error occured. */
	if(!Status)
	{
	//	char	Buffer[80];

		/* The duration timer is converted to a human readable string
		 * with the versatile but still constrained mad_timer_string()
		 * function, in a fashion not unlike strftime(). The main
		 * difference is that the timer is break-down into several
		 * values according some of it's arguments. The units and
		 * fracunits arguments specify the intended conversion to be
		 * executed.
		 *
		 * The conversion unit (MAD_UNIT_MINUTES in our example) also
		 * specify the order and kind of conversion specifications
		 * that can be used in the format string.
		 *
		 * It is best to examine mad's timer.c source-code for details
		 * of the available units, fraction of units, their meanings,
		 * the format arguments, etc.
		 */
		//mad_timer_string(Timer,Buffer,"%lu:%02lu.%03u",
		//				 MAD_UNITS_MINUTES,MAD_UNITS_MILLISECONDS,0);

		/* Usha -- changed
		fprintf(stderr,"%s: %lu frames decoded (%s).\n",
				ProgName,FrameCount,Buffer);
		*/
		/* ******************
		printf("%s: %lu frames decoded (%s).\n",
				ProgName,FrameCount,Buffer);
				******** */
	}

#ifndef NO_PLAY
	microblaze_disable_interrupts();
#endif
#ifndef ONLY_PRINT_LYRICS
    print ("Done\n");
#endif

printf("\r\rDone\r\r");
STOP;
	
	//really_play_sound();
	/* That's the end of the world (in the H. G. Wells way). */
	//	return(Status);
}

#ifndef IGNORE_TIMED_LYRICS

/* use updateLyrics to print and update lyrics, set up timer for the next time */
void TimerCounterHandler(void *CallBackRef, Xuint8 TimerCounterNumber)
{

	if (no_more_lyrics)
		return;

	PreviousLineTime = NextLineTime; // PreviousLineTime should have roughly the current time in this function

/* Print the current line and any lines listed immediately after with times
 * at or before now; update the time for the next line; stop the timer if the end
 * of the file has been reached */

	do
	{

	printf(NextLine);	// Allows printing of multiple lines with the same time from within the function

	// Read the time value 
	count = 0;
	do
	{
		new_time[count] = *lyricsMemPointer;
		if (new_time[count] == 'X') {
//			XTmrCtr_SetOptions(&TimerCounter, TIMER_COUNTER_0,
//				~XTC_INT_MODE_OPTION & ~XTC_AUTO_RELOAD_OPTION);
//			XTmrCtr_Reset(&TimerCounter, TIMER_COUNTER_0);
//			XTmrCtr_Stop(&TimerCounter, TIMER_COUNTER_0);
//			XTmrCtr_mDisable(XPAR_OPB_TIMER_0_BASEADDR, TIMER_COUNTER_0);
			no_more_lyrics = 1;
			return;
		}
		//printf("%c", time[count]);
		lyricsMemPointer++;
		count++;
	} while (new_time[count-1] != '>');

	new_time[count-1] = '\0';  // terminate with null character
	NextLineTime = atoi(new_time);  // convert to integer so that can manipulate later

	// Read the line of text
	count = 0;
	do 
	{
		NextLine[count] = *lyricsMemPointer;
		lyricsMemPointer ++;
		//printf("%c", nextLine[count]);

		count++;
	} while (NextLine[count-1] != '<');

	NextLine[count-1] = '\0';  //terminate string with null character

#ifndef ONLY_PRINT_LYRICS
	// Display for verification
	printf("\nTime: %d: %s\n", NextLineTime, NextLine);  //This works
#endif

	} while (NextLineTime <= PreviousLineTime); // repeat if the line just fetched is not later

	/* Update timer */

	XTmrCtr_Stop(&TimerCounter, TIMER_COUNTER_0); // stop timer so that theh reset value can be changed
	XTmrCtr_SetResetValue(&TimerCounter, 
		TIMER_COUNTER_0, (NextLineTime - PreviousLineTime) * CLOCKS_PER_MS); // set timer reset value
	XTmrCtr_Start(&TimerCounter, TIMER_COUNTER_0); // start timer again
	return;

}

void initializeLyrics() {
	/* Lyrics variable setup */

	lyricsMemPointer = (char *) (0xfa800004 + FileSize);

	// Skip the first '<' character
	if (*lyricsMemPointer != '<') {
		printf("Invalid lyrics file\n");
		printf("First character should be \"<\", not \"%c\"\n",*lyricsMemPointer);
		printf("Attempted to read lyrics file at ");
		printf("%x\n", (unsigned int)lyricsMemPointer - 4);
		STOP;
	}
	lyricsMemPointer = lyricsMemPointer + 1;

	PreviousLineTime = 0;
	no_more_lyrics = 0;
	NextLine[0] = '\0';

/* Print any lines with time 0, and get and set up first line with time > 0 */

	do
	{

	printf(NextLine);	// Allows printing of multiple lines with the same time from within the function

	// Read the time value 
	count = 0;
	do
	{
		new_time[count] = *lyricsMemPointer;
		if (new_time[count] == 'X') {
			no_more_lyrics = 1;
			break;
		}
		//printf("%c", time[count]);
		lyricsMemPointer++;
		count++;
	} while (new_time[count-1] != '>');
	if (no_more_lyrics)
		break;

	new_time[count-1] = '\0';  // terminate with null character
	NextLineTime = atoi(new_time);  // convert to integer so that can manipulate later

	// Read the next line of text
	count = 0;
	do 
	{
		NextLine[count] = *lyricsMemPointer;
		lyricsMemPointer ++;
		//printf("%c", nextLine[count]);

		count++;
	} while (NextLine[count-1] != '<');

	NextLine[count-1] = '\0';  //terminate string with null character

#ifndef ONLY_PRINT_LYRICS
	// Display for verification
	printf("\nTime: %d: %s\n", NextLineTime, NextLine);  //This works
#endif

	} while (NextLineTime <= 0); // repeat if the line just fetched is not later

}


#endif


/****************************************************************************
 * Program entry point.														*
 ****************************************************************************/
/* Usha -- remove
int main(int argc, char *argv[])
{
	char	*cptr;
	int		Status;

*/	/* Keep this for error messages. */
/*	cptr=strrchr(argv[0],'/');
	if(cptr==NULL)
		ProgName=argv[0];
	else
		ProgName=cptr+1;

*/	/* Decode stdin to stdout. */
/*	Status=MpegAudioDecoder(stdin,stdout);
	if(Status)
		fprintf(stderr,"%s: an error occured during decoding.\n",ProgName);

*/	/* All done. */
/*	return(Status);
}
*/

/*
 * Local Variables:
 * tab-width: 4
 * End:
 */

/****************************************************************************
 * End of file madllc.c														*
 ****************************************************************************/
