-------------------------------------------------------------------------------
-- my_ac97_wrapper.vhd
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

library common_v1_00_a;
use common_v1_00_a.all;

library opb_ac97_controller_v1_00_a;
use opb_ac97_controller_v1_00_a.all;

entity my_ac97_wrapper is
  port (
    OPB_Clk : in std_logic;
    OPB_Rst : in std_logic;
    OPB_ABus : in std_logic_vector(0 to 31);
    OPB_BE : in std_logic_vector(0 to 3);
    OPB_RNW : in std_logic;
    OPB_select : in std_logic;
    OPB_seqAddr : in std_logic;
    OPB_DBus : in std_logic_vector(0 to 31);
    opb_ac97_controller_DBus : out std_logic_vector(0 to 31);
    opb_ac97_controller_errAck : out std_logic;
    opb_ac97_controller_retry : out std_logic;
    opb_ac97_controller_toutSup : out std_logic;
    opb_ac97_controller_xferAck : out std_logic;
    Playback_Interrupt : out std_logic;
    Record_Interrupt : out std_logic;
    Bit_Clk : in std_logic;
    Sync : out std_logic;
    SData_Out : out std_logic;
    SData_In : in std_logic;
    FSL_S_CLK : out std_logic;
    FSL_S_READ : out std_logic;
    FSL_S_DATA : in std_logic_vector(0 to 31);
    FSL_S_CONTROL : in std_logic;
    FSL_S_EXISTS : in std_logic
  );
end my_ac97_wrapper;

architecture STRUCTURE of my_ac97_wrapper is

  component opb_ac97_controller is
    generic (
      C_BASEADDR : STD_LOGIC_VECTOR;
      C_HIGHADDR : STD_LOGIC_VECTOR;
      C_OPB_DWIDTH : INTEGER;
      C_OPB_AWIDTH : INTEGER;
      C_PLAYBACK : INTEGER;
      C_RECORD : INTEGER;
      C_PLAY_INTR_LEVEL : INTEGER;
      C_REC_INTR_LEVEL : INTEGER;
      C_FSL_DWIDTH : INTEGER
    );
    port (
      OPB_Clk : in std_logic;
      OPB_Rst : in std_logic;
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_RNW : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      opb_ac97_controller_DBus : out std_logic_vector(0 to 31);
      opb_ac97_controller_errAck : out std_logic;
      opb_ac97_controller_retry : out std_logic;
      opb_ac97_controller_toutSup : out std_logic;
      opb_ac97_controller_xferAck : out std_logic;
      Playback_Interrupt : out std_logic;
      Record_Interrupt : out std_logic;
      Bit_Clk : in std_logic;
      Sync : out std_logic;
      SData_Out : out std_logic;
      SData_In : in std_logic;
      FSL_S_CLK : out std_logic;
      FSL_S_READ : out std_logic;
      FSL_S_DATA : in std_logic_vector(0 to C_FSL_DWIDTH-1);
      FSL_S_CONTROL : in std_logic;
      FSL_S_EXISTS : in std_logic
    );
  end component;

begin

  my_ac97 : opb_ac97_controller
    generic map (
      C_BASEADDR => X"ffffA200",
      C_HIGHADDR => X"ffffA2ff",
      C_OPB_DWIDTH => 32,
      C_OPB_AWIDTH => 32,
      C_PLAYBACK => 1,
      C_RECORD => 0,
      C_PLAY_INTR_LEVEL => 1,
      C_REC_INTR_LEVEL => 4,
      C_FSL_DWIDTH => 32
    )
    port map (
      OPB_Clk => OPB_Clk,
      OPB_Rst => OPB_Rst,
      OPB_ABus => OPB_ABus,
      OPB_BE => OPB_BE,
      OPB_RNW => OPB_RNW,
      OPB_select => OPB_select,
      OPB_seqAddr => OPB_seqAddr,
      OPB_DBus => OPB_DBus,
      opb_ac97_controller_DBus => opb_ac97_controller_DBus,
      opb_ac97_controller_errAck => opb_ac97_controller_errAck,
      opb_ac97_controller_retry => opb_ac97_controller_retry,
      opb_ac97_controller_toutSup => opb_ac97_controller_toutSup,
      opb_ac97_controller_xferAck => opb_ac97_controller_xferAck,
      Playback_Interrupt => Playback_Interrupt,
      Record_Interrupt => Record_Interrupt,
      Bit_Clk => Bit_Clk,
      Sync => Sync,
      SData_Out => SData_Out,
      SData_In => SData_In,
      FSL_S_CLK => FSL_S_CLK,
      FSL_S_READ => FSL_S_READ,
      FSL_S_DATA => FSL_S_DATA,
      FSL_S_CONTROL => FSL_S_CONTROL,
      FSL_S_EXISTS => FSL_S_EXISTS
    );

end architecture STRUCTURE;
