-------------------------------------------------------------------------------
-- system.vhd
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity system is
  port (
    clk_50 : in std_logic;
    extend_dcm_reset : in std_logic;
    ac97_bit_clock_p : in std_logic;
    ac97_synch_p : out std_logic;
    ac97_sdata_in : in std_logic;
    ac97_sdata_out : out std_logic;
    zbt1_oe_n : out std_logic;
    zbt1_adv_ld_n : out std_logic;
    zbt1_ce1_n : out std_logic;
    zbt1_rw_n : out std_logic;
    zbt1_cke_n : out std_logic;
    zbt1_a : out std_logic_vector(0 to 18);
    zbt1_bw_n : out std_logic_vector(1 to 4);
    zbt1_io : inout std_logic_vector(0 to 31);
    zbt2_oe_n : out std_logic;
    zbt2_adv_ld_n : out std_logic;
    zbt2_ce1_n : out std_logic;
    zbt2_rw_n : out std_logic;
    zbt2_cke_n : out std_logic;
    zbt2_a : out std_logic_vector(0 to 18);
    zbt2_bw_n : out std_logic_vector(1 to 4);
    zbt2_io : inout std_logic_vector(0 to 31);
    zbt1_clk : out std_logic;
    zbt2_clk : out std_logic;
    zbt_clk_fbout : out std_logic;
    zbt_clk_fb : in std_logic;
    startup : out std_logic;
    RS232_RX : in std_logic;
    RS232_TX : out std_logic;
    RS232_req_to_send_pin : out std_logic;
    opb_gpio_0_GPIO_IO : inout std_logic_vector(0 to 3);
    volume_buttons_0_PB_CLOCK_P : in std_logic;
    volume_buttons_0_PB_DATA_P : in std_logic
  );
end system;

architecture STRUCTURE of system is

  attribute box_type : STRING;

  component opb_gpio_0_wrapper is
    port (
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_Clk : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      OPB_RNW : in std_logic;
      OPB_Rst : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      Sln_DBus : out std_logic_vector(0 to 31);
      Sln_errAck : out std_logic;
      Sln_retry : out std_logic;
      Sln_toutSup : out std_logic;
      Sln_xferAck : out std_logic;
      IP2INTC_Irpt : out std_logic;
      GPIO_in : in std_logic_vector(0 to 3);
      GPIO_d_out : out std_logic_vector(0 to 3);
      GPIO_t_out : out std_logic_vector(0 to 3);
      GPIO2_in : in std_logic_vector(0 to 3);
      GPIO2_d_out : out std_logic_vector(0 to 3);
      GPIO2_t_out : out std_logic_vector(0 to 3);
      GPIO_IO_I : in std_logic_vector(0 to 3);
      GPIO_IO_O : out std_logic_vector(0 to 3);
      GPIO_IO_T : out std_logic_vector(0 to 3);
      GPIO2_IO_I : in std_logic_vector(0 to 3);
      GPIO2_IO_O : out std_logic_vector(0 to 3);
      GPIO2_IO_T : out std_logic_vector(0 to 3)
    );
  end component;

  attribute box_type of opb_gpio_0_wrapper: component is "black_box";

  component opb_gpio_1_wrapper is
    port (
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_Clk : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      OPB_RNW : in std_logic;
      OPB_Rst : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      Sln_DBus : out std_logic_vector(0 to 31);
      Sln_errAck : out std_logic;
      Sln_retry : out std_logic;
      Sln_toutSup : out std_logic;
      Sln_xferAck : out std_logic;
      IP2INTC_Irpt : out std_logic;
      GPIO_in : in std_logic_vector(0 to 2);
      GPIO_d_out : out std_logic_vector(0 to 2);
      GPIO_t_out : out std_logic_vector(0 to 2);
      GPIO2_in : in std_logic_vector(0 to 2);
      GPIO2_d_out : out std_logic_vector(0 to 2);
      GPIO2_t_out : out std_logic_vector(0 to 2);
      GPIO_IO_I : in std_logic_vector(0 to 2);
      GPIO_IO_O : out std_logic_vector(0 to 2);
      GPIO_IO_T : out std_logic_vector(0 to 2);
      GPIO2_IO_I : in std_logic_vector(0 to 2);
      GPIO2_IO_O : out std_logic_vector(0 to 2);
      GPIO2_IO_T : out std_logic_vector(0 to 2)
    );
  end component;

  attribute box_type of opb_gpio_1_wrapper: component is "black_box";

  component opb_gpio_2_wrapper is
    port (
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_Clk : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      OPB_RNW : in std_logic;
      OPB_Rst : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      Sln_DBus : out std_logic_vector(0 to 31);
      Sln_errAck : out std_logic;
      Sln_retry : out std_logic;
      Sln_toutSup : out std_logic;
      Sln_xferAck : out std_logic;
      IP2INTC_Irpt : out std_logic;
      GPIO_in : in std_logic_vector(0 to 2);
      GPIO_d_out : out std_logic_vector(0 to 2);
      GPIO_t_out : out std_logic_vector(0 to 2);
      GPIO2_in : in std_logic_vector(0 to 2);
      GPIO2_d_out : out std_logic_vector(0 to 2);
      GPIO2_t_out : out std_logic_vector(0 to 2);
      GPIO_IO_I : in std_logic_vector(0 to 2);
      GPIO_IO_O : out std_logic_vector(0 to 2);
      GPIO_IO_T : out std_logic_vector(0 to 2);
      GPIO2_IO_I : in std_logic_vector(0 to 2);
      GPIO2_IO_O : out std_logic_vector(0 to 2);
      GPIO2_IO_T : out std_logic_vector(0 to 2)
    );
  end component;

  attribute box_type of opb_gpio_2_wrapper: component is "black_box";

  component volume_buttons_0_wrapper is
    port (
      clock : in std_logic;
      reset_b : in std_logic;
      PB_CLOCK_P : in std_logic;
      PB_DATA_P : in std_logic;
      mic_volume : out std_logic_vector(2 downto 0);
      song_volume : out std_logic_vector(2 downto 0);
      startup : out std_logic
    );
  end component;

  attribute box_type of volume_buttons_0_wrapper: component is "black_box";

  component clk_50_bufg_wrapper is
    port (
      I : in std_logic;
      O : out std_logic
    );
  end component;

  attribute box_type of clk_50_bufg_wrapper: component is "black_box";

  component main_dcm_wrapper is
    port (
      CLKIN : in std_logic;
      CLKFB : in std_logic;
      DSSEN : in std_logic;
      PSINCDEC : in std_logic;
      PSEN : in std_logic;
      PSCLK : in std_logic;
      RST : in std_logic;
      CLK0 : out std_logic;
      CLK90 : out std_logic;
      CLK180 : out std_logic;
      CLK270 : out std_logic;
      CLK2X : out std_logic;
      CLK2X180 : out std_logic;
      CLKDV : out std_logic;
      CLKFX : out std_logic;
      CLKFX180 : out std_logic;
      LOCKED : out std_logic;
      PSDONE : out std_logic;
      STATUS : out std_logic_vector(7 downto 0);
      RESET_IN : in std_logic;
      FPGA_DONE : in std_logic;
      RESET_OUT : out std_logic
    );
  end component;

  attribute box_type of main_dcm_wrapper: component is "black_box";

  component sys_clk_bufg_wrapper is
    port (
      I : in std_logic;
      O : out std_logic
    );
  end component;

  attribute box_type of sys_clk_bufg_wrapper: component is "black_box";

  component zbt_dcm_wrapper is
    port (
      CLKIN : in std_logic;
      CLKFB : in std_logic;
      DSSEN : in std_logic;
      PSINCDEC : in std_logic;
      PSEN : in std_logic;
      PSCLK : in std_logic;
      RST : in std_logic;
      CLK0 : out std_logic;
      CLK90 : out std_logic;
      CLK180 : out std_logic;
      CLK270 : out std_logic;
      CLK2X : out std_logic;
      CLK2X180 : out std_logic;
      CLKDV : out std_logic;
      CLKFX : out std_logic;
      CLKFX180 : out std_logic;
      LOCKED : out std_logic;
      PSDONE : out std_logic;
      STATUS : out std_logic_vector(7 downto 0);
      RESET_IN : in std_logic;
      FPGA_DONE : in std_logic;
      RESET_OUT : out std_logic
    );
  end component;

  attribute box_type of zbt_dcm_wrapper: component is "black_box";

  component myopb_bus_wrapper is
    port (
      OPB_Clk : in std_logic;
      OPB_Rst : out std_logic;
      SYS_Rst : in std_logic;
      Debug_SYS_Rst : in std_logic;
      WDT_Rst : in std_logic;
      M_ABus : in std_logic_vector(0 to 63);
      M_BE : in std_logic_vector(0 to 7);
      M_beXfer : in std_logic_vector(0 to 1);
      M_busLock : in std_logic_vector(0 to 1);
      M_DBus : in std_logic_vector(0 to 63);
      M_DBusEn : in std_logic_vector(0 to 1);
      M_DBusEn32_63 : in std_logic_vector(0 to 1);
      M_dwXfer : in std_logic_vector(0 to 1);
      M_fwXfer : in std_logic_vector(0 to 1);
      M_hwXfer : in std_logic_vector(0 to 1);
      M_request : in std_logic_vector(0 to 1);
      M_RNW : in std_logic_vector(0 to 1);
      M_select : in std_logic_vector(0 to 1);
      M_seqAddr : in std_logic_vector(0 to 1);
      Sl_beAck : in std_logic_vector(0 to 9);
      Sl_DBus : in std_logic_vector(0 to 319);
      Sl_DBusEn : in std_logic_vector(0 to 9);
      Sl_DBusEn32_63 : in std_logic_vector(0 to 9);
      Sl_errAck : in std_logic_vector(0 to 9);
      Sl_dwAck : in std_logic_vector(0 to 9);
      Sl_fwAck : in std_logic_vector(0 to 9);
      Sl_hwAck : in std_logic_vector(0 to 9);
      Sl_retry : in std_logic_vector(0 to 9);
      Sl_toutSup : in std_logic_vector(0 to 9);
      Sl_xferAck : in std_logic_vector(0 to 9);
      OPB_MRequest : out std_logic_vector(0 to 1);
      OPB_ABus : out std_logic_vector(0 to 31);
      OPB_BE : out std_logic_vector(0 to 3);
      OPB_beXfer : out std_logic;
      OPB_beAck : out std_logic;
      OPB_busLock : out std_logic;
      OPB_rdDBus : out std_logic_vector(0 to 31);
      OPB_wrDBus : out std_logic_vector(0 to 31);
      OPB_DBus : out std_logic_vector(0 to 31);
      OPB_errAck : out std_logic;
      OPB_dwAck : out std_logic;
      OPB_dwXfer : out std_logic;
      OPB_fwAck : out std_logic;
      OPB_fwXfer : out std_logic;
      OPB_hwAck : out std_logic;
      OPB_hwXfer : out std_logic;
      OPB_MGrant : out std_logic_vector(0 to 1);
      OPB_pendReq : out std_logic_vector(0 to 1);
      OPB_retry : out std_logic;
      OPB_RNW : out std_logic;
      OPB_select : out std_logic;
      OPB_seqAddr : out std_logic;
      OPB_timeout : out std_logic;
      OPB_toutSup : out std_logic;
      OPB_xferAck : out std_logic
    );
  end component;

  attribute box_type of myopb_bus_wrapper: component is "black_box";

  component my_blaze_wrapper is
    port (
      CLK : in std_logic;
      RESET : in std_logic;
      INTERRUPT : in std_logic;
      DEBUG_RST : in std_logic;
      EXT_BRK : in std_logic;
      EXT_NM_BRK : in std_logic;
      DBG_STOP : in std_logic;
      INSTR : in std_logic_vector(0 to 31);
      IREADY : in std_logic;
      INSTR_ADDR : out std_logic_vector(0 to 31);
      IFETCH : out std_logic;
      I_AS : out std_logic;
      DATA_READ : in std_logic_vector(0 to 31);
      DREADY : in std_logic;
      DATA_WRITE : out std_logic_vector(0 to 31);
      DATA_ADDR : out std_logic_vector(0 to 31);
      D_AS : out std_logic;
      READ_STROBE : out std_logic;
      WRITE_STROBE : out std_logic;
      BYTE_ENABLE : out std_logic_vector(0 to 3);
      DM_ABUS : out std_logic_vector(0 to 31);
      DM_BE : out std_logic_vector(0 to 3);
      DM_BUSLOCK : out std_logic;
      DM_DBUS : out std_logic_vector(0 to 31);
      DM_REQUEST : out std_logic;
      DM_RNW : out std_logic;
      DM_SELECT : out std_logic;
      DM_SEQADDR : out std_logic;
      DOPB_DBUS : in std_logic_vector(0 to 31);
      DOPB_ERRACK : in std_logic;
      DOPB_MGRANT : in std_logic;
      DOPB_RETRY : in std_logic;
      DOPB_TIMEOUT : in std_logic;
      DOPB_XFERACK : in std_logic;
      IM_ABUS : out std_logic_vector(0 to 31);
      IM_BE : out std_logic_vector(0 to 3);
      IM_BUSLOCK : out std_logic;
      IM_DBUS : out std_logic_vector(0 to 31);
      IM_REQUEST : out std_logic;
      IM_RNW : out std_logic;
      IM_SELECT : out std_logic;
      IM_SEQADDR : out std_logic;
      IOPB_DBUS : in std_logic_vector(0 to 31);
      IOPB_ERRACK : in std_logic;
      IOPB_MGRANT : in std_logic;
      IOPB_RETRY : in std_logic;
      IOPB_TIMEOUT : in std_logic;
      IOPB_XFERACK : in std_logic;
      DBG_CLK : in std_logic;
      DBG_TDI : in std_logic;
      DBG_TDO : out std_logic;
      DBG_REG_EN : in std_logic_vector(0 to 4);
      DBG_CAPTURE : in std_logic;
      DBG_UPDATE : in std_logic;
      VALID_INSTR : out std_logic;
      PC_EX : out std_logic_vector(0 to 31);
      REG_WRITE : out std_logic;
      REG_ADDR : out std_logic_vector(0 to 4);
      MSR_REG : out std_logic_vector(0 to 7);
      NEW_REG_VALUE : out std_logic_vector(0 to 31);
      PIPE_RUNNING : out std_logic;
      INTERRUPT_TAKEN : out std_logic;
      JUMP_TAKEN : out std_logic;
      PREFETCH_ADDR : out std_logic_vector(0 to 3);
      MB_Halted : out std_logic;
      FSL0_S_CLK : out std_logic;
      FSL0_S_READ : out std_logic;
      FSL0_S_DATA : in std_logic_vector(0 to 31);
      FSL0_S_CONTROL : in std_logic;
      FSL0_S_EXISTS : in std_logic;
      FSL0_M_CLK : out std_logic;
      FSL0_M_WRITE : out std_logic;
      FSL0_M_DATA : out std_logic_vector(0 to 31);
      FSL0_M_CONTROL : out std_logic;
      FSL0_M_FULL : in std_logic;
      FSL1_S_CLK : out std_logic;
      FSL1_S_READ : out std_logic;
      FSL1_S_DATA : in std_logic_vector(0 to 31);
      FSL1_S_CONTROL : in std_logic;
      FSL1_S_EXISTS : in std_logic;
      FSL1_M_CLK : out std_logic;
      FSL1_M_WRITE : out std_logic;
      FSL1_M_DATA : out std_logic_vector(0 to 31);
      FSL1_M_CONTROL : out std_logic;
      FSL1_M_FULL : in std_logic;
      FSL2_S_CLK : out std_logic;
      FSL2_S_READ : out std_logic;
      FSL2_S_DATA : in std_logic_vector(0 to 31);
      FSL2_S_CONTROL : in std_logic;
      FSL2_S_EXISTS : in std_logic;
      FSL2_M_CLK : out std_logic;
      FSL2_M_WRITE : out std_logic;
      FSL2_M_DATA : out std_logic_vector(0 to 31);
      FSL2_M_CONTROL : out std_logic;
      FSL2_M_FULL : in std_logic;
      FSL3_S_CLK : out std_logic;
      FSL3_S_READ : out std_logic;
      FSL3_S_DATA : in std_logic_vector(0 to 31);
      FSL3_S_CONTROL : in std_logic;
      FSL3_S_EXISTS : in std_logic;
      FSL3_M_CLK : out std_logic;
      FSL3_M_WRITE : out std_logic;
      FSL3_M_DATA : out std_logic_vector(0 to 31);
      FSL3_M_CONTROL : out std_logic;
      FSL3_M_FULL : in std_logic;
      FSL4_S_CLK : out std_logic;
      FSL4_S_READ : out std_logic;
      FSL4_S_DATA : in std_logic_vector(0 to 31);
      FSL4_S_CONTROL : in std_logic;
      FSL4_S_EXISTS : in std_logic;
      FSL4_M_CLK : out std_logic;
      FSL4_M_WRITE : out std_logic;
      FSL4_M_DATA : out std_logic_vector(0 to 31);
      FSL4_M_CONTROL : out std_logic;
      FSL4_M_FULL : in std_logic;
      FSL5_S_CLK : out std_logic;
      FSL5_S_READ : out std_logic;
      FSL5_S_DATA : in std_logic_vector(0 to 31);
      FSL5_S_CONTROL : in std_logic;
      FSL5_S_EXISTS : in std_logic;
      FSL5_M_CLK : out std_logic;
      FSL5_M_WRITE : out std_logic;
      FSL5_M_DATA : out std_logic_vector(0 to 31);
      FSL5_M_CONTROL : out std_logic;
      FSL5_M_FULL : in std_logic;
      FSL6_S_CLK : out std_logic;
      FSL6_S_READ : out std_logic;
      FSL6_S_DATA : in std_logic_vector(0 to 31);
      FSL6_S_CONTROL : in std_logic;
      FSL6_S_EXISTS : in std_logic;
      FSL6_M_CLK : out std_logic;
      FSL6_M_WRITE : out std_logic;
      FSL6_M_DATA : out std_logic_vector(0 to 31);
      FSL6_M_CONTROL : out std_logic;
      FSL6_M_FULL : in std_logic;
      FSL7_S_CLK : out std_logic;
      FSL7_S_READ : out std_logic;
      FSL7_S_DATA : in std_logic_vector(0 to 31);
      FSL7_S_CONTROL : in std_logic;
      FSL7_S_EXISTS : in std_logic;
      FSL7_M_CLK : out std_logic;
      FSL7_M_WRITE : out std_logic;
      FSL7_M_DATA : out std_logic_vector(0 to 31);
      FSL7_M_CONTROL : out std_logic;
      FSL7_M_FULL : in std_logic
    );
  end component;

  attribute box_type of my_blaze_wrapper: component is "black_box";

  component my_ac97_wrapper is
    port (
      OPB_Clk : in std_logic;
      OPB_Rst : in std_logic;
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_RNW : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      opb_ac97_controller_DBus : out std_logic_vector(0 to 31);
      opb_ac97_controller_errAck : out std_logic;
      opb_ac97_controller_retry : out std_logic;
      opb_ac97_controller_toutSup : out std_logic;
      opb_ac97_controller_xferAck : out std_logic;
      Playback_Interrupt : out std_logic;
      Record_Interrupt : out std_logic;
      Bit_Clk : in std_logic;
      Sync : out std_logic;
      SData_Out : out std_logic;
      SData_In : in std_logic;
      FSL_S_CLK : out std_logic;
      FSL_S_READ : out std_logic;
      FSL_S_DATA : in std_logic_vector(0 to 31);
      FSL_S_CONTROL : in std_logic;
      FSL_S_EXISTS : in std_logic
    );
  end component;

  attribute box_type of my_ac97_wrapper: component is "black_box";

  component my_mdm_wrapper is
    port (
      OPB_Clk : in std_logic;
      OPB_Rst : in std_logic;
      Interrupt : out std_logic;
      Debug_SYS_Rst : out std_logic;
      Debug_Rst : out std_logic;
      Ext_BRK : out std_logic;
      Ext_NM_BRK : out std_logic;
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_RNW : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      MDM_DBus : out std_logic_vector(0 to 31);
      MDM_errAck : out std_logic;
      MDM_retry : out std_logic;
      MDM_toutSup : out std_logic;
      MDM_xferAck : out std_logic;
      Dbg_Clk_0 : out std_logic;
      Dbg_TDI_0 : out std_logic;
      Dbg_TDO_0 : in std_logic;
      Dbg_Reg_En_0 : out std_logic_vector(0 to 4);
      Dbg_Capture_0 : out std_logic;
      Dbg_Update_0 : out std_logic;
      Dbg_Clk_1 : out std_logic;
      Dbg_TDI_1 : out std_logic;
      Dbg_TDO_1 : in std_logic;
      Dbg_Reg_En_1 : out std_logic_vector(0 to 4);
      Dbg_Capture_1 : out std_logic;
      Dbg_Update_1 : out std_logic;
      Dbg_Clk_2 : out std_logic;
      Dbg_TDI_2 : out std_logic;
      Dbg_TDO_2 : in std_logic;
      Dbg_Reg_En_2 : out std_logic_vector(0 to 4);
      Dbg_Capture_2 : out std_logic;
      Dbg_Update_2 : out std_logic;
      Dbg_Clk_3 : out std_logic;
      Dbg_TDI_3 : out std_logic;
      Dbg_TDO_3 : in std_logic;
      Dbg_Reg_En_3 : out std_logic_vector(0 to 4);
      Dbg_Capture_3 : out std_logic;
      Dbg_Update_3 : out std_logic;
      Dbg_Clk_4 : out std_logic;
      Dbg_TDI_4 : out std_logic;
      Dbg_TDO_4 : in std_logic;
      Dbg_Reg_En_4 : out std_logic_vector(0 to 4);
      Dbg_Capture_4 : out std_logic;
      Dbg_Update_4 : out std_logic;
      Dbg_Clk_5 : out std_logic;
      Dbg_TDI_5 : out std_logic;
      Dbg_TDO_5 : in std_logic;
      Dbg_Reg_En_5 : out std_logic_vector(0 to 4);
      Dbg_Capture_5 : out std_logic;
      Dbg_Update_5 : out std_logic;
      Dbg_Clk_6 : out std_logic;
      Dbg_TDI_6 : out std_logic;
      Dbg_TDO_6 : in std_logic;
      Dbg_Reg_En_6 : out std_logic_vector(0 to 4);
      Dbg_Capture_6 : out std_logic;
      Dbg_Update_6 : out std_logic;
      Dbg_Clk_7 : out std_logic;
      Dbg_TDI_7 : out std_logic;
      Dbg_TDO_7 : in std_logic;
      Dbg_Reg_En_7 : out std_logic_vector(0 to 4);
      Dbg_Capture_7 : out std_logic;
      Dbg_Update_7 : out std_logic;
      bscan_tdi : out std_logic;
      bscan_reset : out std_logic;
      bscan_shift : out std_logic;
      bscan_update : out std_logic;
      bscan_capture : out std_logic;
      bscan_sel1 : out std_logic;
      bscan_drck1 : out std_logic;
      bscan_tdo1 : in std_logic;
      FSL0_S_CLK : out std_logic;
      FSL0_S_READ : out std_logic;
      FSL0_S_DATA : in std_logic_vector(0 to 31);
      FSL0_S_CONTROL : in std_logic;
      FSL0_S_EXISTS : in std_logic;
      FSL0_M_CLK : out std_logic;
      FSL0_M_WRITE : out std_logic;
      FSL0_M_DATA : out std_logic_vector(0 to 31);
      FSL0_M_CONTROL : out std_logic;
      FSL0_M_FULL : in std_logic;
      ICON_Control : out std_logic_vector(35 downto 0);
      jtag_clk : out std_logic;
      trig : out std_logic_vector(7 downto 0);
      data : out std_logic_vector(31 downto 0)
    );
  end component;

  attribute box_type of my_mdm_wrapper: component is "black_box";

  component download_link_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of download_link_wrapper: component is "black_box";

  component my_zbt_wrapper is
    port (
      OPB_Clk : in std_logic;
      OPB_Rst : in std_logic;
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_RNW : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      ZBT_DBus : out std_logic_vector(0 to 31);
      ZBT_errAck : out std_logic;
      ZBT_retry : out std_logic;
      ZBT_toutSup : out std_logic;
      ZBT_xferAck : out std_logic;
      ZBT_Clk_FB : in std_logic;
      ZBT_Clk_FBOut : out std_logic;
      ZBT_Clk : out std_logic;
      ZBT_OE_N : out std_logic;
      ZBT_ADV_LD_N : out std_logic;
      ZBT_LBO_N : out std_logic;
      ZBT_CE1_N : out std_logic;
      ZBT_CE2_N : out std_logic;
      ZBT_CE2 : out std_logic;
      ZBT_RW_N : out std_logic;
      ZBT_CKE_N : out std_logic;
      ZBT_A : out std_logic_vector(0 to 18);
      ZBT_BW_N : out std_logic_vector(1 to 4);
      ZBT_IO_I : in std_logic_vector(0 to 31);
      ZBT_IO_O : out std_logic_vector(0 to 31);
      ZBT_IO_T : out std_logic;
      ZBT_IOP_I : in std_logic_vector(1 to 4);
      ZBT_IOP_O : out std_logic_vector(1 to 4);
      ZBT_IOP_T : out std_logic
    );
  end component;

  attribute box_type of my_zbt_wrapper: component is "black_box";

  component my_zbt_2_wrapper is
    port (
      OPB_Clk : in std_logic;
      OPB_Rst : in std_logic;
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_RNW : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      ZBT_DBus : out std_logic_vector(0 to 31);
      ZBT_errAck : out std_logic;
      ZBT_retry : out std_logic;
      ZBT_toutSup : out std_logic;
      ZBT_xferAck : out std_logic;
      ZBT_Clk_FB : in std_logic;
      ZBT_Clk_FBOut : out std_logic;
      ZBT_Clk : out std_logic;
      ZBT_OE_N : out std_logic;
      ZBT_ADV_LD_N : out std_logic;
      ZBT_LBO_N : out std_logic;
      ZBT_CE1_N : out std_logic;
      ZBT_CE2_N : out std_logic;
      ZBT_CE2 : out std_logic;
      ZBT_RW_N : out std_logic;
      ZBT_CKE_N : out std_logic;
      ZBT_A : out std_logic_vector(0 to 18);
      ZBT_BW_N : out std_logic_vector(1 to 4);
      ZBT_IO_I : in std_logic_vector(0 to 31);
      ZBT_IO_O : out std_logic_vector(0 to 31);
      ZBT_IO_T : out std_logic;
      ZBT_IOP_I : in std_logic_vector(1 to 4);
      ZBT_IOP_O : out std_logic_vector(1 to 4);
      ZBT_IOP_T : out std_logic
    );
  end component;

  attribute box_type of my_zbt_2_wrapper: component is "black_box";

  component islmb_wrapper is
    port (
      LMB_Clk : in std_logic;
      LMB_Rst : in std_logic;
      LMB_ABus : in std_logic_vector(0 to 31);
      LMB_WriteDBus : in std_logic_vector(0 to 31);
      LMB_AddrStrobe : in std_logic;
      LMB_ReadStrobe : in std_logic;
      LMB_WriteStrobe : in std_logic;
      LMB_BE : in std_logic_vector(0 to 3);
      Sl_DBus : out std_logic_vector(0 to 31);
      Sl_Ready : out std_logic;
      BRAM_Rst_A : out std_logic;
      BRAM_Clk_A : out std_logic;
      BRAM_EN_A : out std_logic;
      BRAM_WEN_A : out std_logic_vector(0 to 3);
      BRAM_Addr_A : out std_logic_vector(0 to 31);
      BRAM_Din_A : in std_logic_vector(0 to 31);
      BRAM_Dout_A : out std_logic_vector(0 to 31)
    );
  end component;

  attribute box_type of islmb_wrapper: component is "black_box";

  component dslmb_wrapper is
    port (
      LMB_Clk : in std_logic;
      LMB_Rst : in std_logic;
      LMB_ABus : in std_logic_vector(0 to 31);
      LMB_WriteDBus : in std_logic_vector(0 to 31);
      LMB_AddrStrobe : in std_logic;
      LMB_ReadStrobe : in std_logic;
      LMB_WriteStrobe : in std_logic;
      LMB_BE : in std_logic_vector(0 to 3);
      Sl_DBus : out std_logic_vector(0 to 31);
      Sl_Ready : out std_logic;
      BRAM_Rst_A : out std_logic;
      BRAM_Clk_A : out std_logic;
      BRAM_EN_A : out std_logic;
      BRAM_WEN_A : out std_logic_vector(0 to 3);
      BRAM_Addr_A : out std_logic_vector(0 to 31);
      BRAM_Din_A : in std_logic_vector(0 to 31);
      BRAM_Dout_A : out std_logic_vector(0 to 31)
    );
  end component;

  attribute box_type of dslmb_wrapper: component is "black_box";

  component d_lmb_wrapper is
    port (
      LMB_Clk : in std_logic;
      SYS_Rst : in std_logic;
      LMB_Rst : out std_logic;
      M_ABus : in std_logic_vector(0 to 31);
      M_ReadStrobe : in std_logic;
      M_WriteStrobe : in std_logic;
      M_AddrStrobe : in std_logic;
      M_DBus : in std_logic_vector(0 to 31);
      M_BE : in std_logic_vector(0 to 3);
      Sl_DBus : in std_logic_vector(0 to 31);
      Sl_Ready : in std_logic_vector(0 to 0);
      LMB_ABus : out std_logic_vector(0 to 31);
      LMB_ReadStrobe : out std_logic;
      LMB_WriteStrobe : out std_logic;
      LMB_AddrStrobe : out std_logic;
      LMB_ReadDBus : out std_logic_vector(0 to 31);
      LMB_WriteDBus : out std_logic_vector(0 to 31);
      LMB_Ready : out std_logic;
      LMB_BE : out std_logic_vector(0 to 3)
    );
  end component;

  attribute box_type of d_lmb_wrapper: component is "black_box";

  component i_lmb_wrapper is
    port (
      LMB_Clk : in std_logic;
      SYS_Rst : in std_logic;
      LMB_Rst : out std_logic;
      M_ABus : in std_logic_vector(0 to 31);
      M_ReadStrobe : in std_logic;
      M_WriteStrobe : in std_logic;
      M_AddrStrobe : in std_logic;
      M_DBus : in std_logic_vector(0 to 31);
      M_BE : in std_logic_vector(0 to 3);
      Sl_DBus : in std_logic_vector(0 to 31);
      Sl_Ready : in std_logic_vector(0 to 0);
      LMB_ABus : out std_logic_vector(0 to 31);
      LMB_ReadStrobe : out std_logic;
      LMB_WriteStrobe : out std_logic;
      LMB_AddrStrobe : out std_logic;
      LMB_ReadDBus : out std_logic_vector(0 to 31);
      LMB_WriteDBus : out std_logic_vector(0 to 31);
      LMB_Ready : out std_logic;
      LMB_BE : out std_logic_vector(0 to 3)
    );
  end component;

  attribute box_type of i_lmb_wrapper: component is "black_box";

  component lmblmb_bram_i_wrapper is
    port (
      BRAM_Rst_A : in std_logic;
      BRAM_Clk_A : in std_logic;
      BRAM_EN_A : in std_logic;
      BRAM_WEN_A : in std_logic_vector(0 to 3);
      BRAM_Addr_A : in std_logic_vector(0 to 31);
      BRAM_Din_A : out std_logic_vector(0 to 31);
      BRAM_Dout_A : in std_logic_vector(0 to 31);
      BRAM_Rst_B : in std_logic;
      BRAM_Clk_B : in std_logic;
      BRAM_EN_B : in std_logic;
      BRAM_WEN_B : in std_logic_vector(0 to 3);
      BRAM_Addr_B : in std_logic_vector(0 to 31);
      BRAM_Din_B : out std_logic_vector(0 to 31);
      BRAM_Dout_B : in std_logic_vector(0 to 31)
    );
  end component;

  attribute box_type of lmblmb_bram_i_wrapper: component is "black_box";

  component from_mb_fsl_1_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of from_mb_fsl_1_wrapper: component is "black_box";

  component to_mb_fsl_1_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of to_mb_fsl_1_wrapper: component is "black_box";

  component my_dct32_wrapper is
    port (
      Clk : in std_logic;
      Reset : in std_logic;
      FSL_S_CLK : out std_logic;
      FSL_S_READ : out std_logic;
      FSL_S_DATA : in std_logic_vector(0 to 31);
      FSL_S_CONTROL : in std_logic;
      FSL_S_EXISTS : in std_logic;
      FSL_M_CLK : out std_logic;
      FSL_M_WRITE : out std_logic;
      FSL_M_DATA : out std_logic_vector(0 to 31);
      FSL_M_CONTROL : out std_logic;
      FSL_M_FULL : in std_logic
    );
  end component;

  attribute box_type of my_dct32_wrapper: component is "black_box";

  component from_mb_fsl_2_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of from_mb_fsl_2_wrapper: component is "black_box";

  component to_mb_fsl_2_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of to_mb_fsl_2_wrapper: component is "black_box";

  component my_imdct_wrapper is
    port (
      Clk : in std_logic;
      Reset : in std_logic;
      FSL_S_CLK : out std_logic;
      FSL_S_READ : out std_logic;
      FSL_S_DATA : in std_logic_vector(0 to 31);
      FSL_S_CONTROL : in std_logic;
      FSL_S_EXISTS : in std_logic;
      FSL_M_CLK : out std_logic;
      FSL_M_WRITE : out std_logic;
      FSL_M_DATA : out std_logic_vector(0 to 31);
      FSL_M_CONTROL : out std_logic;
      FSL_M_FULL : in std_logic
    );
  end component;

  attribute box_type of my_imdct_wrapper: component is "black_box";

  component from_mb_fsl_3_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of from_mb_fsl_3_wrapper: component is "black_box";

  component to_mb_fsl_3_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of to_mb_fsl_3_wrapper: component is "black_box";

  component my_mp3_mul_28_wrapper is
    port (
      Clk : in std_logic;
      Reset : in std_logic;
      FSL_S_CLK : out std_logic;
      FSL_S_READ : out std_logic;
      FSL_S_DATA : in std_logic_vector(0 to 31);
      FSL_S_CONTROL : in std_logic;
      FSL_S_EXISTS : in std_logic;
      FSL_M_CLK : out std_logic;
      FSL_M_WRITE : out std_logic;
      FSL_M_DATA : out std_logic_vector(0 to 31);
      FSL_M_CONTROL : out std_logic;
      FSL_M_FULL : in std_logic
    );
  end component;

  attribute box_type of my_mp3_mul_28_wrapper: component is "black_box";

  component from_mb_fsl_4_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of from_mb_fsl_4_wrapper: component is "black_box";

  component to_mb_fsl_4_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of to_mb_fsl_4_wrapper: component is "black_box";

  component my_mp3_mul_16_wrapper is
    port (
      Clk : in std_logic;
      Reset : in std_logic;
      FSL_S_CLK : out std_logic;
      FSL_S_READ : out std_logic;
      FSL_S_DATA : in std_logic_vector(0 to 31);
      FSL_S_CONTROL : in std_logic;
      FSL_S_EXISTS : in std_logic;
      FSL_M_CLK : out std_logic;
      FSL_M_WRITE : out std_logic;
      FSL_M_DATA : out std_logic_vector(0 to 31);
      FSL_M_CONTROL : out std_logic;
      FSL_M_FULL : in std_logic
    );
  end component;

  attribute box_type of my_mp3_mul_16_wrapper: component is "black_box";

  component ac_97_play_buffer_wrapper is
    port (
      FSL_Clk : in std_logic;
      SYS_Rst : in std_logic;
      FSL_Rst : out std_logic;
      FSL_M_Clk : in std_logic;
      FSL_M_Data : in std_logic_vector(0 to 31);
      FSL_M_Control : in std_logic;
      FSL_M_Write : in std_logic;
      FSL_M_Full : out std_logic;
      FSL_S_Clk : in std_logic;
      FSL_S_Data : out std_logic_vector(0 to 31);
      FSL_S_Control : out std_logic;
      FSL_S_Read : in std_logic;
      FSL_S_Exists : out std_logic
    );
  end component;

  attribute box_type of ac_97_play_buffer_wrapper: component is "black_box";

  component opb_timer_0_wrapper is
    port (
      OPB_Clk : in std_logic;
      OPB_Rst : in std_logic;
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_DBus : in std_logic_vector(0 to 31);
      OPB_RNW : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      TC_DBus : out std_logic_vector(0 to 31);
      TC_errAck : out std_logic;
      TC_retry : out std_logic;
      TC_toutSup : out std_logic;
      TC_xferAck : out std_logic;
      CaptureTrig0 : in std_logic;
      CaptureTrig1 : in std_logic;
      GenerateOut0 : out std_logic;
      GenerateOut1 : out std_logic;
      PWM0 : out std_logic;
      Interrupt : out std_logic;
      Freeze : in std_logic
    );
  end component;

  attribute box_type of opb_timer_0_wrapper: component is "black_box";

  component opb_intc_0_wrapper is
    port (
      OPB_Clk : in std_logic;
      Intr : in std_logic_vector(0 downto 0);
      OPB_Rst : in std_logic;
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_RNW : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      IntC_DBus : out std_logic_vector(0 to 31);
      IntC_errAck : out std_logic;
      IntC_retry : out std_logic;
      IntC_toutSup : out std_logic;
      IntC_xferAck : out std_logic;
      Irq : out std_logic
    );
  end component;

  attribute box_type of opb_intc_0_wrapper: component is "black_box";

  component rs232_wrapper is
    port (
      OPB_Clk : in std_logic;
      OPB_Rst : in std_logic;
      Interrupt : out std_logic;
      OPB_ABus : in std_logic_vector(0 to 31);
      OPB_BE : in std_logic_vector(0 to 3);
      OPB_RNW : in std_logic;
      OPB_select : in std_logic;
      OPB_seqAddr : in std_logic;
      OPB_DBus : in std_logic_vector(0 to 31);
      UART_DBus : out std_logic_vector(0 to 31);
      UART_errAck : out std_logic;
      UART_retry : out std_logic;
      UART_toutSup : out std_logic;
      UART_xferAck : out std_logic;
      RX : in std_logic;
      TX : out std_logic
    );
  end component;

  attribute box_type of rs232_wrapper: component is "black_box";

  component BUFGP is
    port (
      I : in std_logic;
      O : out std_logic
    );
  end component;

  component IBUF is
    port (
      I : in std_logic;
      O : out std_logic
    );
  end component;

  component OBUF is
    port (
      I : in std_logic;
      O : out std_logic
    );
  end component;

  component IOBUF is
    port (
      I : in std_logic;
      IO : inout std_logic;
      O : out std_logic;
      T : in std_logic
    );
  end component;

  -- Internal signals

  signal AC97_BIT_CLOCK_P_BUFGP : std_logic;
  signal AC97_SDATA_IN_IBUF : std_logic;
  signal AC97_SDATA_OUT_OBUF : std_logic;
  signal AC97_SYNCH_P_OBUF : std_logic;
  signal Dbg_Capture : std_logic;
  signal Dbg_Clk : std_logic;
  signal Dbg_Reg_En : std_logic_vector(0 to 4);
  signal Dbg_TDI : std_logic;
  signal Dbg_TDO : std_logic;
  signal Dbg_Update : std_logic;
  signal Debug_Rst : std_logic;
  signal Ext_BRK : std_logic;
  signal Ext_NM_BRK : std_logic;
  signal RS232_RX_IBUF : std_logic;
  signal RS232_TX_OBUF : std_logic;
  signal ZBT1_ADV_LD_N_OBUF : std_logic;
  signal ZBT1_A_OBUF : std_logic_vector(0 to 18);
  signal ZBT1_BW_N_OBUF : std_logic_vector(1 to 4);
  signal ZBT1_CE1_N_OBUF : std_logic;
  signal ZBT1_CKE_N_OBUF : std_logic;
  signal ZBT1_OE_N_OBUF : std_logic;
  signal ZBT1_RW_N_OBUF : std_logic;
  signal ZBT2_ADV_LD_N_OBUF : std_logic;
  signal ZBT2_A_OBUF : std_logic_vector(0 to 18);
  signal ZBT2_BW_N_OBUF : std_logic_vector(1 to 4);
  signal ZBT2_CE1_N_OBUF : std_logic;
  signal ZBT2_CKE_N_OBUF : std_logic;
  signal ZBT2_OE_N_OBUF : std_logic;
  signal ZBT2_RW_N_OBUF : std_logic;
  signal ZBT_CLK_FB_BUFGP : std_logic;
  signal ZBT_Clk_FBOut_OBUF : std_logic;
  signal ac_97_play_buffer_FSL_M_Clk : std_logic;
  signal ac_97_play_buffer_FSL_M_Control : std_logic;
  signal ac_97_play_buffer_FSL_M_Data : std_logic_vector(0 to 31);
  signal ac_97_play_buffer_FSL_M_Full : std_logic;
  signal ac_97_play_buffer_FSL_M_Write : std_logic;
  signal ac_97_play_buffer_FSL_S_Clk : std_logic;
  signal ac_97_play_buffer_FSL_S_Control : std_logic;
  signal ac_97_play_buffer_FSL_S_Data : std_logic_vector(0 to 31);
  signal ac_97_play_buffer_FSL_S_Exists : std_logic;
  signal ac_97_play_buffer_FSL_S_Read : std_logic;
  signal clk_50_BUFGP : std_logic;
  signal d_lmb_LMB_ABus : std_logic_vector(0 to 31);
  signal d_lmb_LMB_AddrStrobe : std_logic;
  signal d_lmb_LMB_BE : std_logic_vector(0 to 3);
  signal d_lmb_LMB_ReadDBus : std_logic_vector(0 to 31);
  signal d_lmb_LMB_ReadStrobe : std_logic;
  signal d_lmb_LMB_Ready : std_logic;
  signal d_lmb_LMB_WriteDBus : std_logic_vector(0 to 31);
  signal d_lmb_LMB_WriteStrobe : std_logic;
  signal d_lmb_M_ABus : std_logic_vector(0 to 31);
  signal d_lmb_M_AddrStrobe : std_logic;
  signal d_lmb_M_BE : std_logic_vector(0 to 3);
  signal d_lmb_M_DBus : std_logic_vector(0 to 31);
  signal d_lmb_M_ReadStrobe : std_logic;
  signal d_lmb_M_WriteStrobe : std_logic;
  signal d_lmb_OPB_Rst : std_logic;
  signal d_lmb_Sl_DBus : std_logic_vector(0 to 31);
  signal d_lmb_Sl_Ready : std_logic_vector(0 to 0);
  signal dlmb_portb_BRAM_Addr : std_logic_vector(0 to 31);
  signal dlmb_portb_BRAM_Clk : std_logic;
  signal dlmb_portb_BRAM_Din : std_logic_vector(0 to 31);
  signal dlmb_portb_BRAM_Dout : std_logic_vector(0 to 31);
  signal dlmb_portb_BRAM_EN : std_logic;
  signal dlmb_portb_BRAM_Rst : std_logic;
  signal dlmb_portb_BRAM_WEN : std_logic_vector(0 to 3);
  signal download_link_FSL_M_Clk : std_logic;
  signal download_link_FSL_M_Control : std_logic;
  signal download_link_FSL_M_Data : std_logic_vector(0 to 31);
  signal download_link_FSL_M_Full : std_logic;
  signal download_link_FSL_M_Write : std_logic;
  signal download_link_FSL_S_Clk : std_logic;
  signal download_link_FSL_S_Control : std_logic;
  signal download_link_FSL_S_Data : std_logic_vector(0 to 31);
  signal download_link_FSL_S_Exists : std_logic;
  signal download_link_FSL_S_Read : std_logic;
  signal extend_dcm_reset_IBUF : std_logic;
  signal from_mb_fsl_1_FSL_M_Clk : std_logic;
  signal from_mb_fsl_1_FSL_M_Control : std_logic;
  signal from_mb_fsl_1_FSL_M_Data : std_logic_vector(0 to 31);
  signal from_mb_fsl_1_FSL_M_Full : std_logic;
  signal from_mb_fsl_1_FSL_M_Write : std_logic;
  signal from_mb_fsl_1_FSL_S_Clk : std_logic;
  signal from_mb_fsl_1_FSL_S_Control : std_logic;
  signal from_mb_fsl_1_FSL_S_Data : std_logic_vector(0 to 31);
  signal from_mb_fsl_1_FSL_S_Exists : std_logic;
  signal from_mb_fsl_1_FSL_S_Read : std_logic;
  signal from_mb_fsl_2_FSL_M_Clk : std_logic;
  signal from_mb_fsl_2_FSL_M_Control : std_logic;
  signal from_mb_fsl_2_FSL_M_Data : std_logic_vector(0 to 31);
  signal from_mb_fsl_2_FSL_M_Full : std_logic;
  signal from_mb_fsl_2_FSL_M_Write : std_logic;
  signal from_mb_fsl_2_FSL_S_Clk : std_logic;
  signal from_mb_fsl_2_FSL_S_Control : std_logic;
  signal from_mb_fsl_2_FSL_S_Data : std_logic_vector(0 to 31);
  signal from_mb_fsl_2_FSL_S_Exists : std_logic;
  signal from_mb_fsl_2_FSL_S_Read : std_logic;
  signal from_mb_fsl_3_FSL_M_Clk : std_logic;
  signal from_mb_fsl_3_FSL_M_Control : std_logic;
  signal from_mb_fsl_3_FSL_M_Data : std_logic_vector(0 to 31);
  signal from_mb_fsl_3_FSL_M_Full : std_logic;
  signal from_mb_fsl_3_FSL_M_Write : std_logic;
  signal from_mb_fsl_3_FSL_S_Clk : std_logic;
  signal from_mb_fsl_3_FSL_S_Control : std_logic;
  signal from_mb_fsl_3_FSL_S_Data : std_logic_vector(0 to 31);
  signal from_mb_fsl_3_FSL_S_Exists : std_logic;
  signal from_mb_fsl_3_FSL_S_Read : std_logic;
  signal from_mb_fsl_4_FSL_M_Clk : std_logic;
  signal from_mb_fsl_4_FSL_M_Control : std_logic;
  signal from_mb_fsl_4_FSL_M_Data : std_logic_vector(0 to 31);
  signal from_mb_fsl_4_FSL_M_Full : std_logic;
  signal from_mb_fsl_4_FSL_M_Write : std_logic;
  signal from_mb_fsl_4_FSL_S_Clk : std_logic;
  signal from_mb_fsl_4_FSL_S_Control : std_logic;
  signal from_mb_fsl_4_FSL_S_Data : std_logic_vector(0 to 31);
  signal from_mb_fsl_4_FSL_S_Exists : std_logic;
  signal from_mb_fsl_4_FSL_S_Read : std_logic;
  signal i_lmb_LMB_ABus : std_logic_vector(0 to 31);
  signal i_lmb_LMB_AddrStrobe : std_logic;
  signal i_lmb_LMB_BE : std_logic_vector(0 to 3);
  signal i_lmb_LMB_ReadDBus : std_logic_vector(0 to 31);
  signal i_lmb_LMB_ReadStrobe : std_logic;
  signal i_lmb_LMB_Ready : std_logic;
  signal i_lmb_LMB_WriteDBus : std_logic_vector(0 to 31);
  signal i_lmb_LMB_WriteStrobe : std_logic;
  signal i_lmb_M_ABus : std_logic_vector(0 to 31);
  signal i_lmb_M_AddrStrobe : std_logic;
  signal i_lmb_M_ReadStrobe : std_logic;
  signal i_lmb_OPB_Rst : std_logic;
  signal i_lmb_Sl_DBus : std_logic_vector(0 to 31);
  signal i_lmb_Sl_Ready : std_logic_vector(0 to 0);
  signal ilmb_porta_BRAM_Addr : std_logic_vector(0 to 31);
  signal ilmb_porta_BRAM_Clk : std_logic;
  signal ilmb_porta_BRAM_Din : std_logic_vector(0 to 31);
  signal ilmb_porta_BRAM_Dout : std_logic_vector(0 to 31);
  signal ilmb_porta_BRAM_EN : std_logic;
  signal ilmb_porta_BRAM_Rst : std_logic;
  signal ilmb_porta_BRAM_WEN : std_logic_vector(0 to 3);
  signal intc_to_myblaze : std_logic;
  signal main_reset : std_logic;
  signal myopb_bus_M_ABus : std_logic_vector(0 to 63);
  signal myopb_bus_M_BE : std_logic_vector(0 to 7);
  signal myopb_bus_M_DBus : std_logic_vector(0 to 63);
  signal myopb_bus_M_RNW : std_logic_vector(0 to 1);
  signal myopb_bus_M_busLock : std_logic_vector(0 to 1);
  signal myopb_bus_M_request : std_logic_vector(0 to 1);
  signal myopb_bus_M_select : std_logic_vector(0 to 1);
  signal myopb_bus_M_seqAddr : std_logic_vector(0 to 1);
  signal myopb_bus_OPB_ABus : std_logic_vector(0 to 31);
  signal myopb_bus_OPB_BE : std_logic_vector(0 to 3);
  signal myopb_bus_OPB_DBus : std_logic_vector(0 to 31);
  signal myopb_bus_OPB_MGrant : std_logic_vector(0 to 1);
  signal myopb_bus_OPB_RNW : std_logic;
  signal myopb_bus_OPB_Rst : std_logic;
  signal myopb_bus_OPB_errAck : std_logic;
  signal myopb_bus_OPB_retry : std_logic;
  signal myopb_bus_OPB_select : std_logic;
  signal myopb_bus_OPB_seqAddr : std_logic;
  signal myopb_bus_OPB_timeout : std_logic;
  signal myopb_bus_OPB_xferAck : std_logic;
  signal myopb_bus_Sl_DBus : std_logic_vector(0 to 319);
  signal myopb_bus_Sl_errAck : std_logic_vector(0 to 9);
  signal myopb_bus_Sl_retry : std_logic_vector(0 to 9);
  signal myopb_bus_Sl_toutSup : std_logic_vector(0 to 9);
  signal myopb_bus_Sl_xferAck : std_logic_vector(0 to 9);
  signal net_gnd0 : std_logic;
  signal net_gnd10 : std_logic_vector(0 to 9);
  signal net_gnd2 : std_logic_vector(0 to 1);
  signal net_gnd3 : std_logic_vector(0 to 2);
  signal net_gnd32 : std_logic_vector(0 to 31);
  signal net_gnd4 : std_logic_vector(0 to 3);
  signal net_vcc0 : std_logic;
  signal net_vcc10 : std_logic_vector(0 to 9);
  signal net_vcc2 : std_logic_vector(0 to 1);
  signal opb_gpio_0_GPIO_IO_I : std_logic_vector(0 to 3);
  signal opb_gpio_0_GPIO_IO_O : std_logic_vector(0 to 3);
  signal opb_gpio_0_GPIO_IO_T : std_logic_vector(0 to 3);
  signal opb_gpio_1_GPIO_in : std_logic_vector(0 to 2);
  signal opb_gpio_2_GPIO_in : std_logic_vector(0 to 2);
  signal sys_clk : std_logic;
  signal sys_clk_100 : std_logic;
  signal sys_clk_50 : std_logic;
  signal sys_clk_50_i : std_logic;
  signal timer_to_intc : std_logic_vector(0 downto 0);
  signal to_mb_fsl_1_FSL_M_Clk : std_logic;
  signal to_mb_fsl_1_FSL_M_Control : std_logic;
  signal to_mb_fsl_1_FSL_M_Data : std_logic_vector(0 to 31);
  signal to_mb_fsl_1_FSL_M_Full : std_logic;
  signal to_mb_fsl_1_FSL_M_Write : std_logic;
  signal to_mb_fsl_1_FSL_S_Clk : std_logic;
  signal to_mb_fsl_1_FSL_S_Control : std_logic;
  signal to_mb_fsl_1_FSL_S_Data : std_logic_vector(0 to 31);
  signal to_mb_fsl_1_FSL_S_Exists : std_logic;
  signal to_mb_fsl_1_FSL_S_Read : std_logic;
  signal to_mb_fsl_1_OPB_Rst : std_logic;
  signal to_mb_fsl_2_FSL_M_Clk : std_logic;
  signal to_mb_fsl_2_FSL_M_Control : std_logic;
  signal to_mb_fsl_2_FSL_M_Data : std_logic_vector(0 to 31);
  signal to_mb_fsl_2_FSL_M_Full : std_logic;
  signal to_mb_fsl_2_FSL_M_Write : std_logic;
  signal to_mb_fsl_2_FSL_S_Clk : std_logic;
  signal to_mb_fsl_2_FSL_S_Control : std_logic;
  signal to_mb_fsl_2_FSL_S_Data : std_logic_vector(0 to 31);
  signal to_mb_fsl_2_FSL_S_Exists : std_logic;
  signal to_mb_fsl_2_FSL_S_Read : std_logic;
  signal to_mb_fsl_2_OPB_Rst : std_logic;
  signal to_mb_fsl_3_FSL_M_Clk : std_logic;
  signal to_mb_fsl_3_FSL_M_Control : std_logic;
  signal to_mb_fsl_3_FSL_M_Data : std_logic_vector(0 to 31);
  signal to_mb_fsl_3_FSL_M_Full : std_logic;
  signal to_mb_fsl_3_FSL_M_Write : std_logic;
  signal to_mb_fsl_3_FSL_S_Clk : std_logic;
  signal to_mb_fsl_3_FSL_S_Control : std_logic;
  signal to_mb_fsl_3_FSL_S_Data : std_logic_vector(0 to 31);
  signal to_mb_fsl_3_FSL_S_Exists : std_logic;
  signal to_mb_fsl_3_FSL_S_Read : std_logic;
  signal to_mb_fsl_3_OPB_Rst : std_logic;
  signal to_mb_fsl_4_FSL_M_Clk : std_logic;
  signal to_mb_fsl_4_FSL_M_Control : std_logic;
  signal to_mb_fsl_4_FSL_M_Data : std_logic_vector(0 to 31);
  signal to_mb_fsl_4_FSL_M_Full : std_logic;
  signal to_mb_fsl_4_FSL_M_Write : std_logic;
  signal to_mb_fsl_4_FSL_S_Clk : std_logic;
  signal to_mb_fsl_4_FSL_S_Control : std_logic;
  signal to_mb_fsl_4_FSL_S_Data : std_logic_vector(0 to 31);
  signal to_mb_fsl_4_FSL_S_Exists : std_logic;
  signal to_mb_fsl_4_FSL_S_Read : std_logic;
  signal to_mb_fsl_4_OPB_Rst : std_logic;
  signal volume_buttons_0_PB_CLOCK_P_IBUF : std_logic;
  signal volume_buttons_0_PB_DATA_P_IBUF : std_logic;
  signal zbt1_io_I : std_logic_vector(0 to 31);
  signal zbt1_io_O : std_logic_vector(0 to 31);
  signal zbt1_io_T : std_logic;
  signal zbt2_io_I : std_logic_vector(0 to 31);
  signal zbt2_io_O : std_logic_vector(0 to 31);
  signal zbt2_io_T : std_logic;
  signal zbt_Clk_100 : std_logic;

begin

  -- Internal assignments

  net_gnd0 <= '0';
  net_gnd10(0 to 9) <= B"0000000000";
  net_gnd2(0 to 1) <= B"00";
  net_gnd3(0 to 2) <= B"000";
  net_gnd32(0 to 31) <= B"00000000000000000000000000000000";
  net_gnd4(0 to 3) <= B"0000";
  net_vcc0 <= '1';
  net_vcc10(0 to 9) <= B"1111111111";
  net_vcc2(0 to 1) <= B"11";

  opb_gpio_0 : opb_gpio_0_wrapper
    port map (
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_Clk => sys_clk,
      OPB_DBus => myopb_bus_OPB_DBus,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_Rst => myopb_bus_OPB_Rst,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      Sln_DBus => myopb_bus_Sl_DBus(0 to 31),
      Sln_errAck => myopb_bus_Sl_errAck(0),
      Sln_retry => myopb_bus_Sl_retry(0),
      Sln_toutSup => myopb_bus_Sl_toutSup(0),
      Sln_xferAck => myopb_bus_Sl_xferAck(0),
      IP2INTC_Irpt => open,
      GPIO_in => net_gnd4,
      GPIO_d_out => open,
      GPIO_t_out => open,
      GPIO2_in => net_gnd4,
      GPIO2_d_out => open,
      GPIO2_t_out => open,
      GPIO_IO_I => opb_gpio_0_GPIO_IO_I,
      GPIO_IO_O => opb_gpio_0_GPIO_IO_O,
      GPIO_IO_T => opb_gpio_0_GPIO_IO_T,
      GPIO2_IO_I => net_gnd4,
      GPIO2_IO_O => open,
      GPIO2_IO_T => open
    );

  opb_gpio_1 : opb_gpio_1_wrapper
    port map (
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_Clk => sys_clk,
      OPB_DBus => myopb_bus_OPB_DBus,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_Rst => myopb_bus_OPB_Rst,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      Sln_DBus => myopb_bus_Sl_DBus(32 to 63),
      Sln_errAck => myopb_bus_Sl_errAck(1),
      Sln_retry => myopb_bus_Sl_retry(1),
      Sln_toutSup => myopb_bus_Sl_toutSup(1),
      Sln_xferAck => myopb_bus_Sl_xferAck(1),
      IP2INTC_Irpt => open,
      GPIO_in => opb_gpio_1_GPIO_in,
      GPIO_d_out => open,
      GPIO_t_out => open,
      GPIO2_in => net_gnd3,
      GPIO2_d_out => open,
      GPIO2_t_out => open,
      GPIO_IO_I => net_gnd3,
      GPIO_IO_O => open,
      GPIO_IO_T => open,
      GPIO2_IO_I => net_gnd3,
      GPIO2_IO_O => open,
      GPIO2_IO_T => open
    );

  opb_gpio_2 : opb_gpio_2_wrapper
    port map (
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_Clk => sys_clk,
      OPB_DBus => myopb_bus_OPB_DBus,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_Rst => myopb_bus_OPB_Rst,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      Sln_DBus => myopb_bus_Sl_DBus(64 to 95),
      Sln_errAck => myopb_bus_Sl_errAck(2),
      Sln_retry => myopb_bus_Sl_retry(2),
      Sln_toutSup => myopb_bus_Sl_toutSup(2),
      Sln_xferAck => myopb_bus_Sl_xferAck(2),
      IP2INTC_Irpt => open,
      GPIO_in => opb_gpio_2_GPIO_in,
      GPIO_d_out => open,
      GPIO_t_out => open,
      GPIO2_in => net_gnd3,
      GPIO2_d_out => open,
      GPIO2_t_out => open,
      GPIO_IO_I => net_gnd3,
      GPIO_IO_O => open,
      GPIO_IO_T => open,
      GPIO2_IO_I => net_gnd3,
      GPIO2_IO_O => open,
      GPIO2_IO_T => open
    );

  volume_buttons_0 : volume_buttons_0_wrapper
    port map (
      clock => sys_clk,
      reset_b => extend_dcm_reset_IBUF,
      PB_CLOCK_P => volume_buttons_0_PB_CLOCK_P_IBUF,
      PB_DATA_P => volume_buttons_0_PB_DATA_P_IBUF,
      mic_volume => opb_gpio_1_GPIO_in(0 to 2),
      song_volume => opb_gpio_2_GPIO_in(0 to 2),
      startup => open
    );

  clk_50_bufg : clk_50_bufg_wrapper
    port map (
      I => sys_clk_50,
      O => sys_clk_50_i
    );

  main_dcm : main_dcm_wrapper
    port map (
      CLKIN => clk_50_BUFGP,
      CLKFB => sys_clk_50_i,
      DSSEN => net_gnd0,
      PSINCDEC => net_gnd0,
      PSEN => net_gnd0,
      PSCLK => net_gnd0,
      RST => net_gnd0,
      CLK0 => sys_clk_50,
      CLK90 => open,
      CLK180 => open,
      CLK270 => open,
      CLK2X => sys_clk_100,
      CLK2X180 => open,
      CLKDV => open,
      CLKFX => open,
      CLKFX180 => open,
      LOCKED => open,
      PSDONE => open,
      STATUS => open,
      RESET_IN => extend_dcm_reset_IBUF,
      FPGA_DONE => extend_dcm_reset_IBUF,
      RESET_OUT => main_reset
    );

  sys_clk_bufg : sys_clk_bufg_wrapper
    port map (
      I => sys_clk_100,
      O => sys_clk
    );

  zbt_dcm : zbt_dcm_wrapper
    port map (
      CLKIN => clk_50_BUFGP,
      CLKFB => ZBT_CLK_FB_BUFGP,
      DSSEN => net_gnd0,
      PSINCDEC => net_gnd0,
      PSEN => net_gnd0,
      PSCLK => net_gnd0,
      RST => net_gnd0,
      CLK0 => ZBT_Clk_FBOut_OBUF,
      CLK90 => open,
      CLK180 => open,
      CLK270 => open,
      CLK2X => zbt_Clk_100,
      CLK2X180 => open,
      CLKDV => open,
      CLKFX => open,
      CLKFX180 => open,
      LOCKED => open,
      PSDONE => open,
      STATUS => open,
      RESET_IN => extend_dcm_reset_IBUF,
      FPGA_DONE => extend_dcm_reset_IBUF,
      RESET_OUT => open
    );

  myopb_bus : myopb_bus_wrapper
    port map (
      OPB_Clk => sys_clk,
      OPB_Rst => myopb_bus_OPB_Rst,
      SYS_Rst => main_reset,
      Debug_SYS_Rst => net_gnd0,
      WDT_Rst => net_gnd0,
      M_ABus => myopb_bus_M_ABus,
      M_BE => myopb_bus_M_BE,
      M_beXfer => net_gnd2,
      M_busLock => myopb_bus_M_busLock,
      M_DBus => myopb_bus_M_DBus,
      M_DBusEn => net_gnd2,
      M_DBusEn32_63 => net_vcc2,
      M_dwXfer => net_gnd2,
      M_fwXfer => net_gnd2,
      M_hwXfer => net_gnd2,
      M_request => myopb_bus_M_request,
      M_RNW => myopb_bus_M_RNW,
      M_select => myopb_bus_M_select,
      M_seqAddr => myopb_bus_M_seqAddr,
      Sl_beAck => net_gnd10,
      Sl_DBus => myopb_bus_Sl_DBus,
      Sl_DBusEn => net_vcc10,
      Sl_DBusEn32_63 => net_vcc10,
      Sl_errAck => myopb_bus_Sl_errAck,
      Sl_dwAck => net_gnd10,
      Sl_fwAck => net_gnd10,
      Sl_hwAck => net_gnd10,
      Sl_retry => myopb_bus_Sl_retry,
      Sl_toutSup => myopb_bus_Sl_toutSup,
      Sl_xferAck => myopb_bus_Sl_xferAck,
      OPB_MRequest => open,
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_beXfer => open,
      OPB_beAck => open,
      OPB_busLock => open,
      OPB_rdDBus => open,
      OPB_wrDBus => open,
      OPB_DBus => myopb_bus_OPB_DBus,
      OPB_errAck => myopb_bus_OPB_errAck,
      OPB_dwAck => open,
      OPB_dwXfer => open,
      OPB_fwAck => open,
      OPB_fwXfer => open,
      OPB_hwAck => open,
      OPB_hwXfer => open,
      OPB_MGrant => myopb_bus_OPB_MGrant,
      OPB_pendReq => open,
      OPB_retry => myopb_bus_OPB_retry,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      OPB_timeout => myopb_bus_OPB_timeout,
      OPB_toutSup => open,
      OPB_xferAck => myopb_bus_OPB_xferAck
    );

  my_blaze : my_blaze_wrapper
    port map (
      CLK => sys_clk,
      RESET => myopb_bus_OPB_Rst,
      INTERRUPT => intc_to_myblaze,
      DEBUG_RST => Debug_Rst,
      EXT_BRK => Ext_BRK,
      EXT_NM_BRK => Ext_NM_BRK,
      DBG_STOP => net_gnd0,
      INSTR => i_lmb_LMB_ReadDBus,
      IREADY => i_lmb_LMB_Ready,
      INSTR_ADDR => i_lmb_M_ABus,
      IFETCH => i_lmb_M_ReadStrobe,
      I_AS => i_lmb_M_AddrStrobe,
      DATA_READ => d_lmb_LMB_ReadDBus,
      DREADY => d_lmb_LMB_Ready,
      DATA_WRITE => d_lmb_M_DBus,
      DATA_ADDR => d_lmb_M_ABus,
      D_AS => d_lmb_M_AddrStrobe,
      READ_STROBE => d_lmb_M_ReadStrobe,
      WRITE_STROBE => d_lmb_M_WriteStrobe,
      BYTE_ENABLE => d_lmb_M_BE,
      DM_ABUS => myopb_bus_M_ABus(0 to 31),
      DM_BE => myopb_bus_M_BE(0 to 3),
      DM_BUSLOCK => myopb_bus_M_busLock(0),
      DM_DBUS => myopb_bus_M_DBus(0 to 31),
      DM_REQUEST => myopb_bus_M_request(0),
      DM_RNW => myopb_bus_M_RNW(0),
      DM_SELECT => myopb_bus_M_select(0),
      DM_SEQADDR => myopb_bus_M_seqAddr(0),
      DOPB_DBUS => myopb_bus_OPB_DBus,
      DOPB_ERRACK => myopb_bus_OPB_errAck,
      DOPB_MGRANT => myopb_bus_OPB_MGrant(0),
      DOPB_RETRY => myopb_bus_OPB_retry,
      DOPB_TIMEOUT => myopb_bus_OPB_timeout,
      DOPB_XFERACK => myopb_bus_OPB_xferAck,
      IM_ABUS => myopb_bus_M_ABus(32 to 63),
      IM_BE => myopb_bus_M_BE(4 to 7),
      IM_BUSLOCK => myopb_bus_M_busLock(1),
      IM_DBUS => myopb_bus_M_DBus(32 to 63),
      IM_REQUEST => myopb_bus_M_request(1),
      IM_RNW => myopb_bus_M_RNW(1),
      IM_SELECT => myopb_bus_M_select(1),
      IM_SEQADDR => myopb_bus_M_seqAddr(1),
      IOPB_DBUS => myopb_bus_OPB_DBus,
      IOPB_ERRACK => myopb_bus_OPB_errAck,
      IOPB_MGRANT => myopb_bus_OPB_MGrant(1),
      IOPB_RETRY => myopb_bus_OPB_retry,
      IOPB_TIMEOUT => myopb_bus_OPB_timeout,
      IOPB_XFERACK => myopb_bus_OPB_xferAck,
      DBG_CLK => Dbg_Clk,
      DBG_TDI => Dbg_TDI,
      DBG_TDO => Dbg_TDO,
      DBG_REG_EN => Dbg_Reg_En,
      DBG_CAPTURE => Dbg_Capture,
      DBG_UPDATE => Dbg_Update,
      VALID_INSTR => open,
      PC_EX => open,
      REG_WRITE => open,
      REG_ADDR => open,
      MSR_REG => open,
      NEW_REG_VALUE => open,
      PIPE_RUNNING => open,
      INTERRUPT_TAKEN => open,
      JUMP_TAKEN => open,
      PREFETCH_ADDR => open,
      MB_Halted => open,
      FSL0_S_CLK => download_link_FSL_S_Clk,
      FSL0_S_READ => download_link_FSL_S_Read,
      FSL0_S_DATA => download_link_FSL_S_Data,
      FSL0_S_CONTROL => download_link_FSL_S_Control,
      FSL0_S_EXISTS => download_link_FSL_S_Exists,
      FSL0_M_CLK => open,
      FSL0_M_WRITE => open,
      FSL0_M_DATA => open,
      FSL0_M_CONTROL => open,
      FSL0_M_FULL => net_gnd0,
      FSL1_S_CLK => to_mb_fsl_1_FSL_S_Clk,
      FSL1_S_READ => to_mb_fsl_1_FSL_S_Read,
      FSL1_S_DATA => to_mb_fsl_1_FSL_S_Data,
      FSL1_S_CONTROL => to_mb_fsl_1_FSL_S_Control,
      FSL1_S_EXISTS => to_mb_fsl_1_FSL_S_Exists,
      FSL1_M_CLK => from_mb_fsl_1_FSL_M_Clk,
      FSL1_M_WRITE => from_mb_fsl_1_FSL_M_Write,
      FSL1_M_DATA => from_mb_fsl_1_FSL_M_Data,
      FSL1_M_CONTROL => from_mb_fsl_1_FSL_M_Control,
      FSL1_M_FULL => from_mb_fsl_1_FSL_M_Full,
      FSL2_S_CLK => to_mb_fsl_2_FSL_S_Clk,
      FSL2_S_READ => to_mb_fsl_2_FSL_S_Read,
      FSL2_S_DATA => to_mb_fsl_2_FSL_S_Data,
      FSL2_S_CONTROL => to_mb_fsl_2_FSL_S_Control,
      FSL2_S_EXISTS => to_mb_fsl_2_FSL_S_Exists,
      FSL2_M_CLK => from_mb_fsl_2_FSL_M_Clk,
      FSL2_M_WRITE => from_mb_fsl_2_FSL_M_Write,
      FSL2_M_DATA => from_mb_fsl_2_FSL_M_Data,
      FSL2_M_CONTROL => from_mb_fsl_2_FSL_M_Control,
      FSL2_M_FULL => from_mb_fsl_2_FSL_M_Full,
      FSL3_S_CLK => to_mb_fsl_3_FSL_S_Clk,
      FSL3_S_READ => to_mb_fsl_3_FSL_S_Read,
      FSL3_S_DATA => to_mb_fsl_3_FSL_S_Data,
      FSL3_S_CONTROL => to_mb_fsl_3_FSL_S_Control,
      FSL3_S_EXISTS => to_mb_fsl_3_FSL_S_Exists,
      FSL3_M_CLK => from_mb_fsl_3_FSL_M_Clk,
      FSL3_M_WRITE => from_mb_fsl_3_FSL_M_Write,
      FSL3_M_DATA => from_mb_fsl_3_FSL_M_Data,
      FSL3_M_CONTROL => from_mb_fsl_3_FSL_M_Control,
      FSL3_M_FULL => from_mb_fsl_3_FSL_M_Full,
      FSL4_S_CLK => to_mb_fsl_4_FSL_S_Clk,
      FSL4_S_READ => to_mb_fsl_4_FSL_S_Read,
      FSL4_S_DATA => to_mb_fsl_4_FSL_S_Data,
      FSL4_S_CONTROL => to_mb_fsl_4_FSL_S_Control,
      FSL4_S_EXISTS => to_mb_fsl_4_FSL_S_Exists,
      FSL4_M_CLK => from_mb_fsl_4_FSL_M_Clk,
      FSL4_M_WRITE => from_mb_fsl_4_FSL_M_Write,
      FSL4_M_DATA => from_mb_fsl_4_FSL_M_Data,
      FSL4_M_CONTROL => from_mb_fsl_4_FSL_M_Control,
      FSL4_M_FULL => from_mb_fsl_4_FSL_M_Full,
      FSL5_S_CLK => open,
      FSL5_S_READ => open,
      FSL5_S_DATA => net_gnd32,
      FSL5_S_CONTROL => net_gnd0,
      FSL5_S_EXISTS => net_gnd0,
      FSL5_M_CLK => ac_97_play_buffer_FSL_M_Clk,
      FSL5_M_WRITE => ac_97_play_buffer_FSL_M_Write,
      FSL5_M_DATA => ac_97_play_buffer_FSL_M_Data,
      FSL5_M_CONTROL => ac_97_play_buffer_FSL_M_Control,
      FSL5_M_FULL => ac_97_play_buffer_FSL_M_Full,
      FSL6_S_CLK => open,
      FSL6_S_READ => open,
      FSL6_S_DATA => net_gnd32,
      FSL6_S_CONTROL => net_gnd0,
      FSL6_S_EXISTS => net_gnd0,
      FSL6_M_CLK => open,
      FSL6_M_WRITE => open,
      FSL6_M_DATA => open,
      FSL6_M_CONTROL => open,
      FSL6_M_FULL => net_gnd0,
      FSL7_S_CLK => open,
      FSL7_S_READ => open,
      FSL7_S_DATA => net_gnd32,
      FSL7_S_CONTROL => net_gnd0,
      FSL7_S_EXISTS => net_gnd0,
      FSL7_M_CLK => open,
      FSL7_M_WRITE => open,
      FSL7_M_DATA => open,
      FSL7_M_CONTROL => open,
      FSL7_M_FULL => net_gnd0
    );

  my_ac97 : my_ac97_wrapper
    port map (
      OPB_Clk => sys_clk,
      OPB_Rst => myopb_bus_OPB_Rst,
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      OPB_DBus => myopb_bus_OPB_DBus,
      opb_ac97_controller_DBus => myopb_bus_Sl_DBus(96 to 127),
      opb_ac97_controller_errAck => myopb_bus_Sl_errAck(3),
      opb_ac97_controller_retry => myopb_bus_Sl_retry(3),
      opb_ac97_controller_toutSup => myopb_bus_Sl_toutSup(3),
      opb_ac97_controller_xferAck => myopb_bus_Sl_xferAck(3),
      Playback_Interrupt => open,
      Record_Interrupt => open,
      Bit_Clk => AC97_BIT_CLOCK_P_BUFGP,
      Sync => AC97_SYNCH_P_OBUF,
      SData_Out => AC97_SDATA_OUT_OBUF,
      SData_In => AC97_SDATA_IN_IBUF,
      FSL_S_CLK => ac_97_play_buffer_FSL_S_Clk,
      FSL_S_READ => ac_97_play_buffer_FSL_S_Read,
      FSL_S_DATA => ac_97_play_buffer_FSL_S_Data,
      FSL_S_CONTROL => ac_97_play_buffer_FSL_S_Control,
      FSL_S_EXISTS => ac_97_play_buffer_FSL_S_Exists
    );

  my_mdm : my_mdm_wrapper
    port map (
      OPB_Clk => sys_clk,
      OPB_Rst => myopb_bus_OPB_Rst,
      Interrupt => open,
      Debug_SYS_Rst => open,
      Debug_Rst => Debug_Rst,
      Ext_BRK => Ext_BRK,
      Ext_NM_BRK => Ext_NM_BRK,
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      OPB_DBus => myopb_bus_OPB_DBus,
      MDM_DBus => myopb_bus_Sl_DBus(128 to 159),
      MDM_errAck => myopb_bus_Sl_errAck(4),
      MDM_retry => myopb_bus_Sl_retry(4),
      MDM_toutSup => myopb_bus_Sl_toutSup(4),
      MDM_xferAck => myopb_bus_Sl_xferAck(4),
      Dbg_Clk_0 => Dbg_Clk,
      Dbg_TDI_0 => Dbg_TDI,
      Dbg_TDO_0 => Dbg_TDO,
      Dbg_Reg_En_0 => Dbg_Reg_En,
      Dbg_Capture_0 => Dbg_Capture,
      Dbg_Update_0 => Dbg_Update,
      Dbg_Clk_1 => open,
      Dbg_TDI_1 => open,
      Dbg_TDO_1 => net_gnd0,
      Dbg_Reg_En_1 => open,
      Dbg_Capture_1 => open,
      Dbg_Update_1 => open,
      Dbg_Clk_2 => open,
      Dbg_TDI_2 => open,
      Dbg_TDO_2 => net_gnd0,
      Dbg_Reg_En_2 => open,
      Dbg_Capture_2 => open,
      Dbg_Update_2 => open,
      Dbg_Clk_3 => open,
      Dbg_TDI_3 => open,
      Dbg_TDO_3 => net_gnd0,
      Dbg_Reg_En_3 => open,
      Dbg_Capture_3 => open,
      Dbg_Update_3 => open,
      Dbg_Clk_4 => open,
      Dbg_TDI_4 => open,
      Dbg_TDO_4 => net_gnd0,
      Dbg_Reg_En_4 => open,
      Dbg_Capture_4 => open,
      Dbg_Update_4 => open,
      Dbg_Clk_5 => open,
      Dbg_TDI_5 => open,
      Dbg_TDO_5 => net_gnd0,
      Dbg_Reg_En_5 => open,
      Dbg_Capture_5 => open,
      Dbg_Update_5 => open,
      Dbg_Clk_6 => open,
      Dbg_TDI_6 => open,
      Dbg_TDO_6 => net_gnd0,
      Dbg_Reg_En_6 => open,
      Dbg_Capture_6 => open,
      Dbg_Update_6 => open,
      Dbg_Clk_7 => open,
      Dbg_TDI_7 => open,
      Dbg_TDO_7 => net_gnd0,
      Dbg_Reg_En_7 => open,
      Dbg_Capture_7 => open,
      Dbg_Update_7 => open,
      bscan_tdi => open,
      bscan_reset => open,
      bscan_shift => open,
      bscan_update => open,
      bscan_capture => open,
      bscan_sel1 => open,
      bscan_drck1 => open,
      bscan_tdo1 => net_gnd0,
      FSL0_S_CLK => open,
      FSL0_S_READ => open,
      FSL0_S_DATA => net_gnd32,
      FSL0_S_CONTROL => net_gnd0,
      FSL0_S_EXISTS => net_gnd0,
      FSL0_M_CLK => download_link_FSL_M_Clk,
      FSL0_M_WRITE => download_link_FSL_M_Write,
      FSL0_M_DATA => download_link_FSL_M_Data,
      FSL0_M_CONTROL => download_link_FSL_M_Control,
      FSL0_M_FULL => download_link_FSL_M_Full,
      ICON_Control => open,
      jtag_clk => open,
      trig => open,
      data => open
    );

  download_link : download_link_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => open,
      FSL_M_Clk => download_link_FSL_M_Clk,
      FSL_M_Data => download_link_FSL_M_Data,
      FSL_M_Control => download_link_FSL_M_Control,
      FSL_M_Write => download_link_FSL_M_Write,
      FSL_M_Full => download_link_FSL_M_Full,
      FSL_S_Clk => download_link_FSL_S_Clk,
      FSL_S_Data => download_link_FSL_S_Data,
      FSL_S_Control => download_link_FSL_S_Control,
      FSL_S_Read => download_link_FSL_S_Read,
      FSL_S_Exists => download_link_FSL_S_Exists
    );

  my_zbt : my_zbt_wrapper
    port map (
      OPB_Clk => sys_clk,
      OPB_Rst => myopb_bus_OPB_Rst,
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      OPB_DBus => myopb_bus_OPB_DBus,
      ZBT_DBus => myopb_bus_Sl_DBus(160 to 191),
      ZBT_errAck => myopb_bus_Sl_errAck(5),
      ZBT_retry => myopb_bus_Sl_retry(5),
      ZBT_toutSup => myopb_bus_Sl_toutSup(5),
      ZBT_xferAck => myopb_bus_Sl_xferAck(5),
      ZBT_Clk_FB => net_gnd0,
      ZBT_Clk_FBOut => open,
      ZBT_Clk => open,
      ZBT_OE_N => ZBT1_OE_N_OBUF,
      ZBT_ADV_LD_N => ZBT1_ADV_LD_N_OBUF,
      ZBT_LBO_N => open,
      ZBT_CE1_N => ZBT1_CE1_N_OBUF,
      ZBT_CE2_N => open,
      ZBT_CE2 => open,
      ZBT_RW_N => ZBT1_RW_N_OBUF,
      ZBT_CKE_N => ZBT1_CKE_N_OBUF,
      ZBT_A => ZBT1_A_OBUF,
      ZBT_BW_N => ZBT1_BW_N_OBUF,
      ZBT_IO_I => zbt1_io_I,
      ZBT_IO_O => zbt1_io_O,
      ZBT_IO_T => zbt1_io_T,
      ZBT_IOP_I => net_gnd4,
      ZBT_IOP_O => open,
      ZBT_IOP_T => open
    );

  my_zbt_2 : my_zbt_2_wrapper
    port map (
      OPB_Clk => sys_clk,
      OPB_Rst => myopb_bus_OPB_Rst,
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      OPB_DBus => myopb_bus_OPB_DBus,
      ZBT_DBus => myopb_bus_Sl_DBus(192 to 223),
      ZBT_errAck => myopb_bus_Sl_errAck(6),
      ZBT_retry => myopb_bus_Sl_retry(6),
      ZBT_toutSup => myopb_bus_Sl_toutSup(6),
      ZBT_xferAck => myopb_bus_Sl_xferAck(6),
      ZBT_Clk_FB => net_gnd0,
      ZBT_Clk_FBOut => open,
      ZBT_Clk => open,
      ZBT_OE_N => ZBT2_OE_N_OBUF,
      ZBT_ADV_LD_N => ZBT2_ADV_LD_N_OBUF,
      ZBT_LBO_N => open,
      ZBT_CE1_N => ZBT2_CE1_N_OBUF,
      ZBT_CE2_N => open,
      ZBT_CE2 => open,
      ZBT_RW_N => ZBT2_RW_N_OBUF,
      ZBT_CKE_N => ZBT2_CKE_N_OBUF,
      ZBT_A => ZBT2_A_OBUF,
      ZBT_BW_N => ZBT2_BW_N_OBUF,
      ZBT_IO_I => zbt2_io_I,
      ZBT_IO_O => zbt2_io_O,
      ZBT_IO_T => zbt2_io_T,
      ZBT_IOP_I => net_gnd4,
      ZBT_IOP_O => open,
      ZBT_IOP_T => open
    );

  islmb : islmb_wrapper
    port map (
      LMB_Clk => sys_clk,
      LMB_Rst => i_lmb_OPB_Rst,
      LMB_ABus => i_lmb_LMB_ABus,
      LMB_WriteDBus => i_lmb_LMB_WriteDBus,
      LMB_AddrStrobe => i_lmb_LMB_AddrStrobe,
      LMB_ReadStrobe => i_lmb_LMB_ReadStrobe,
      LMB_WriteStrobe => i_lmb_LMB_WriteStrobe,
      LMB_BE => i_lmb_LMB_BE,
      Sl_DBus => i_lmb_Sl_DBus,
      Sl_Ready => i_lmb_Sl_Ready(0),
      BRAM_Rst_A => ilmb_porta_BRAM_Rst,
      BRAM_Clk_A => ilmb_porta_BRAM_Clk,
      BRAM_EN_A => ilmb_porta_BRAM_EN,
      BRAM_WEN_A => ilmb_porta_BRAM_WEN,
      BRAM_Addr_A => ilmb_porta_BRAM_Addr,
      BRAM_Din_A => ilmb_porta_BRAM_Din,
      BRAM_Dout_A => ilmb_porta_BRAM_Dout
    );

  dslmb : dslmb_wrapper
    port map (
      LMB_Clk => sys_clk,
      LMB_Rst => d_lmb_OPB_Rst,
      LMB_ABus => d_lmb_LMB_ABus,
      LMB_WriteDBus => d_lmb_LMB_WriteDBus,
      LMB_AddrStrobe => d_lmb_LMB_AddrStrobe,
      LMB_ReadStrobe => d_lmb_LMB_ReadStrobe,
      LMB_WriteStrobe => d_lmb_LMB_WriteStrobe,
      LMB_BE => d_lmb_LMB_BE,
      Sl_DBus => d_lmb_Sl_DBus,
      Sl_Ready => d_lmb_Sl_Ready(0),
      BRAM_Rst_A => dlmb_portb_BRAM_Rst,
      BRAM_Clk_A => dlmb_portb_BRAM_Clk,
      BRAM_EN_A => dlmb_portb_BRAM_EN,
      BRAM_WEN_A => dlmb_portb_BRAM_WEN,
      BRAM_Addr_A => dlmb_portb_BRAM_Addr,
      BRAM_Din_A => dlmb_portb_BRAM_Din,
      BRAM_Dout_A => dlmb_portb_BRAM_Dout
    );

  d_lmb : d_lmb_wrapper
    port map (
      LMB_Clk => sys_clk,
      SYS_Rst => main_reset,
      LMB_Rst => d_lmb_OPB_Rst,
      M_ABus => d_lmb_M_ABus,
      M_ReadStrobe => d_lmb_M_ReadStrobe,
      M_WriteStrobe => d_lmb_M_WriteStrobe,
      M_AddrStrobe => d_lmb_M_AddrStrobe,
      M_DBus => d_lmb_M_DBus,
      M_BE => d_lmb_M_BE,
      Sl_DBus => d_lmb_Sl_DBus,
      Sl_Ready => d_lmb_Sl_Ready(0 to 0),
      LMB_ABus => d_lmb_LMB_ABus,
      LMB_ReadStrobe => d_lmb_LMB_ReadStrobe,
      LMB_WriteStrobe => d_lmb_LMB_WriteStrobe,
      LMB_AddrStrobe => d_lmb_LMB_AddrStrobe,
      LMB_ReadDBus => d_lmb_LMB_ReadDBus,
      LMB_WriteDBus => d_lmb_LMB_WriteDBus,
      LMB_Ready => d_lmb_LMB_Ready,
      LMB_BE => d_lmb_LMB_BE
    );

  i_lmb : i_lmb_wrapper
    port map (
      LMB_Clk => sys_clk,
      SYS_Rst => main_reset,
      LMB_Rst => i_lmb_OPB_Rst,
      M_ABus => i_lmb_M_ABus,
      M_ReadStrobe => i_lmb_M_ReadStrobe,
      M_WriteStrobe => net_gnd0,
      M_AddrStrobe => i_lmb_M_AddrStrobe,
      M_DBus => net_gnd32,
      M_BE => net_gnd4,
      Sl_DBus => i_lmb_Sl_DBus,
      Sl_Ready => i_lmb_Sl_Ready(0 to 0),
      LMB_ABus => i_lmb_LMB_ABus,
      LMB_ReadStrobe => i_lmb_LMB_ReadStrobe,
      LMB_WriteStrobe => i_lmb_LMB_WriteStrobe,
      LMB_AddrStrobe => i_lmb_LMB_AddrStrobe,
      LMB_ReadDBus => i_lmb_LMB_ReadDBus,
      LMB_WriteDBus => i_lmb_LMB_WriteDBus,
      LMB_Ready => i_lmb_LMB_Ready,
      LMB_BE => i_lmb_LMB_BE
    );

  lmblmb_bram_i : lmblmb_bram_i_wrapper
    port map (
      BRAM_Rst_A => ilmb_porta_BRAM_Rst,
      BRAM_Clk_A => ilmb_porta_BRAM_Clk,
      BRAM_EN_A => ilmb_porta_BRAM_EN,
      BRAM_WEN_A => ilmb_porta_BRAM_WEN,
      BRAM_Addr_A => ilmb_porta_BRAM_Addr,
      BRAM_Din_A => ilmb_porta_BRAM_Din,
      BRAM_Dout_A => ilmb_porta_BRAM_Dout,
      BRAM_Rst_B => dlmb_portb_BRAM_Rst,
      BRAM_Clk_B => dlmb_portb_BRAM_Clk,
      BRAM_EN_B => dlmb_portb_BRAM_EN,
      BRAM_WEN_B => dlmb_portb_BRAM_WEN,
      BRAM_Addr_B => dlmb_portb_BRAM_Addr,
      BRAM_Din_B => dlmb_portb_BRAM_Din,
      BRAM_Dout_B => dlmb_portb_BRAM_Dout
    );

  from_mb_fsl_1 : from_mb_fsl_1_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => open,
      FSL_M_Clk => from_mb_fsl_1_FSL_M_Clk,
      FSL_M_Data => from_mb_fsl_1_FSL_M_Data,
      FSL_M_Control => from_mb_fsl_1_FSL_M_Control,
      FSL_M_Write => from_mb_fsl_1_FSL_M_Write,
      FSL_M_Full => from_mb_fsl_1_FSL_M_Full,
      FSL_S_Clk => from_mb_fsl_1_FSL_S_Clk,
      FSL_S_Data => from_mb_fsl_1_FSL_S_Data,
      FSL_S_Control => from_mb_fsl_1_FSL_S_Control,
      FSL_S_Read => from_mb_fsl_1_FSL_S_Read,
      FSL_S_Exists => from_mb_fsl_1_FSL_S_Exists
    );

  to_mb_fsl_1 : to_mb_fsl_1_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => to_mb_fsl_1_OPB_Rst,
      FSL_M_Clk => to_mb_fsl_1_FSL_M_Clk,
      FSL_M_Data => to_mb_fsl_1_FSL_M_Data,
      FSL_M_Control => to_mb_fsl_1_FSL_M_Control,
      FSL_M_Write => to_mb_fsl_1_FSL_M_Write,
      FSL_M_Full => to_mb_fsl_1_FSL_M_Full,
      FSL_S_Clk => to_mb_fsl_1_FSL_S_Clk,
      FSL_S_Data => to_mb_fsl_1_FSL_S_Data,
      FSL_S_Control => to_mb_fsl_1_FSL_S_Control,
      FSL_S_Read => to_mb_fsl_1_FSL_S_Read,
      FSL_S_Exists => to_mb_fsl_1_FSL_S_Exists
    );

  my_dct32 : my_dct32_wrapper
    port map (
      Clk => sys_clk,
      Reset => to_mb_fsl_1_OPB_Rst,
      FSL_S_CLK => from_mb_fsl_1_FSL_S_Clk,
      FSL_S_READ => from_mb_fsl_1_FSL_S_Read,
      FSL_S_DATA => from_mb_fsl_1_FSL_S_Data,
      FSL_S_CONTROL => from_mb_fsl_1_FSL_S_Control,
      FSL_S_EXISTS => from_mb_fsl_1_FSL_S_Exists,
      FSL_M_CLK => to_mb_fsl_1_FSL_M_Clk,
      FSL_M_WRITE => to_mb_fsl_1_FSL_M_Write,
      FSL_M_DATA => to_mb_fsl_1_FSL_M_Data,
      FSL_M_CONTROL => to_mb_fsl_1_FSL_M_Control,
      FSL_M_FULL => to_mb_fsl_1_FSL_M_Full
    );

  from_mb_fsl_2 : from_mb_fsl_2_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => open,
      FSL_M_Clk => from_mb_fsl_2_FSL_M_Clk,
      FSL_M_Data => from_mb_fsl_2_FSL_M_Data,
      FSL_M_Control => from_mb_fsl_2_FSL_M_Control,
      FSL_M_Write => from_mb_fsl_2_FSL_M_Write,
      FSL_M_Full => from_mb_fsl_2_FSL_M_Full,
      FSL_S_Clk => from_mb_fsl_2_FSL_S_Clk,
      FSL_S_Data => from_mb_fsl_2_FSL_S_Data,
      FSL_S_Control => from_mb_fsl_2_FSL_S_Control,
      FSL_S_Read => from_mb_fsl_2_FSL_S_Read,
      FSL_S_Exists => from_mb_fsl_2_FSL_S_Exists
    );

  to_mb_fsl_2 : to_mb_fsl_2_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => to_mb_fsl_2_OPB_Rst,
      FSL_M_Clk => to_mb_fsl_2_FSL_M_Clk,
      FSL_M_Data => to_mb_fsl_2_FSL_M_Data,
      FSL_M_Control => to_mb_fsl_2_FSL_M_Control,
      FSL_M_Write => to_mb_fsl_2_FSL_M_Write,
      FSL_M_Full => to_mb_fsl_2_FSL_M_Full,
      FSL_S_Clk => to_mb_fsl_2_FSL_S_Clk,
      FSL_S_Data => to_mb_fsl_2_FSL_S_Data,
      FSL_S_Control => to_mb_fsl_2_FSL_S_Control,
      FSL_S_Read => to_mb_fsl_2_FSL_S_Read,
      FSL_S_Exists => to_mb_fsl_2_FSL_S_Exists
    );

  my_imdct : my_imdct_wrapper
    port map (
      Clk => sys_clk,
      Reset => to_mb_fsl_2_OPB_Rst,
      FSL_S_CLK => from_mb_fsl_2_FSL_S_Clk,
      FSL_S_READ => from_mb_fsl_2_FSL_S_Read,
      FSL_S_DATA => from_mb_fsl_2_FSL_S_Data,
      FSL_S_CONTROL => from_mb_fsl_2_FSL_S_Control,
      FSL_S_EXISTS => from_mb_fsl_2_FSL_S_Exists,
      FSL_M_CLK => to_mb_fsl_2_FSL_M_Clk,
      FSL_M_WRITE => to_mb_fsl_2_FSL_M_Write,
      FSL_M_DATA => to_mb_fsl_2_FSL_M_Data,
      FSL_M_CONTROL => to_mb_fsl_2_FSL_M_Control,
      FSL_M_FULL => to_mb_fsl_2_FSL_M_Full
    );

  from_mb_fsl_3 : from_mb_fsl_3_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => open,
      FSL_M_Clk => from_mb_fsl_3_FSL_M_Clk,
      FSL_M_Data => from_mb_fsl_3_FSL_M_Data,
      FSL_M_Control => from_mb_fsl_3_FSL_M_Control,
      FSL_M_Write => from_mb_fsl_3_FSL_M_Write,
      FSL_M_Full => from_mb_fsl_3_FSL_M_Full,
      FSL_S_Clk => from_mb_fsl_3_FSL_S_Clk,
      FSL_S_Data => from_mb_fsl_3_FSL_S_Data,
      FSL_S_Control => from_mb_fsl_3_FSL_S_Control,
      FSL_S_Read => from_mb_fsl_3_FSL_S_Read,
      FSL_S_Exists => from_mb_fsl_3_FSL_S_Exists
    );

  to_mb_fsl_3 : to_mb_fsl_3_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => to_mb_fsl_3_OPB_Rst,
      FSL_M_Clk => to_mb_fsl_3_FSL_M_Clk,
      FSL_M_Data => to_mb_fsl_3_FSL_M_Data,
      FSL_M_Control => to_mb_fsl_3_FSL_M_Control,
      FSL_M_Write => to_mb_fsl_3_FSL_M_Write,
      FSL_M_Full => to_mb_fsl_3_FSL_M_Full,
      FSL_S_Clk => to_mb_fsl_3_FSL_S_Clk,
      FSL_S_Data => to_mb_fsl_3_FSL_S_Data,
      FSL_S_Control => to_mb_fsl_3_FSL_S_Control,
      FSL_S_Read => to_mb_fsl_3_FSL_S_Read,
      FSL_S_Exists => to_mb_fsl_3_FSL_S_Exists
    );

  my_mp3_mul_28 : my_mp3_mul_28_wrapper
    port map (
      Clk => sys_clk,
      Reset => to_mb_fsl_3_OPB_Rst,
      FSL_S_CLK => from_mb_fsl_3_FSL_S_Clk,
      FSL_S_READ => from_mb_fsl_3_FSL_S_Read,
      FSL_S_DATA => from_mb_fsl_3_FSL_S_Data,
      FSL_S_CONTROL => from_mb_fsl_3_FSL_S_Control,
      FSL_S_EXISTS => from_mb_fsl_3_FSL_S_Exists,
      FSL_M_CLK => to_mb_fsl_3_FSL_M_Clk,
      FSL_M_WRITE => to_mb_fsl_3_FSL_M_Write,
      FSL_M_DATA => to_mb_fsl_3_FSL_M_Data,
      FSL_M_CONTROL => to_mb_fsl_3_FSL_M_Control,
      FSL_M_FULL => to_mb_fsl_3_FSL_M_Full
    );

  from_mb_fsl_4 : from_mb_fsl_4_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => open,
      FSL_M_Clk => from_mb_fsl_4_FSL_M_Clk,
      FSL_M_Data => from_mb_fsl_4_FSL_M_Data,
      FSL_M_Control => from_mb_fsl_4_FSL_M_Control,
      FSL_M_Write => from_mb_fsl_4_FSL_M_Write,
      FSL_M_Full => from_mb_fsl_4_FSL_M_Full,
      FSL_S_Clk => from_mb_fsl_4_FSL_S_Clk,
      FSL_S_Data => from_mb_fsl_4_FSL_S_Data,
      FSL_S_Control => from_mb_fsl_4_FSL_S_Control,
      FSL_S_Read => from_mb_fsl_4_FSL_S_Read,
      FSL_S_Exists => from_mb_fsl_4_FSL_S_Exists
    );

  to_mb_fsl_4 : to_mb_fsl_4_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => to_mb_fsl_4_OPB_Rst,
      FSL_M_Clk => to_mb_fsl_4_FSL_M_Clk,
      FSL_M_Data => to_mb_fsl_4_FSL_M_Data,
      FSL_M_Control => to_mb_fsl_4_FSL_M_Control,
      FSL_M_Write => to_mb_fsl_4_FSL_M_Write,
      FSL_M_Full => to_mb_fsl_4_FSL_M_Full,
      FSL_S_Clk => to_mb_fsl_4_FSL_S_Clk,
      FSL_S_Data => to_mb_fsl_4_FSL_S_Data,
      FSL_S_Control => to_mb_fsl_4_FSL_S_Control,
      FSL_S_Read => to_mb_fsl_4_FSL_S_Read,
      FSL_S_Exists => to_mb_fsl_4_FSL_S_Exists
    );

  my_mp3_mul_16 : my_mp3_mul_16_wrapper
    port map (
      Clk => sys_clk,
      Reset => to_mb_fsl_4_OPB_Rst,
      FSL_S_CLK => from_mb_fsl_4_FSL_S_Clk,
      FSL_S_READ => from_mb_fsl_4_FSL_S_Read,
      FSL_S_DATA => from_mb_fsl_4_FSL_S_Data,
      FSL_S_CONTROL => from_mb_fsl_4_FSL_S_Control,
      FSL_S_EXISTS => from_mb_fsl_4_FSL_S_Exists,
      FSL_M_CLK => to_mb_fsl_4_FSL_M_Clk,
      FSL_M_WRITE => to_mb_fsl_4_FSL_M_Write,
      FSL_M_DATA => to_mb_fsl_4_FSL_M_Data,
      FSL_M_CONTROL => to_mb_fsl_4_FSL_M_Control,
      FSL_M_FULL => to_mb_fsl_4_FSL_M_Full
    );

  ac_97_play_buffer : ac_97_play_buffer_wrapper
    port map (
      FSL_Clk => sys_clk,
      SYS_Rst => main_reset,
      FSL_Rst => open,
      FSL_M_Clk => ac_97_play_buffer_FSL_M_Clk,
      FSL_M_Data => ac_97_play_buffer_FSL_M_Data,
      FSL_M_Control => ac_97_play_buffer_FSL_M_Control,
      FSL_M_Write => ac_97_play_buffer_FSL_M_Write,
      FSL_M_Full => ac_97_play_buffer_FSL_M_Full,
      FSL_S_Clk => ac_97_play_buffer_FSL_S_Clk,
      FSL_S_Data => ac_97_play_buffer_FSL_S_Data,
      FSL_S_Control => ac_97_play_buffer_FSL_S_Control,
      FSL_S_Read => ac_97_play_buffer_FSL_S_Read,
      FSL_S_Exists => ac_97_play_buffer_FSL_S_Exists
    );

  opb_timer_0 : opb_timer_0_wrapper
    port map (
      OPB_Clk => sys_clk,
      OPB_Rst => myopb_bus_OPB_Rst,
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_DBus => myopb_bus_OPB_DBus,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      TC_DBus => myopb_bus_Sl_DBus(224 to 255),
      TC_errAck => myopb_bus_Sl_errAck(7),
      TC_retry => myopb_bus_Sl_retry(7),
      TC_toutSup => myopb_bus_Sl_toutSup(7),
      TC_xferAck => myopb_bus_Sl_xferAck(7),
      CaptureTrig0 => net_gnd0,
      CaptureTrig1 => net_gnd0,
      GenerateOut0 => open,
      GenerateOut1 => open,
      PWM0 => open,
      Interrupt => timer_to_intc(0),
      Freeze => net_gnd0
    );

  opb_intc_0 : opb_intc_0_wrapper
    port map (
      OPB_Clk => sys_clk,
      Intr => timer_to_intc(0 downto 0),
      OPB_Rst => myopb_bus_OPB_Rst,
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      OPB_DBus => myopb_bus_OPB_DBus,
      IntC_DBus => myopb_bus_Sl_DBus(256 to 287),
      IntC_errAck => myopb_bus_Sl_errAck(8),
      IntC_retry => myopb_bus_Sl_retry(8),
      IntC_toutSup => myopb_bus_Sl_toutSup(8),
      IntC_xferAck => myopb_bus_Sl_xferAck(8),
      Irq => intc_to_myblaze
    );

  rs232 : rs232_wrapper
    port map (
      OPB_Clk => sys_clk,
      OPB_Rst => myopb_bus_OPB_Rst,
      Interrupt => open,
      OPB_ABus => myopb_bus_OPB_ABus,
      OPB_BE => myopb_bus_OPB_BE,
      OPB_RNW => myopb_bus_OPB_RNW,
      OPB_select => myopb_bus_OPB_select,
      OPB_seqAddr => myopb_bus_OPB_seqAddr,
      OPB_DBus => myopb_bus_OPB_DBus,
      UART_DBus => myopb_bus_Sl_DBus(288 to 319),
      UART_errAck => myopb_bus_Sl_errAck(9),
      UART_retry => myopb_bus_Sl_retry(9),
      UART_toutSup => myopb_bus_Sl_toutSup(9),
      UART_xferAck => myopb_bus_Sl_xferAck(9),
      RX => RS232_RX_IBUF,
      TX => RS232_TX_OBUF
    );

  bufgp_0 : BUFGP
    port map (
      I => clk_50,
      O => clk_50_BUFGP
    );

  ibuf_1 : IBUF
    port map (
      I => extend_dcm_reset,
      O => extend_dcm_reset_IBUF
    );

  bufgp_2 : BUFGP
    port map (
      I => ac97_bit_clock_p,
      O => AC97_BIT_CLOCK_P_BUFGP
    );

  obuf_3 : OBUF
    port map (
      I => AC97_SYNCH_P_OBUF,
      O => ac97_synch_p
    );

  ibuf_4 : IBUF
    port map (
      I => ac97_sdata_in,
      O => AC97_SDATA_IN_IBUF
    );

  obuf_5 : OBUF
    port map (
      I => AC97_SDATA_OUT_OBUF,
      O => ac97_sdata_out
    );

  obuf_6 : OBUF
    port map (
      I => ZBT1_OE_N_OBUF,
      O => zbt1_oe_n
    );

  obuf_7 : OBUF
    port map (
      I => ZBT1_ADV_LD_N_OBUF,
      O => zbt1_adv_ld_n
    );

  obuf_8 : OBUF
    port map (
      I => ZBT1_CE1_N_OBUF,
      O => zbt1_ce1_n
    );

  obuf_9 : OBUF
    port map (
      I => ZBT1_RW_N_OBUF,
      O => zbt1_rw_n
    );

  obuf_10 : OBUF
    port map (
      I => ZBT1_CKE_N_OBUF,
      O => zbt1_cke_n
    );

  obuf_11 : OBUF
    port map (
      I => ZBT1_A_OBUF(0),
      O => zbt1_a(0)
    );

  obuf_12 : OBUF
    port map (
      I => ZBT1_A_OBUF(1),
      O => zbt1_a(1)
    );

  obuf_13 : OBUF
    port map (
      I => ZBT1_A_OBUF(2),
      O => zbt1_a(2)
    );

  obuf_14 : OBUF
    port map (
      I => ZBT1_A_OBUF(3),
      O => zbt1_a(3)
    );

  obuf_15 : OBUF
    port map (
      I => ZBT1_A_OBUF(4),
      O => zbt1_a(4)
    );

  obuf_16 : OBUF
    port map (
      I => ZBT1_A_OBUF(5),
      O => zbt1_a(5)
    );

  obuf_17 : OBUF
    port map (
      I => ZBT1_A_OBUF(6),
      O => zbt1_a(6)
    );

  obuf_18 : OBUF
    port map (
      I => ZBT1_A_OBUF(7),
      O => zbt1_a(7)
    );

  obuf_19 : OBUF
    port map (
      I => ZBT1_A_OBUF(8),
      O => zbt1_a(8)
    );

  obuf_20 : OBUF
    port map (
      I => ZBT1_A_OBUF(9),
      O => zbt1_a(9)
    );

  obuf_21 : OBUF
    port map (
      I => ZBT1_A_OBUF(10),
      O => zbt1_a(10)
    );

  obuf_22 : OBUF
    port map (
      I => ZBT1_A_OBUF(11),
      O => zbt1_a(11)
    );

  obuf_23 : OBUF
    port map (
      I => ZBT1_A_OBUF(12),
      O => zbt1_a(12)
    );

  obuf_24 : OBUF
    port map (
      I => ZBT1_A_OBUF(13),
      O => zbt1_a(13)
    );

  obuf_25 : OBUF
    port map (
      I => ZBT1_A_OBUF(14),
      O => zbt1_a(14)
    );

  obuf_26 : OBUF
    port map (
      I => ZBT1_A_OBUF(15),
      O => zbt1_a(15)
    );

  obuf_27 : OBUF
    port map (
      I => ZBT1_A_OBUF(16),
      O => zbt1_a(16)
    );

  obuf_28 : OBUF
    port map (
      I => ZBT1_A_OBUF(17),
      O => zbt1_a(17)
    );

  obuf_29 : OBUF
    port map (
      I => ZBT1_A_OBUF(18),
      O => zbt1_a(18)
    );

  obuf_30 : OBUF
    port map (
      I => ZBT1_BW_N_OBUF(1),
      O => zbt1_bw_n(1)
    );

  obuf_31 : OBUF
    port map (
      I => ZBT1_BW_N_OBUF(2),
      O => zbt1_bw_n(2)
    );

  obuf_32 : OBUF
    port map (
      I => ZBT1_BW_N_OBUF(3),
      O => zbt1_bw_n(3)
    );

  obuf_33 : OBUF
    port map (
      I => ZBT1_BW_N_OBUF(4),
      O => zbt1_bw_n(4)
    );

  iobuf_34 : IOBUF
    port map (
      I => zbt1_io_O(0),
      IO => zbt1_io(0),
      O => zbt1_io_I(0),
      T => zbt1_io_T
    );

  iobuf_35 : IOBUF
    port map (
      I => zbt1_io_O(1),
      IO => zbt1_io(1),
      O => zbt1_io_I(1),
      T => zbt1_io_T
    );

  iobuf_36 : IOBUF
    port map (
      I => zbt1_io_O(2),
      IO => zbt1_io(2),
      O => zbt1_io_I(2),
      T => zbt1_io_T
    );

  iobuf_37 : IOBUF
    port map (
      I => zbt1_io_O(3),
      IO => zbt1_io(3),
      O => zbt1_io_I(3),
      T => zbt1_io_T
    );

  iobuf_38 : IOBUF
    port map (
      I => zbt1_io_O(4),
      IO => zbt1_io(4),
      O => zbt1_io_I(4),
      T => zbt1_io_T
    );

  iobuf_39 : IOBUF
    port map (
      I => zbt1_io_O(5),
      IO => zbt1_io(5),
      O => zbt1_io_I(5),
      T => zbt1_io_T
    );

  iobuf_40 : IOBUF
    port map (
      I => zbt1_io_O(6),
      IO => zbt1_io(6),
      O => zbt1_io_I(6),
      T => zbt1_io_T
    );

  iobuf_41 : IOBUF
    port map (
      I => zbt1_io_O(7),
      IO => zbt1_io(7),
      O => zbt1_io_I(7),
      T => zbt1_io_T
    );

  iobuf_42 : IOBUF
    port map (
      I => zbt1_io_O(8),
      IO => zbt1_io(8),
      O => zbt1_io_I(8),
      T => zbt1_io_T
    );

  iobuf_43 : IOBUF
    port map (
      I => zbt1_io_O(9),
      IO => zbt1_io(9),
      O => zbt1_io_I(9),
      T => zbt1_io_T
    );

  iobuf_44 : IOBUF
    port map (
      I => zbt1_io_O(10),
      IO => zbt1_io(10),
      O => zbt1_io_I(10),
      T => zbt1_io_T
    );

  iobuf_45 : IOBUF
    port map (
      I => zbt1_io_O(11),
      IO => zbt1_io(11),
      O => zbt1_io_I(11),
      T => zbt1_io_T
    );

  iobuf_46 : IOBUF
    port map (
      I => zbt1_io_O(12),
      IO => zbt1_io(12),
      O => zbt1_io_I(12),
      T => zbt1_io_T
    );

  iobuf_47 : IOBUF
    port map (
      I => zbt1_io_O(13),
      IO => zbt1_io(13),
      O => zbt1_io_I(13),
      T => zbt1_io_T
    );

  iobuf_48 : IOBUF
    port map (
      I => zbt1_io_O(14),
      IO => zbt1_io(14),
      O => zbt1_io_I(14),
      T => zbt1_io_T
    );

  iobuf_49 : IOBUF
    port map (
      I => zbt1_io_O(15),
      IO => zbt1_io(15),
      O => zbt1_io_I(15),
      T => zbt1_io_T
    );

  iobuf_50 : IOBUF
    port map (
      I => zbt1_io_O(16),
      IO => zbt1_io(16),
      O => zbt1_io_I(16),
      T => zbt1_io_T
    );

  iobuf_51 : IOBUF
    port map (
      I => zbt1_io_O(17),
      IO => zbt1_io(17),
      O => zbt1_io_I(17),
      T => zbt1_io_T
    );

  iobuf_52 : IOBUF
    port map (
      I => zbt1_io_O(18),
      IO => zbt1_io(18),
      O => zbt1_io_I(18),
      T => zbt1_io_T
    );

  iobuf_53 : IOBUF
    port map (
      I => zbt1_io_O(19),
      IO => zbt1_io(19),
      O => zbt1_io_I(19),
      T => zbt1_io_T
    );

  iobuf_54 : IOBUF
    port map (
      I => zbt1_io_O(20),
      IO => zbt1_io(20),
      O => zbt1_io_I(20),
      T => zbt1_io_T
    );

  iobuf_55 : IOBUF
    port map (
      I => zbt1_io_O(21),
      IO => zbt1_io(21),
      O => zbt1_io_I(21),
      T => zbt1_io_T
    );

  iobuf_56 : IOBUF
    port map (
      I => zbt1_io_O(22),
      IO => zbt1_io(22),
      O => zbt1_io_I(22),
      T => zbt1_io_T
    );

  iobuf_57 : IOBUF
    port map (
      I => zbt1_io_O(23),
      IO => zbt1_io(23),
      O => zbt1_io_I(23),
      T => zbt1_io_T
    );

  iobuf_58 : IOBUF
    port map (
      I => zbt1_io_O(24),
      IO => zbt1_io(24),
      O => zbt1_io_I(24),
      T => zbt1_io_T
    );

  iobuf_59 : IOBUF
    port map (
      I => zbt1_io_O(25),
      IO => zbt1_io(25),
      O => zbt1_io_I(25),
      T => zbt1_io_T
    );

  iobuf_60 : IOBUF
    port map (
      I => zbt1_io_O(26),
      IO => zbt1_io(26),
      O => zbt1_io_I(26),
      T => zbt1_io_T
    );

  iobuf_61 : IOBUF
    port map (
      I => zbt1_io_O(27),
      IO => zbt1_io(27),
      O => zbt1_io_I(27),
      T => zbt1_io_T
    );

  iobuf_62 : IOBUF
    port map (
      I => zbt1_io_O(28),
      IO => zbt1_io(28),
      O => zbt1_io_I(28),
      T => zbt1_io_T
    );

  iobuf_63 : IOBUF
    port map (
      I => zbt1_io_O(29),
      IO => zbt1_io(29),
      O => zbt1_io_I(29),
      T => zbt1_io_T
    );

  iobuf_64 : IOBUF
    port map (
      I => zbt1_io_O(30),
      IO => zbt1_io(30),
      O => zbt1_io_I(30),
      T => zbt1_io_T
    );

  iobuf_65 : IOBUF
    port map (
      I => zbt1_io_O(31),
      IO => zbt1_io(31),
      O => zbt1_io_I(31),
      T => zbt1_io_T
    );

  obuf_66 : OBUF
    port map (
      I => ZBT2_OE_N_OBUF,
      O => zbt2_oe_n
    );

  obuf_67 : OBUF
    port map (
      I => ZBT2_ADV_LD_N_OBUF,
      O => zbt2_adv_ld_n
    );

  obuf_68 : OBUF
    port map (
      I => ZBT2_CE1_N_OBUF,
      O => zbt2_ce1_n
    );

  obuf_69 : OBUF
    port map (
      I => ZBT2_RW_N_OBUF,
      O => zbt2_rw_n
    );

  obuf_70 : OBUF
    port map (
      I => ZBT2_CKE_N_OBUF,
      O => zbt2_cke_n
    );

  obuf_71 : OBUF
    port map (
      I => ZBT2_A_OBUF(0),
      O => zbt2_a(0)
    );

  obuf_72 : OBUF
    port map (
      I => ZBT2_A_OBUF(1),
      O => zbt2_a(1)
    );

  obuf_73 : OBUF
    port map (
      I => ZBT2_A_OBUF(2),
      O => zbt2_a(2)
    );

  obuf_74 : OBUF
    port map (
      I => ZBT2_A_OBUF(3),
      O => zbt2_a(3)
    );

  obuf_75 : OBUF
    port map (
      I => ZBT2_A_OBUF(4),
      O => zbt2_a(4)
    );

  obuf_76 : OBUF
    port map (
      I => ZBT2_A_OBUF(5),
      O => zbt2_a(5)
    );

  obuf_77 : OBUF
    port map (
      I => ZBT2_A_OBUF(6),
      O => zbt2_a(6)
    );

  obuf_78 : OBUF
    port map (
      I => ZBT2_A_OBUF(7),
      O => zbt2_a(7)
    );

  obuf_79 : OBUF
    port map (
      I => ZBT2_A_OBUF(8),
      O => zbt2_a(8)
    );

  obuf_80 : OBUF
    port map (
      I => ZBT2_A_OBUF(9),
      O => zbt2_a(9)
    );

  obuf_81 : OBUF
    port map (
      I => ZBT2_A_OBUF(10),
      O => zbt2_a(10)
    );

  obuf_82 : OBUF
    port map (
      I => ZBT2_A_OBUF(11),
      O => zbt2_a(11)
    );

  obuf_83 : OBUF
    port map (
      I => ZBT2_A_OBUF(12),
      O => zbt2_a(12)
    );

  obuf_84 : OBUF
    port map (
      I => ZBT2_A_OBUF(13),
      O => zbt2_a(13)
    );

  obuf_85 : OBUF
    port map (
      I => ZBT2_A_OBUF(14),
      O => zbt2_a(14)
    );

  obuf_86 : OBUF
    port map (
      I => ZBT2_A_OBUF(15),
      O => zbt2_a(15)
    );

  obuf_87 : OBUF
    port map (
      I => ZBT2_A_OBUF(16),
      O => zbt2_a(16)
    );

  obuf_88 : OBUF
    port map (
      I => ZBT2_A_OBUF(17),
      O => zbt2_a(17)
    );

  obuf_89 : OBUF
    port map (
      I => ZBT2_A_OBUF(18),
      O => zbt2_a(18)
    );

  obuf_90 : OBUF
    port map (
      I => ZBT2_BW_N_OBUF(1),
      O => zbt2_bw_n(1)
    );

  obuf_91 : OBUF
    port map (
      I => ZBT2_BW_N_OBUF(2),
      O => zbt2_bw_n(2)
    );

  obuf_92 : OBUF
    port map (
      I => ZBT2_BW_N_OBUF(3),
      O => zbt2_bw_n(3)
    );

  obuf_93 : OBUF
    port map (
      I => ZBT2_BW_N_OBUF(4),
      O => zbt2_bw_n(4)
    );

  iobuf_94 : IOBUF
    port map (
      I => zbt2_io_O(0),
      IO => zbt2_io(0),
      O => zbt2_io_I(0),
      T => zbt2_io_T
    );

  iobuf_95 : IOBUF
    port map (
      I => zbt2_io_O(1),
      IO => zbt2_io(1),
      O => zbt2_io_I(1),
      T => zbt2_io_T
    );

  iobuf_96 : IOBUF
    port map (
      I => zbt2_io_O(2),
      IO => zbt2_io(2),
      O => zbt2_io_I(2),
      T => zbt2_io_T
    );

  iobuf_97 : IOBUF
    port map (
      I => zbt2_io_O(3),
      IO => zbt2_io(3),
      O => zbt2_io_I(3),
      T => zbt2_io_T
    );

  iobuf_98 : IOBUF
    port map (
      I => zbt2_io_O(4),
      IO => zbt2_io(4),
      O => zbt2_io_I(4),
      T => zbt2_io_T
    );

  iobuf_99 : IOBUF
    port map (
      I => zbt2_io_O(5),
      IO => zbt2_io(5),
      O => zbt2_io_I(5),
      T => zbt2_io_T
    );

  iobuf_100 : IOBUF
    port map (
      I => zbt2_io_O(6),
      IO => zbt2_io(6),
      O => zbt2_io_I(6),
      T => zbt2_io_T
    );

  iobuf_101 : IOBUF
    port map (
      I => zbt2_io_O(7),
      IO => zbt2_io(7),
      O => zbt2_io_I(7),
      T => zbt2_io_T
    );

  iobuf_102 : IOBUF
    port map (
      I => zbt2_io_O(8),
      IO => zbt2_io(8),
      O => zbt2_io_I(8),
      T => zbt2_io_T
    );

  iobuf_103 : IOBUF
    port map (
      I => zbt2_io_O(9),
      IO => zbt2_io(9),
      O => zbt2_io_I(9),
      T => zbt2_io_T
    );

  iobuf_104 : IOBUF
    port map (
      I => zbt2_io_O(10),
      IO => zbt2_io(10),
      O => zbt2_io_I(10),
      T => zbt2_io_T
    );

  iobuf_105 : IOBUF
    port map (
      I => zbt2_io_O(11),
      IO => zbt2_io(11),
      O => zbt2_io_I(11),
      T => zbt2_io_T
    );

  iobuf_106 : IOBUF
    port map (
      I => zbt2_io_O(12),
      IO => zbt2_io(12),
      O => zbt2_io_I(12),
      T => zbt2_io_T
    );

  iobuf_107 : IOBUF
    port map (
      I => zbt2_io_O(13),
      IO => zbt2_io(13),
      O => zbt2_io_I(13),
      T => zbt2_io_T
    );

  iobuf_108 : IOBUF
    port map (
      I => zbt2_io_O(14),
      IO => zbt2_io(14),
      O => zbt2_io_I(14),
      T => zbt2_io_T
    );

  iobuf_109 : IOBUF
    port map (
      I => zbt2_io_O(15),
      IO => zbt2_io(15),
      O => zbt2_io_I(15),
      T => zbt2_io_T
    );

  iobuf_110 : IOBUF
    port map (
      I => zbt2_io_O(16),
      IO => zbt2_io(16),
      O => zbt2_io_I(16),
      T => zbt2_io_T
    );

  iobuf_111 : IOBUF
    port map (
      I => zbt2_io_O(17),
      IO => zbt2_io(17),
      O => zbt2_io_I(17),
      T => zbt2_io_T
    );

  iobuf_112 : IOBUF
    port map (
      I => zbt2_io_O(18),
      IO => zbt2_io(18),
      O => zbt2_io_I(18),
      T => zbt2_io_T
    );

  iobuf_113 : IOBUF
    port map (
      I => zbt2_io_O(19),
      IO => zbt2_io(19),
      O => zbt2_io_I(19),
      T => zbt2_io_T
    );

  iobuf_114 : IOBUF
    port map (
      I => zbt2_io_O(20),
      IO => zbt2_io(20),
      O => zbt2_io_I(20),
      T => zbt2_io_T
    );

  iobuf_115 : IOBUF
    port map (
      I => zbt2_io_O(21),
      IO => zbt2_io(21),
      O => zbt2_io_I(21),
      T => zbt2_io_T
    );

  iobuf_116 : IOBUF
    port map (
      I => zbt2_io_O(22),
      IO => zbt2_io(22),
      O => zbt2_io_I(22),
      T => zbt2_io_T
    );

  iobuf_117 : IOBUF
    port map (
      I => zbt2_io_O(23),
      IO => zbt2_io(23),
      O => zbt2_io_I(23),
      T => zbt2_io_T
    );

  iobuf_118 : IOBUF
    port map (
      I => zbt2_io_O(24),
      IO => zbt2_io(24),
      O => zbt2_io_I(24),
      T => zbt2_io_T
    );

  iobuf_119 : IOBUF
    port map (
      I => zbt2_io_O(25),
      IO => zbt2_io(25),
      O => zbt2_io_I(25),
      T => zbt2_io_T
    );

  iobuf_120 : IOBUF
    port map (
      I => zbt2_io_O(26),
      IO => zbt2_io(26),
      O => zbt2_io_I(26),
      T => zbt2_io_T
    );

  iobuf_121 : IOBUF
    port map (
      I => zbt2_io_O(27),
      IO => zbt2_io(27),
      O => zbt2_io_I(27),
      T => zbt2_io_T
    );

  iobuf_122 : IOBUF
    port map (
      I => zbt2_io_O(28),
      IO => zbt2_io(28),
      O => zbt2_io_I(28),
      T => zbt2_io_T
    );

  iobuf_123 : IOBUF
    port map (
      I => zbt2_io_O(29),
      IO => zbt2_io(29),
      O => zbt2_io_I(29),
      T => zbt2_io_T
    );

  iobuf_124 : IOBUF
    port map (
      I => zbt2_io_O(30),
      IO => zbt2_io(30),
      O => zbt2_io_I(30),
      T => zbt2_io_T
    );

  iobuf_125 : IOBUF
    port map (
      I => zbt2_io_O(31),
      IO => zbt2_io(31),
      O => zbt2_io_I(31),
      T => zbt2_io_T
    );

  obuf_126 : OBUF
    port map (
      I => zbt_Clk_100,
      O => zbt1_clk
    );

  obuf_127 : OBUF
    port map (
      I => zbt_Clk_100,
      O => zbt2_clk
    );

  obuf_128 : OBUF
    port map (
      I => ZBT_Clk_FBOut_OBUF,
      O => zbt_clk_fbout
    );

  bufgp_129 : BUFGP
    port map (
      I => zbt_clk_fb,
      O => ZBT_CLK_FB_BUFGP
    );

  obuf_130 : OBUF
    port map (
      I => net_vcc0,
      O => startup
    );

  ibuf_131 : IBUF
    port map (
      I => RS232_RX,
      O => RS232_RX_IBUF
    );

  obuf_132 : OBUF
    port map (
      I => RS232_TX_OBUF,
      O => RS232_TX
    );

  obuf_133 : OBUF
    port map (
      I => net_gnd0,
      O => RS232_req_to_send_pin
    );

  iobuf_134 : IOBUF
    port map (
      I => opb_gpio_0_GPIO_IO_O(0),
      IO => opb_gpio_0_GPIO_IO(0),
      O => opb_gpio_0_GPIO_IO_I(0),
      T => opb_gpio_0_GPIO_IO_T(0)
    );

  iobuf_135 : IOBUF
    port map (
      I => opb_gpio_0_GPIO_IO_O(1),
      IO => opb_gpio_0_GPIO_IO(1),
      O => opb_gpio_0_GPIO_IO_I(1),
      T => opb_gpio_0_GPIO_IO_T(1)
    );

  iobuf_136 : IOBUF
    port map (
      I => opb_gpio_0_GPIO_IO_O(2),
      IO => opb_gpio_0_GPIO_IO(2),
      O => opb_gpio_0_GPIO_IO_I(2),
      T => opb_gpio_0_GPIO_IO_T(2)
    );

  iobuf_137 : IOBUF
    port map (
      I => opb_gpio_0_GPIO_IO_O(3),
      IO => opb_gpio_0_GPIO_IO(3),
      O => opb_gpio_0_GPIO_IO_I(3),
      T => opb_gpio_0_GPIO_IO_T(3)
    );

  ibuf_138 : IBUF
    port map (
      I => volume_buttons_0_PB_CLOCK_P,
      O => volume_buttons_0_PB_CLOCK_P_IBUF
    );

  ibuf_139 : IBUF
    port map (
      I => volume_buttons_0_PB_DATA_P,
      O => volume_buttons_0_PB_DATA_P_IBUF
    );

end architecture STRUCTURE;
