library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library unisim;
use unisim.all;

ENTITY dcm_block is
PORT (
     CLKIN      : in std_logic;
     CLKFB      : in std_logic;
     DSSEN      : in std_logic;
     PSINCDEC   : in std_logic;
     PSEN       : in std_logic;
     PSCLK      : in std_logic;
     RST        : in std_logic;
     CLK0       : out std_logic;
     CLK90      : out std_logic;
     CLK180     : out std_logic;
     CLK270     : out std_logic;
     CLK2X      : out std_logic;
     CLK2X180   : out std_logic;
     CLKDV      : out std_logic;
     CLKFX      : out std_logic;
     CLKFX180   : out std_logic;
     LOCKED     : out std_logic;
     PSDONE     : out std_logic;
     STATUS     : out std_logic_vector(7 downto 0);
     RESET_IN   : in  std_logic;
     RESET_OUT  : out  std_logic;
     FPGA_DONE  : in  std_logic
     );

END ENTITY dcm_block;

ARCHITECTURE IMP of dcm_block is

COMPONENT DCM
PORT (
      CLKIN    : in  std_logic;
      CLKFB    : in  std_logic;
      DSSEN    : in  std_logic;
      PSINCDEC : in  std_logic;
      PSEN     : in  std_logic;
      PSCLK    : in  std_logic;
      RST      : in  std_logic;
      CLK0     : out std_logic;
      CLK90    : out std_logic;
      CLK180   : out std_logic;
      CLK270   : out std_logic;
      CLK2X    : out std_logic;
      CLK2X180 : out std_logic;
      CLKDV    : out std_logic;
      CLKFX    : out std_logic;
      CLKFX180 : out std_logic;
      LOCKED   : out std_logic;
      PSDONE   : out std_logic;
      STATUS   : out std_logic_vector(7 downto 0)
      );
END COMPONENT;

--------------------------------------------------------------------------------
-- internal signals
--------------------------------------------------------------------------------
    SIGNAL not_ready : STD_LOGIC;
    SIGNAL net_gnd0 : STD_LOGIC;
    SIGNAL enable_reset_counter : STD_LOGIC;
    SIGNAL dcm_reset_counter : STD_LOGIC_VECTOR(15 DOWNTO 0);
    SIGNAL terminal_count    : STD_LOGIC;
    SIGNAL dcm_lock          : STD_LOGIC;
    SIGNAL reset             : STD_LOGIC;
    SIGNAL dcm_reset         : STD_LOGIC;
    
BEGIN

--------------------------------------------------------------------------------
-- Power assignments
net_gnd0 <= '0';
--------------------------------------------------------------------------------
-- Constant assignments
--------------------------------------------------------------------------------
-- Top-level port assignments
LOCKED <= dcm_lock;
RESET_OUT <= reset;
--------------------------------------------------------------------------------
-- Lower-level assignments
--------------------------------------------------------------------------------
-- Tri-state assignments

--------------------------------------------------------------------------------
  
  DCM_INST : DCM
  PORT MAP(
        CLKIN => CLKIN,
	CLKFB => CLKFB,
	CLK0 => CLK0,
	CLK2X => CLK2X,
	DSSEN => DSSEN,
	PSINCDEC => PSINCDEC,
	PSEN => PSEN,
	PSCLK => PSCLK,
	RST => dcm_reset,
	CLK90 => CLK90,
	CLK180 => CLK180,
	CLK270 => CLK270,
	CLK2X180 => CLK2X180,
	CLKDV => CLKDV,
	CLKFX => CLKFX,
	CLKFX180 => CLKFX180,
	LOCKED => dcm_lock,
	PSDONE => PSDONE,
	STATUS => STATUS
        );
  
  -- Create a reset pulse for the dcm triggered by FPGA_DONE
  enable_reset_counter <= not(terminal_count) and FPGA_DONE;
  
  Counter : PROCESS(CLKIN) IS
  BEGIN  -- process Counter
    IF CLKIN'event and CLKIN = '1' THEN  -- rising clock edge
       IF (enable_reset_counter = '1') THEN
        dcm_reset_counter <= STD_LOGIC_VECTOR(UNSIGNED(dcm_reset_counter) + 1);
       END IF;
    END IF;
  END PROCESS Counter;
  
  terminal_count <= '1' WHEN dcm_reset_counter = X"FFFF" ELSE '0';
  dcm_reset <= not(dcm_reset_counter(15));
  not_ready <= not(dcm_reset_counter(15)) or not(dcm_lock);

  reset <= RESET_IN or not_ready;

END ARCHITECTURE IMP;
