-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- dual_register.vhd - Entity and architecture
--
--
-------------------------------------------------------------------------------
-- Filename:        dual_register.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              dual_register.vhd
--
-------------------------------------------------------------------------------
-- Author:          
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--     2003-11-07    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

entity dual_register is
  
  generic (
    C_SIZE : integer);

  port (
    Clk    : in  std_logic;
    RAdr   : in  std_logic_vector(0 to 3);
    WAdr   : in  std_logic_vector(0 to 3);
    WE     : in  std_logic;
    D_In   : in  std_logic_vector(0 to C_SIZE-1);
    WD_Out : out std_logic_vector(0 to C_SIZE-1);
    RD_Out : out std_logic_vector(0 to C_SIZE-1)
    );

end entity dual_register;

library unisim;
use unisim.all;

architecture IMP of dual_register is
  component RAM16x1D is
    port (
      WE    : in  std_logic;
      D     : in  std_logic;
      WClk  : in  std_logic;
      A0    : in  std_logic;
      A1    : in  std_logic;
      A2    : in  std_logic;
      A3    : in  std_logic;
      DPRA0 : in  std_logic;
      DPRA1 : in  std_logic;
      DPRA2 : in  std_logic;
      DPRA3 : in  std_logic;
      SPO   : out std_logic;
      DPO   : out std_logic);
  end component RAM16x1D;
begin  -- architecture IMP

  The_Register : for I in 0 to C_SIZE-1 generate
    RAM16x1D_I1 : RAM16x1D
      port map (
        WE    => WE,                    -- [in  std_logic]
        D     => D_In(I),               -- [in  std_logic]
        WClk  => Clk,                   -- [in  std_logic]
        A0    => WAdr(0),               -- [in  std_logic]
        A1    => WAdr(1),               -- [in  std_logic]
        A2    => WAdr(2),               -- [in  std_logic]
        A3    => WAdr(3),               -- [in  std_logic]
        DPRA0 => RAdr(0),               -- [in  std_logic]
        DPRA1 => RAdr(1),               -- [in  std_logic]
        DPRA2 => RAdr(2),               -- [in  std_logic]
        DPRA3 => RAdr(3),               -- [in  std_logic]
        SPO   => WD_Out(I),             -- [out std_logic]
        DPO   => RD_Out(I));            -- [out std_logic]
  end generate The_Register;
  

end architecture IMP;
