-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- fsl_dct32.vhd - Entity and architecture
--
--
-------------------------------------------------------------------------------
-- Filename:        fsl_dct32.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              fsl_dct32.vhd
--
-------------------------------------------------------------------------------
-- Author:          
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--     2003-11-12    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity fsl_dct32 is
  port (
    Clk   : in std_logic;
    Reset : in std_logic;
    FSL_S_Clk     : out std_logic;
    FSL_S_Read    : out std_logic;
    FSL_S_Data    : in  std_logic_vector(0 to 31);
    FSL_S_Control : in  std_logic;
    FSL_S_Exists  : in  std_logic;

    FSL_M_Clk     : out std_logic;
    FSL_M_Write   : out std_logic;
    FSL_M_Data    : out std_logic_vector(0 to 31);
    FSL_M_Control : out std_logic;
    FSL_M_Full    : in  std_logic
    );
end entity fsl_dct32;

library unisim;
use unisim.all;

library fsl_dct32_v1_00_a;
use fsl_dct32_v1_00_a.do_the_mult;
use fsl_dct32_v1_00_a.dual_register;

architecture IMP of fsl_dct32 is

  component dual_register is
    generic (
      C_SIZE : integer);
    port (
      Clk    : in  std_logic;
      RAdr   : in  std_logic_vector(0 to 3);
      WAdr   : in  std_logic_vector(0 to 3);
      WE     : in  std_logic;
      D_In   : in  std_logic_vector(0 to C_SIZE-1);
      WD_Out : out std_logic_vector(0 to C_SIZE-1);
      RD_Out : out std_logic_vector(0 to C_SIZE-1));
  end component dual_register;
  
  component RAMB16_S36_S36 is
    generic (
      INIT_00 : bit_vector;
      INIT_01 : bit_vector;
      INIT_02 : bit_vector;
      INIT_03 : bit_vector;
      INIT_04 : bit_vector;
      INIT_05 : bit_vector;
      INIT_06 : bit_vector;
      INIT_07 : bit_vector;
      INIT_08 : bit_vector;
      INIT_09 : bit_vector;
      INIT_0A : bit_vector;
      INIT_0B : bit_vector;
      INIT_0C : bit_vector;
      INIT_0D : bit_vector;
      INIT_0E : bit_vector;
      INIT_0F : bit_vector;
      INIT_10 : bit_vector;
      INIT_11 : bit_vector;
      INIT_12 : bit_vector;
      INIT_13 : bit_vector;
      INIT_14 : bit_vector;
      INIT_15 : bit_vector;
      INIT_16 : bit_vector;
      INIT_17 : bit_vector;
      INIT_18 : bit_vector;
      INIT_19 : bit_vector;
      INIT_1A : bit_vector;
      INIT_1B : bit_vector;
      INIT_1C : bit_vector;
      INIT_1D : bit_vector;
      INIT_1E : bit_vector;
      INIT_1F : bit_vector;
      INIT_20 : bit_vector;
      INIT_21 : bit_vector;
      INIT_22 : bit_vector;
      INIT_23 : bit_vector;
      INIT_24 : bit_vector;
      INIT_25 : bit_vector;
      INIT_26 : bit_vector;
      INIT_27 : bit_vector;
      INIT_28 : bit_vector;
      INIT_29 : bit_vector;
      INIT_2A : bit_vector;
      INIT_2B : bit_vector;
      INIT_2C : bit_vector;
      INIT_2D : bit_vector;
      INIT_2E : bit_vector;
      INIT_2F : bit_vector;
      INIT_30 : bit_vector;
      INIT_31 : bit_vector;
      INIT_32 : bit_vector;
      INIT_33 : bit_vector;
      INIT_34 : bit_vector;
      INIT_35 : bit_vector;
      INIT_36 : bit_vector;
      INIT_37 : bit_vector;
      INIT_38 : bit_vector;
      INIT_39 : bit_vector;
      INIT_3A : bit_vector;
      INIT_3B : bit_vector;
      INIT_3C : bit_vector;
      INIT_3D : bit_vector;
      INIT_3E : bit_vector;
      INIT_3F : bit_vector);
    port (
      DIA   : in  std_logic_vector (31 downto 0);
      DIB   : in  std_logic_vector (31 downto 0);
      DIPA  : in  std_logic_vector (3 downto 0);
      DIPB  : in  std_logic_vector (3 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (8 downto 0);
      ADDRB : in  std_logic_vector (8 downto 0);
      DOA   : out std_logic_vector (31 downto 0);
      DOB   : out std_logic_vector (31 downto 0);
      DOPA  : out std_logic_vector (3 downto 0);
      DOPB  : out std_logic_vector (3 downto 0));
  end component RAMB16_S36_S36;

  component do_the_mult is
    port (
      Clk         : in  std_logic;
      Reset       : in  std_logic;
      CE          : in  std_logic;
      Valid_Input : in  std_logic;
      Op1         : in  std_logic_vector(0 to 31);
      Op2         : in  std_logic_vector(0 to 31);
      Valid_Res   : out std_logic;
      Result      : out std_logic_vector(0 to 31));
  end component do_the_mult;

  type ROM_TYPE is array (natural range 0 to 511) of std_logic_vector(0 to 31);

  constant Rom_Table_Low : ROM_TYPE :=
    ( 0 => X"0b504f33", 1 => X"0abeb4a0", 2 => X"0a267994", 3 => X"0987fbfc",
      4 => X"08e39d9c", 5 => X"0839c3cc", 6 => X"078ad74c", 7 => X"06d743f4",
      8 => X"061f78a8", 9 => X"0563e6a8", 10 => X"04a5018c", 11 => X"03e33f2c",
      12 => X"031f1704", 13 => X"0259020c", 14 => X"01917a64", 15 => X"00c8fb2c",
      16 => X"f4afb0cd", 17 => X"f3260fdc", 18 => X"f1e3a687", 19 => X"f0ef6f80",
      20 => X"f04eb419", 21 => X"f004ef0c", 22 => X"f013b92b", 23 => X"f07ac084",
      24 => X"f137ca16", 25 => X"f246be58", 26 => X"f3a1bfcb", 27 => X"f5414b6c",
      28 => X"f71c6263", 29 => X"f928bbfc", 30 => X"fb5afe6f", 31 => X"fda6fdec",
      32 => X"f4afb0cd", 33 => X"f7c63c30", 34 => X"fb5afe75", 35 => X"ff3704d2",
      36 => X"031f1707", 37 => X"06d74404", 38 => X"0a267993", 39 => X"0cd9f026",
      40 => X"0ec835e8", 41 => X"0fd3aac0", 42 => X"0fec46d5", 43 => X"0f109082",
      44 => X"0d4db315", 45 => X"0abeb498", 46 => X"078ad74b", 47 => X"03e33f2a",
      48 => X"0b504f33", 49 => X"0e76bd7c", 50 => X"0fec46d2", 51 => X"0f853f7e",
      52 => X"0d4db318", 53 => X"0987fbf8", 54 => X"04a5018e", 55 => X"ff3704d6",
      56 => X"f9e08756", 57 => X"f5414b5e", 58 => X"f1e3a682", 59 => X"f02c5548",
      60 => X"f04eb410", 61 => X"f246be56", 62 => X"f5d9866a", 63 => X"fa9c1964",
      64 => X"0b504f33", 65 => X"0563e69a", 66 => X"fe6e8593", 67 => X"f7c63c34",
      68 => X"f2b24ce8", 69 => X"f02c553c", 70 => X"f0b05f51", 71 => X"f425106e",
      72 => X"f9e08756", 73 => X"00c8fb30", 74 => X"078ad751", 75 => X"0cd9f026",
      76 => X"0fb14bea", 77 => X"0f853f7a", 78 => X"0c5e4033", 79 => X"06d74408",
      80 => X"f4afb0cd", 81 => X"f07ac081", 82 => X"f0b05f52", 83 => X"f5414b61",
      84 => X"fce0e8f9", 85 => X"0563e69f", 86 => X"0c5e4034", 87 => X"0fd3aabb",
      88 => X"0ec835e8", 89 => X"0987fc05", 90 => X"01917a6c", 91 => X"f928bbf9",
      92 => X"f2b24ced", 93 => X"f004ef0b", 94 => X"f1e3a682", 95 => X"f7c63c33",
      96 => X"f4afb0cd", 97 => X"fda6fdf4", 98 => X"078ad750", 99 => X"0e76bd7a",
      100 => X"0fb14be7", 101 => X"0abeb49a", 102 => X"01917a6c", 103 => X"f7c63c34",
      104 => X"f137ca16", 105 => X"f07ac082", 106 => X"f5d9866c", 107 => X"ff3704d0",
      108 => X"08e39d9f", 109 => X"0f109088", 110 => X"0f4fa0ac", 111 => X"0987fbfa",
      112 => X"0b504f33", 113 => X"0ffb10f1", 114 => X"0c5e403c", 115 => X"0259020d",
      116 => X"f71c6264", 117 => X"f07ac085", 118 => X"f1e3a682", 119 => X"fa9c1969",
      120 => X"061f78a8", 121 => X"0e76bd6f", 122 => X"0f4fa0b0", 123 => X"0839c3cb",
      124 => X"fce0e8f6", 125 => X"f3260fd7", 126 => X"f013b91e", 127 => X"f5414b63",
      128 => X"0b504f33", 129 => X"ff3704d0", 130 => X"f3a1bfc8", 131 => X"f02c5546",
      132 => X"f71c6264", 133 => X"03e33f28", 134 => X"0e1c5978", 135 => X"0f109086",
      136 => X"061f78a8", 137 => X"f928bbf8", 138 => X"f0b05f54", 139 => X"f246be66",
      140 => X"fce0e8f6", 141 => X"0987fbf4", 142 => X"0fec46d0", 143 => X"0bdaef8a",
      144 => X"f4afb0cd", 145 => X"f02c5540", 146 => X"f87528b0", 147 => X"06d74402",
      148 => X"0fb14be7", 149 => X"0bdaef90", 150 => X"fe6e8596", 151 => X"f246be62",
      152 => X"f137ca16", 153 => X"fc1cc0ce", 154 => X"0a267994", 155 => X"0ffb10f4",
      156 => X"08e39d9f", 157 => X"fa9c1966", 158 => X"f0b05f4e", 159 => X"f3260fdc",
      160 => X"f4afb0cd", 161 => X"03e33f33", 162 => X"0f4fa0aa", 163 => X"0bdaef8f",
      164 => X"fce0e8f9", 165 => X"f0ef6f7d", 166 => X"f3a1bfca", 167 => X"0259020d",
      168 => X"0ec835e8", 169 => X"0cd9f025", 170 => X"fe6e8598", 171 => X"f189428d",
      172 => X"f2b24ced", 173 => X"00c8fb27", 174 => X"0e1c5978", 175 => X"0db941ab",
      176 => X"0b504f33", 177 => X"0f109088", 178 => X"01917a69", 179 => X"f246be5c",
      180 => X"f2b24ce8", 181 => X"02590204", 182 => X"0f4fa0ad", 183 => X"0abeb4a0",
      184 => X"f9e08756", 185 => X"f004ef06", 186 => X"f87528af", 187 => X"0987fc0a",
      188 => X"0fb14bea", 189 => X"03e33f2a", 190 => X"f3a1bfcb", 191 => X"f1894276",
      192 => X"0b504f33", 193 => X"f928bbf3", 194 => X"f013b92a", 195 => X"fc1cc0d3",
      196 => X"0d4db318", 197 => X"0cd9f01d", 198 => X"fb5afe6c", 199 => X"f004ef09",
      200 => X"f9e08756", 201 => X"0bdaef93", 202 => X"0e1c597e", 203 => X"fda6fdfb",
      204 => X"f04eb410", 205 => X"f7c63c29", 206 => X"0a267998", 207 => X"0f10908d",
      208 => X"f4afb0cd", 209 => X"f246be59", 210 => X"04a5018b", 211 => X"0ffb10f3",
      212 => X"031f1707", 213 => X"f1894287", 214 => X"f5d9866b", 215 => X"0987fbf9",
      216 => X"0ec835e8", 217 => X"fda6fdf7", 218 => X"f013b92f", 219 => X"fa9c195d",
      220 => X"0d4db315", 221 => X"0bdaef95", 222 => X"f87528af", 223 => X"f07ac083",
      224 => X"f4afb0cd", 225 => X"0987fc02", 226 => X"0e1c5979", 227 => X"fa9c1960",
      228 => X"f04eb419", 229 => X"00c8fb30", 230 => X"0fec46d3", 231 => X"03e33f2e",
      232 => X"f137ca16", 233 => X"f7c63c34", 234 => X"0c5e4035", 235 => X"0bdaef96",
      236 => X"f71c6263", 237 => X"f1894282", 238 => X"04a5018f", 239 => X"0fd3aabc",
      240 => X"0b504f33", 241 => X"0bdaef9e", 242 => X"f5d98670", 243 => X"f3260fd2",
      244 => X"08e39d9c", 245 => X"0db941ac", 246 => X"f87528b2", 247 => X"f189427c",
      248 => X"061f78a8", 249 => X"0f109088", 250 => X"fb5afe74", 251 => X"f07ac074",
      252 => X"031f1704", 253 => X"0fd3aaca", 254 => X"fe6e858a", 255 => X"f004ef02",
      256 => X"0b504f33", 257 => X"f425106e", 258 => X"f5d98670", 259 => X"0cd9f028",
      260 => X"08e39d9c", 261 => X"f246be5c", 262 => X"f87528b2", 263 => X"0e76bd76",
      264 => X"061f78a8", 265 => X"f0ef6f80", 266 => X"fb5afe74", 267 => X"0f853f7a",
      268 => X"031f1704", 269 => X"f02c553e", 270 => X"fe6e858a", 271 => X"0ffb10e8",
      272 => X"f4afb0cd", 273 => X"f67803fe", 274 => X"0e1c5979", 275 => X"0563e69e",
      276 => X"f04eb419", 277 => X"ff3704d4", 278 => X"0fec46d3", 279 => X"fc1cc0cc",
      280 => X"f137ca16", 281 => X"0839c3d0", 282 => X"0c5e4035", 283 => X"f4251070",
      284 => X"f71c6263", 285 => X"0e76bd7a", 286 => X"04a5018f", 287 => X"f02c553a",
      288 => X"f4afb0cd", 289 => X"0db9419f", 290 => X"04a5018b", 291 => X"f004ef0f",
      292 => X"031f1707", 293 => X"0e76bd79", 294 => X"f5d9866b", 295 => X"f6780405",
      296 => X"0ec835e8", 297 => X"0259020d", 298 => X"f013b92f", 299 => X"0563e69d",
      300 => X"0d4db315", 301 => X"f4251073", 302 => X"f87528af", 303 => X"0f853f6f",
      304 => X"0b504f33", 305 => X"06d74401", 306 => X"f013b92a", 307 => X"03e33f27",
      308 => X"0d4db318", 309 => X"f3260fdf", 310 => X"fb5afe6c", 311 => X"0ffb10e5",
      312 => X"f9e08756", 313 => X"f4251079", 314 => X"0e1c597e", 315 => X"02590207",
      316 => X"f04eb410", 317 => X"0839c3d7", 318 => X"0a267998", 319 => X"f0ef6f75",
      320 => X"0b504f33", 321 => X"f0ef6f74", 322 => X"01917a69", 323 => X"0db9419e",
      324 => X"f2b24ce8", 325 => X"fda6fdf8", 326 => X"0f4fa0ad", 327 => X"f5414b5a",
      328 => X"f9e08756", 329 => X"0ffb10fa", 330 => X"f87528af", 331 => X"f67803f8",
      332 => X"0fb14bea", 333 => X"fc1cc0d6", 334 => X"f3a1bfcb", 335 => X"0e76bd8c",
      336 => X"f4afb0cd", 337 => X"fc1cc0d1", 338 => X"0f4fa0aa", 339 => X"f425106f",
      340 => X"fce0e8f9", 341 => X"0f10907f", 342 => X"f3a1bfca", 343 => X"fda6fdf5",
      344 => X"0ec835e8", 345 => X"f3260fdb", 346 => X"fe6e8598", 347 => X"0e76bd79",
      348 => X"f2b24ced", 349 => X"ff3704d1", 350 => X"0e1c5978", 351 => X"f246be5f",
      352 => X"f4afb0cd", 353 => X"0fd3aac0", 354 => X"f87528b0", 355 => X"f928bc00",
      356 => X"0fb14be7", 357 => X"f425106c", 358 => X"fe6e8596", 359 => X"0db941a0",
      360 => X"f137ca16", 361 => X"03e33f32", 362 => X"0a267994", 363 => X"f004ef12",
      364 => X"08e39d9f", 365 => X"0563e69a", 366 => X"f0b05f4e", 367 => X"0cd9f026",
      368 => X"0b504f33", 369 => X"00c8fb30", 370 => X"f3a1bfc8", 371 => X"0fd3aab4",
      372 => X"f71c6264", 373 => X"fc1cc0d8", 374 => X"0e1c5978", 375 => X"f0ef6f80",
      376 => X"061f78a8", 377 => X"06d74400", 378 => X"f0b05f54", 379 => X"0db94194",
      380 => X"fce0e8f6", 381 => X"f678040c", 382 => X"0fec46d0", 383 => X"f425106c",
      384 => X"0b504f33", 385 => X"f004ef0b", 386 => X"0c5e403c", 387 => X"fda6fded",
      388 => X"f71c6264", 389 => X"0f853f83", 390 => X"f1e3a682", 391 => X"0563e699",
      392 => X"061f78a8", 393 => X"f1894289", 394 => X"0f4fa0b0", 395 => X"f7c63c2b",
      396 => X"fce0e8f6", 397 => X"0cd9f029", 398 => X"f013b91e", 399 => X"0abeb497",
      400 => X"f4afb0cd", 401 => X"0259020c", 402 => X"078ad750", 403 => X"f1894280",
      404 => X"0fb14be7", 405 => X"f5414b6a", 406 => X"01917a6c", 407 => X"0839c3ce",
      408 => X"f137ca16", 409 => X"0f853f7e", 410 => X"f5d9866c", 411 => X"00c8fb32",
      412 => X"08e39d9f", 413 => X"f0ef6f7c", 414 => X"0f4fa0ac", 415 => X"f67803f8",
      416 => X"f4afb0cd", 417 => X"0f853f83", 418 => X"f0b05f52", 419 => X"0abeb499",
      420 => X"fce0e8f9", 421 => X"fa9c1965", 422 => X"0c5e4034", 423 => X"f02c5543",
      424 => X"0ec835e8", 425 => X"f67803ff", 426 => X"01917a6c", 427 => X"06d743fd",
      428 => X"f2b24ced", 429 => X"0ffb10f9", 430 => X"f1e3a682", 431 => X"0839c3d7",
      432 => X"0b504f33", 433 => X"fa9c196a", 434 => X"fe6e8593", 435 => X"0839c3c6",
      436 => X"f2b24ce8", 437 => X"0fd3aac0", 438 => X"f0b05f51", 439 => X"0bdaef98",
      440 => X"f9e08756", 441 => X"ff3704c8", 442 => X"078ad751", 443 => X"f3260fd8",
      444 => X"0fb14bea", 445 => X"f07ac08a", 446 => X"0c5e4033", 447 => X"f928bbf6",
      448 => X"0b504f33", 449 => X"f1894280", 450 => X"0fec46d2", 451 => X"f07ac080",
      452 => X"0d4db318", 453 => X"f6780400", 454 => X"04a5018e", 455 => X"00c8fb24",
      456 => X"f9e08756", 457 => X"0abeb4a2", 458 => X"f1e3a682", 459 => X"0fd3aabe",
      460 => X"f04eb410", 461 => X"0db941a6", 462 => X"f5d9866a", 463 => X"0563e69e",
      464 => X"f4afb0cd", 465 => X"0839c3c8", 466 => X"fb5afe75", 467 => X"00c8fb30",
      468 => X"031f1707", 469 => X"f928bbfc", 470 => X"0a267993", 471 => X"f3260fe0",
      472 => X"0ec835e8", 473 => X"f02c553c", 474 => X"0fec46d5", 475 => X"f0ef6f80",
      476 => X"0d4db315", 477 => X"f5414b60", 478 => X"078ad74b", 479 => X"fc1cc0d0",
      480 => X"f4afb0cd", 481 => X"0cd9f028", 482 => X"f1e3a687", 483 => X"0f109086",
      484 => X"f04eb419", 485 => X"0ffb10f4", 486 => X"f013b92b", 487 => X"0f853f7a",
      488 => X"f137ca16", 489 => X"0db941a8", 490 => X"f3a1bfcb", 491 => X"0abeb49a",
      492 => X"f71c6263", 493 => X"06d74408", 494 => X"fb5afe6f", 495 => X"02590212",
      496 => X"0b504f33", 497 => X"f5414b6c", 498 => X"0a267994", 499 => X"f6780402",
      500 => X"08e39d9c", 501 => X"f7c63c34", 502 => X"078ad74c", 503 => X"f928bbfa",
      504 => X"061f78a8", 505 => X"fa9c1964", 506 => X"04a5018c", 507 => X"fc1cc0ce",
      508 => X"031f1704", 509 => X"fda6fdec", 510 => X"01917a64", 511 => X"ff3704c6");

  constant Rom_Table_High : ROM_TYPE :=
    ( 0 => X"0bdaef8a", 1 => X"0c5e4032", 2 => X"0cd9f022", 3 => X"0d4db314",
      4 => X"0db941a2", 5 => X"0e1c597a", 6 => X"0e76bd7e", 7 => X"0ec835e8",
      8 => X"0f10907e", 9 => X"0f4fa0aa", 10 => X"0f853f7e", 11 => X"0fb14be8",
      12 => X"0fd3aabe", 13 => X"0fec46d2", 14 => X"0ffb10f2", 15 => X"10000000",
      16 => X"f6780400", 17 => X"f87528b3", 18 => X"fa9c1964", 19 => X"fce0e8f7",
      20 => X"ff3704cc", 21 => X"01917a6b", 22 => X"03e33f34", 23 => X"061f78aa",
      24 => X"0839c3c8", 25 => X"0a267993", 26 => X"0bdaef94", 27 => X"0d4db315",
      28 => X"0e76bd78", 29 => X"0f4fa0ab", 30 => X"0fd3aac0", 31 => X"10000000",
      32 => X"f246be60", 33 => X"f0b05f53", 34 => X"f004ef0e", 35 => X"f04eb417",
      36 => X"f1894284", 37 => X"f3a1bfc9", 38 => X"f6780402", 39 => X"f9e08756",
      40 => X"fda6fdf0", 41 => X"01917a6b", 42 => X"0563e69e", 43 => X"08e39d9d",
      44 => X"0bdaef90", 45 => X"0e1c5979", 46 => X"0f853f7e", 47 => X"10000000",
      48 => X"06d743fe", 49 => X"01917a6a", 50 => X"fc1cc0d0", 51 => X"f71c6260",
      52 => X"f3260fde", 53 => X"f0b05f52", 54 => X"f004ef0c", 55 => X"f137ca18",
      56 => X"f4251070", 57 => X"f87528b2", 58 => X"fda6fdf2", 59 => X"031f1708",
      60 => X"0839c3cc", 61 => X"0c5e4036", 62 => X"0f109082", 63 => X"10000000",
      64 => X"0f109084", 65 => X"0fec46d3", 66 => X"0db941a2", 67 => X"08e39d9e",
      68 => X"0259020e", 69 => X"fb5afe75", 70 => X"f5414b64", 71 => X"f137ca18",
      72 => X"f004ef0e", 73 => X"f1e3a685", 74 => X"f6780400", 75 => X"fce0e8f8",
      76 => X"03e33f30", 77 => X"0a267993", 78 => X"0e76bd7a", 79 => X"10000000",
      80 => X"fc1cc0d1", 81 => X"04a5018e", 82 => X"0bdaef95", 83 => X"0fb14be7",
      84 => X"0f109083", 85 => X"0a267994", 86 => X"0259020b", 87 => X"f9e08756",
      88 => X"f3260fdd", 89 => X"f013b92c", 90 => X"f1894285", 91 => X"f71c6263",
      92 => X"ff3704cf", 93 => X"078ad74e", 94 => X"0db941a3", 95 => X"10000000",
      96 => X"f02c553e", 97 => X"f1e3a684", 98 => X"f928bbfc", 99 => X"031f1707",
      100 => X"0bdaef90", 101 => X"0fec46d4", 102 => X"0db941a2", 103 => X"061f78aa",
      104 => X"fc1cc0d0", 105 => X"f3a1bfc8", 106 => X"f004ef0e", 107 => X"f2b24ceb",
      108 => X"fa9c1962", 109 => X"04a5018c", 110 => X"0cd9f024", 111 => X"10000000",
      112 => X"00c8fb2f", 113 => X"f5d98666", 114 => X"f02c553f", 115 => X"f2b24cea",
      116 => X"fc1cc0cf", 117 => X"078ad750", 118 => X"0f10907f", 119 => X"0ec835e8",
      120 => X"06d74405", 121 => X"fb5afe72", 122 => X"f246be5d", 123 => X"f04eb418",
      124 => X"f6780401", 125 => X"01917a6c", 126 => X"0bdaef91", 127 => X"10000000",
      128 => X"0ffb10f0", 129 => X"0a267994", 130 => X"fda6fdee", 131 => X"f2b24cea",
      132 => X"f07ac084", 133 => X"f87528b0", 134 => X"0563e69a", 135 => X"0ec835e8",
      136 => X"0e76bd7c", 137 => X"04a5018c", 138 => X"f7c63c32", 139 => X"f04eb418",
      140 => X"f3260fdc", 141 => X"fe6e8594", 142 => X"0abeb49a", 143 => X"10000000",
      144 => X"0259020e", 145 => X"0e1c597a", 146 => X"0e76bd7c", 147 => X"031f1707",
      148 => X"f5414b62", 149 => X"f013b92c", 150 => X"f7c63c30", 151 => X"061f78aa",
      152 => X"0f853f80", 153 => X"0c5e4036", 154 => X"ff3704d2", 155 => X"f2b24ceb",
      156 => X"f0ef6f7c", 157 => X"fb5afe74", 158 => X"0987fbfe", 159 => X"10000000",
      160 => X"f07ac07d", 161 => X"fb5afe74", 162 => X"0abeb49d", 163 => X"0fb14be7",
      164 => X"0563e69b", 165 => X"f5d9866c", 166 => X"f02c553f", 167 => X"f9e08756",
      168 => X"0987fbff", 169 => X"0fec46d2", 170 => X"06d74403", 171 => X"f71c6263",
      172 => X"f004ef0d", 173 => X"f87528b2", 174 => X"0839c3cd", 175 => X"10000000",
      176 => X"fa9c195c", 177 => X"f013b92f", 178 => X"f7c63c34", 179 => X"08e39d9e",
      180 => X"0fd3aac4", 181 => X"04a5018b", 182 => X"f425106c", 183 => X"f137ca18",
      184 => X"ff3704d2", 185 => X"0e1c5979", 186 => X"0cd9f022", 187 => X"fce0e8f8",
      188 => X"f07ac082", 189 => X"f5d9866d", 190 => X"06d74402", 191 => X"10000000",
      192 => X"0e76bd83", 193 => X"fe6e8598", 194 => X"f07ac07f", 195 => X"f71c6260",
      196 => X"0987fc01", 197 => X"0f4fa0ae", 198 => X"00c8fb31", 199 => X"f137ca18",
      200 => X"f5414b63", 201 => X"078ad74c", 202 => X"0fd3aabf", 203 => X"031f1708",
      204 => X"f246be5d", 205 => X"f3a1bfca", 206 => X"0563e69d", 207 => X"10000000",
      208 => X"0839c3d1", 209 => X"0f4fa0ab", 210 => X"ff3704cf", 211 => X"f04eb417",
      212 => X"f928bbff", 213 => X"0c5e4037", 214 => X"0cd9f025", 215 => X"f9e08756",
      216 => X"f02c553f", 217 => X"fe6e8593", 218 => X"0f109081", 219 => X"08e39d9d",
      220 => X"f5414b65", 221 => X"f1e3a687", 222 => X"03e33f2f", 223 => X"10000000",
      224 => X"f3260fd8", 225 => X"078ad74b", 226 => X"0f109086", 227 => X"fce0e8f7",
      228 => X"f004ef0a", 229 => X"fe6e8595", 230 => X"0f853f80", 231 => X"061f78aa",
      232 => X"f246be5a", 233 => X"f5d9866b", 234 => X"0abeb49c", 235 => X"0d4db315",
      236 => X"f928bbfc", 237 => X"f0b05f55", 238 => X"0259020e", 239 => X"10000000",
      240 => X"f5414b58", 241 => X"f3a1bfc8", 242 => X"0987fc04", 243 => X"0d4db314",
      244 => X"f7c63c2e", 245 => X"f1e3a686", 246 => X"06d74406", 247 => X"0ec835e8",
      248 => X"fa9c195e", 249 => X"f0b05f54", 250 => X"03e33f32", 251 => X"0fb14be8",
      252 => X"fda6fdf0", 253 => X"f013b92e", 254 => X"00c8fb30", 255 => X"10000000",
      256 => X"0abeb49a", 257 => X"f3a1bfc8", 258 => X"f67803fc", 259 => X"0d4db314",
      260 => X"0839c3cc", 261 => X"f1e3a686", 262 => X"f928bbfe", 263 => X"0ec835e8",
      264 => X"0563e69c", 265 => X"f0b05f54", 266 => X"fc1cc0ce", 267 => X"0fb14be8",
      268 => X"0259020e", 269 => X"f013b92e", 270 => X"ff3704d0", 271 => X"10000000",
      272 => X"0cd9f026", 273 => X"078ad74b", 274 => X"f0ef6f7e", 275 => X"fce0e8f7",
      276 => X"0ffb10f0", 277 => X"fe6e8595", 278 => X"f07ac080", 279 => X"061f78aa",
      280 => X"0db941a4", 281 => X"f5d9866b", 282 => X"f5414b64", 283 => X"0d4db315",
      284 => X"06d74402", 285 => X"f0b05f55", 286 => X"fda6fdf2", 287 => X"10000000",
      288 => X"f7c63c35", 289 => X"0f4fa0ab", 290 => X"00c8fb31", 291 => X"f04eb417",
      292 => X"06d743ff", 293 => X"0c5e4037", 294 => X"f3260fdf", 295 => X"f9e08756",
      296 => X"0fd3aabb", 297 => X"fe6e8593", 298 => X"f0ef6f7f", 299 => X"08e39d9d",
      300 => X"0abeb499", 301 => X"f1e3a687", 302 => X"fc1cc0d1", 303 => X"10000000",
      304 => X"f1894287", 305 => X"fe6e8598", 306 => X"0f853f81", 307 => X"f71c6260",
      308 => X"f6780401", 309 => X"0f4fa0ae", 310 => X"ff3704d3", 311 => X"f137ca18",
      312 => X"0abeb497", 313 => X"078ad74c", 314 => X"f02c5541", 315 => X"031f1708",
      316 => X"0db941a1", 317 => X"f3a1bfca", 318 => X"fa9c1963", 319 => X"10000000",
      320 => X"0563e6a6", 321 => X"f013b92f", 322 => X"0839c3cc", 323 => X"08e39d9e",
      324 => X"f02c553e", 325 => X"04a5018b", 326 => X"0bdaef94", 327 => X"f137ca18",
      328 => X"00c8fb2c", 329 => X"0e1c5979", 330 => X"f3260fde", 331 => X"fce0e8f8",
      332 => X"0f853f7c", 333 => X"f5d9866d", 334 => X"f928bbfe", 335 => X"10000000",
      336 => X"0f853f7d", 337 => X"fb5afe74", 338 => X"f5414b67", 339 => X"0fb14be7",
      340 => X"fa9c1963", 341 => X"f5d9866c", 342 => X"0fd3aabd", 343 => X"f9e08756",
      344 => X"f6780403", 345 => X"0fec46d2", 346 => X"f928bbfd", 347 => X"f71c6263",
      348 => X"0ffb10f1", 349 => X"f87528b2", 350 => X"f7c63c33", 351 => X"10000000",
      352 => X"fda6fdf0", 353 => X"0e1c597a", 354 => X"f1894284", 355 => X"031f1707",
      356 => X"0abeb49c", 357 => X"f013b92c", 358 => X"0839c3cc", 359 => X"061f78aa",
      360 => X"f07ac082", 361 => X"0c5e4036", 362 => X"00c8fb2e", 363 => X"f2b24ceb",
      364 => X"0f109082", 365 => X"fb5afe74", 366 => X"f6780402", 367 => X"10000000",
      368 => X"f004ef0e", 369 => X"0a267994", 370 => X"02590216", 371 => X"f2b24cea",
      372 => X"0f853f76", 373 => X"f87528b0", 374 => X"fa9c1962", 375 => X"0ec835e8",
      376 => X"f1894286", 377 => X"04a5018c", 378 => X"0839c3ce", 379 => X"f04eb418",
      380 => X"0cd9f022", 381 => X"fe6e8594", 382 => X"f5414b66", 383 => X"10000000",
      384 => X"ff3704d3", 385 => X"f5d98666", 386 => X"0fd3aac1", 387 => X"f2b24cea",
      388 => X"03e33f2b", 389 => X"078ad750", 390 => X"f0ef6f7d", 391 => X"0ec835e8",
      392 => X"f928bbfd", 393 => X"fb5afe72", 394 => X"0db941a3", 395 => X"f04eb418",
      396 => X"0987fbfd", 397 => X"01917a6c", 398 => X"f425106f", 399 => X"10000000",
      400 => X"0fd3aac0", 401 => X"f1e3a684", 402 => X"06d74408", 403 => X"031f1707",
      404 => X"f425106a", 405 => X"0fec46d4", 406 => X"f246be5a", 407 => X"061f78aa",
      408 => X"03e33f32", 409 => X"f3a1bfc8", 410 => X"0ffb10f2", 411 => X"f2b24ceb",
      412 => X"0563e69c", 413 => X"04a5018c", 414 => X"f3260fdc", 415 => X"10000000",
      416 => X"03e33f29", 417 => X"04a5018e", 418 => X"f425106f", 419 => X"0fb14be7",
      420 => X"f0ef6f7b", 421 => X"0a267994", 422 => X"fda6fdf1", 423 => X"f9e08756",
      424 => X"0cd9f025", 425 => X"f013b92c", 426 => X"0e76bd7b", 427 => X"f71c6263",
      428 => X"00c8fb2f", 429 => X"078ad74e", 430 => X"f246be5d", 431 => X"10000000",
      432 => X"f0ef6f76", 433 => X"0fec46d3", 434 => X"f246be5e", 435 => X"08e39d9e",
      436 => X"fda6fdf4", 437 => X"fb5afe75", 438 => X"0abeb498", 439 => X"f137ca18",
      440 => X"0ffb10f4", 441 => X"f1e3a685", 442 => X"0987fc00", 443 => X"fce0e8f8",
      444 => X"fc1cc0ce", 445 => X"0a267993", 446 => X"f1894286", 447 => X"10000000",
      448 => X"f928bc04", 449 => X"01917a6a", 450 => X"03e33f30", 451 => X"f71c6260",
      452 => X"0cd9f024", 453 => X"f0b05f52", 454 => X"0ffb10f4", 455 => X"f137ca18",
      456 => X"0bdaef8e", 457 => X"f87528b2", 458 => X"0259020e", 459 => X"031f1708",
      460 => X"f7c63c32", 461 => X"0c5e4036", 462 => X"f0ef6f7e", 463 => X"10000000",
      464 => X"0db941a6", 465 => X"f0b05f53", 466 => X"0ffb10f2", 467 => X"f04eb417",
      468 => X"0e76bd7a", 469 => X"f3a1bfc9", 470 => X"0987fc02", 471 => X"f9e08756",
      472 => X"0259020a", 473 => X"01917a6b", 474 => X"fa9c1962", 475 => X"08e39d9d",
      476 => X"f425106e", 477 => X"0e1c5979", 478 => X"f07ac082", 479 => X"10000000",
      480 => X"0987fbfa", 481 => X"f87528b3", 482 => X"0563e698", 483 => X"fce0e8f7",
      484 => X"00c8fb32", 485 => X"01917a6b", 486 => X"fc1cc0d0", 487 => X"061f78aa",
      488 => X"f7c63c32", 489 => X"0a267993", 490 => X"f425106c", 491 => X"0d4db315",
      492 => X"f1894286", 493 => X"0f4fa0ab", 494 => X"f02c5540", 495 => X"10000000",
      496 => X"f4251068", 497 => X"0c5e4032", 498 => X"f3260fda", 499 => X"0d4db314",
      500 => X"f246be5c", 501 => X"0e1c597a", 502 => X"f1894286", 503 => X"0ec835e8",
      504 => X"f0ef6f7c", 505 => X"0f4fa0aa", 506 => X"f07ac082", 507 => X"0fb14be8",
      508 => X"f02c5540", 509 => X"0fec46d2", 510 => X"f004ef0e", 511 => X"10000000");

  type state_type is (idle, executing, writing);
  signal state : state_type;

  signal writing_done : std_logic;
  
  signal CE               : std_logic;
  signal Valid_Input_Low  : std_logic;
  signal Valid_Input_High : std_logic;
  signal Op1              : std_logic_vector(0 to 31);
  signal Op2_low          : std_logic_vector(0 to 31);
  signal Op2_high         : std_logic_vector(0 to 31);
  signal Valid_Res_low    : std_logic;
  signal Valid_Res_high   : std_logic;
  signal Result_low       : std_logic_vector(0 to 31);
  signal Result_high      : std_logic_vector(0 to 31);

  signal table_low_addr  : std_logic_vector(0 to 8);
  signal table_high_addr : std_logic_vector(0 to 8);

  signal zero32 : std_logic_vector(0 to 31);
  signal zero9  : std_logic_vector(0 to 8);
  signal zero4  : std_logic_vector(0 to 3);

  signal Count_In_32 : std_logic_vector(0 to 4);
  signal Count_In_16 : std_logic_vector(0 to 3);
  
  signal Count_out_32      : std_logic_vector(0 to 4);
  signal Count_out_16      : std_logic_vector(0 to 3);

  type sum_array is array (natural range 0 to 15) of std_logic_vector(0 to 31);
  
  signal result_low_sum    : sum_array;
  signal result_high_sum   : sum_array;

  signal count         : std_logic_vector(0 to 4);
  signal FSL_M_Write_i : std_logic;

  signal new_res_low_sum    : std_logic_vector(0 to 31);
  signal old_res_low_sum    : std_logic_vector(0 to 31);
  signal result_low_sum_out : std_logic_vector(0 to 31);

  signal new_res_high_sum    : std_logic_vector(0 to 31);
  signal old_res_high_sum    : std_logic_vector(0 to 31);
  signal result_high_sum_out : std_logic_vector(0 to 31);

begin  -- architecture IMP

  zero32 <= (others => '0');
  zero9  <= (others => '0');
  zero4  <= (others => '0');

  -----------------------------------------------------------------------------
  -- The constant tables (low and high)
  -----------------------------------------------------------------------------
  Constant_Table_Low: process (Clk) is
  begin  -- process Constant_Table_Low
    if Clk'event and Clk = '1' then  -- rising clock edge
      op2_low <= Rom_Table_Low(to_integer(unsigned(table_low_addr)));
    end if;
  end process Constant_Table_Low;

  Constant_Table_High: process (Clk) is
  begin  -- process Constant_Table_High
    if Clk'event and Clk = '1' then  -- rising clock edge
      op2_high <= Rom_Table_High(to_integer(unsigned(table_high_addr)));
    end if;
  end process Constant_Table_High;

  -----------------------------------------------------------------------------
  -- Handles the state of the core
  -----------------------------------------------------------------------------
  State_Machine: process (Clk, Reset) is
  begin  -- process State_Machine
    if Reset = '1' then                 -- asynchronous reset (active high)
      state <= idle;
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      case state is
        when Idle =>
          if (FSL_S_Exists = '1') then
            state <= executing;
          end if;
        when executing =>
          if (Count_In_32 = "11111" and Count_In_16 = "1111") then
            state <= writing;
          end if;
        when writing =>
          if (writing_done = '1') then
            state <= idle;
          end if;
        when others => null;
      end case;
    end if;

  end process State_Machine;
  
  -----------------------------------------------------------------------------
  -- Handles the input side
  -----------------------------------------------------------------------------
  Count_32_In : process (Clk, Reset) is
  begin  -- process Count_32_In
    if Reset = '1' then                 -- asynchronous reset (active high)
      Count_In_32 <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (writing_done = '1') then
        Count_In_32 <= (others => '0');
      elsif (CE = '1') then
        if (Count_In_16 = "1111") then
          Count_In_32 <= std_logic_vector(unsigned(Count_In_32) + 1);
        end if;
      end if;
    end if;
  end process Count_32_In;

  Count_16_In: process (Clk, Reset) is
  begin  -- process Count_16_In
    if Reset = '1' then                 -- asynchronous reset (active high)
      Count_In_16 <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (writing_done = '1') then
        Count_In_16 <= (others => '0');
      elsif (CE = '1') then
        Count_In_16 <= std_logic_vector(unsigned(Count_In_16) + 1);          
      end if;
    end if;
  end process Count_16_In;

  table_low_addr <= Count_In_32 & Count_In_16;
  table_high_addr <= Count_In_32 & Count_In_16;

  Get_Op1 : process (Clk, Reset) is
  begin  -- process Get_Op1
    if Reset = '1' then                 -- asynchronous reset (active high)
      FSL_S_Read <= '0';
      Op1        <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      FSL_S_Read <= '0';
      if (state = executing) and (Count_In_16 = "0000") and (FSL_S_Exists = '1') then
        FSL_S_Read <= '1';
        Op1        <= FSL_S_Data;
      end if;
    end if;
  end process Get_Op1;

  Valid_Input: process (Clk, Reset) is
  begin  -- process Valid_Input
    if Reset = '1' then                 -- asynchronous reset (active high)
      Valid_Input_Low <= '0';
      Valid_Input_High <= '0';
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      Valid_Input_Low <= '0';
      Valid_Input_High <= '0';
      if (state = executing) then
        Valid_Input_Low  <= CE;
        Valid_Input_High <= CE;        
      end if;
    end if;
  end process Valid_Input;

  CE_Calc: process (state,Count_In_16, FSL_S_Exists) is
  begin  -- process CE_Calc
    CE <= '0';
    if (state = executing) then
      CE <= '1';
      if (Count_In_16 = "0000" and FSL_S_Exists = '0') then
        CE <= '0';
      end if;
    end if;
    if (state = writing) then
      CE <= '1';
      if (FSL_M_Full = '1') then
        CE <= '0';
      end if;
    end if;
  end process CE_Calc;
  
  
  -----------------------------------------------------------------------------
  -- The mult modules
  -----------------------------------------------------------------------------
  do_the_mult_low : do_the_mult
    port map (
      Clk         => Clk,               -- [in  std_logic]
      Reset       => Reset,             -- [in  std_logic]
      CE          => CE,                -- [in  std_logic]
      Valid_Input => Valid_Input_Low,   -- [in  std_logic]
      Op1         => Op1,               -- [in  std_logic_vector(0 to 31)]
      Op2         => Op2_low,           -- [in  std_logic_vector(0 to 31)]
      Valid_Res   => Valid_Res_low,     -- [out std_logic]
      Result      => Result_low);       -- [out std_logic_vector(0 to 31)]

  do_the_mult_high : do_the_mult
    port map (
      Clk         => Clk,               -- [in  std_logic]
      Reset       => Reset,             -- [in  std_logic]
      CE          => CE,                -- [in  std_logic]
      Valid_Input => Valid_Input_High,  -- [in  std_logic]
      Op1         => Op1,               -- [in  std_logic_vector(0 to 31)]
      Op2         => Op2_high,          -- [in  std_logic_vector(0 to 31)]
      Valid_Res   => Valid_Res_high,    -- [out std_logic]
      Result      => Result_high);      -- [out std_logic_vector(0 to 31)]

  -----------------------------------------------------------------------------
  -- Do the summation
  -----------------------------------------------------------------------------
  Count_32_Out: process (Clk, Reset) is
  begin  -- process Count_32_Out
    if Reset = '1' then                 -- asynchronous reset (active high)
      Count_Out_32 <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (Valid_Res_low = '1') then
        if (Count_out_16 = "1111") then
          Count_Out_32 <= std_logic_vector(unsigned(Count_Out_32) + 1);          
        end if;
      end if;
    end if;
  end process Count_32_Out;

  Count_16_Out: process (Clk, Reset) is
  begin  -- process Count_16_Out
    if Reset = '1' then                 -- asynchronous reset (active high)
      Count_Out_16 <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (Valid_Res_low = '1') then
        Count_Out_16 <= std_logic_vector(unsigned(Count_Out_16) + 1);
      end if;
    end if;
  end process Count_16_Out;

--  result_low_sum_handle: process (Clk, Reset) is
--  begin  -- process result_low_sum_handle
--    if Clk'event and Clk = '1' then  -- rising clock edge
--      if Valid_Res_low = '1' then
--        if (Count_out_32 = "00000") then
--          result_low_sum(to_integer(unsigned(Count_out_16))) <= Result_low;
--        else
--          result_low_sum(to_integer(unsigned(Count_out_16))) <= std_logic_vector(unsigned(result_low_sum(to_integer(unsigned(Count_out_16)))) + unsigned(Result_low));
--        end if;
--      end if;
--    end if;
--  end process result_low_sum_handle;

--  result_high_sum_handle : process (Clk, Reset) is
--  begin  -- process result_high_sum_handle
--    if Clk'event and Clk = '1' then  -- rising clock edge
--      if Valid_Res_high = '1' then
--        if (Count_out_32 = "00000") then
--          result_high_sum(to_integer(unsigned(Count_out_16))) <= Result_high;
--        else
--          result_high_sum(to_integer(unsigned(Count_out_16))) <= std_logic_vector(unsigned(result_high_sum(to_integer(unsigned(Count_out_16)))) + unsigned(Result_high));
--        end if;
--      end if;
--    end if;
--  end process result_high_sum_handle;

  new_res_low_sum <= Result_low when Count_out_32 = "00000" else std_logic_vector(unsigned(old_res_low_sum) + unsigned(Result_low));
  
  dual_register_low : dual_register
    generic map (
      C_SIZE => 32)                     -- [integer]
    port map (
      Clk    => Clk,                    -- [in  std_logic]
      RAdr   => Count(0 to 3),                   -- [in  std_logic_vector(0 to 3)]
      WAdr   => Count_out_16,           -- [in  std_logic_vector(0 to 3)]
      WE     => Valid_Res_low,         -- [in  std_logic]
      D_In   => new_res_low_sum,      -- [in  std_logic_vector(0 to C_SIZE-1)]
      WD_Out => old_res_low_sum,      -- [out std_logic_vector(0 to C_SIZE-1)]
      RD_Out => Result_low_sum_out);  -- [out std_logic_vector(0 to C_SIZE-1)]

  new_res_high_sum <= Result_high when Count_out_32 = "00000" else std_logic_vector(unsigned(old_res_high_sum) + unsigned(Result_high));
  
  dual_register_high : dual_register
    generic map (
      C_SIZE => 32)                     -- [integer]
    port map (
      Clk    => Clk,                    -- [in  std_logic]
      RAdr   => Count(0 to 3),                   -- [in  std_logic_vector(0 to 3)]
      WAdr   => Count_out_16,           -- [in  std_logic_vector(0 to 3)]
      WE     => Valid_Res_high,         -- [in  std_logic]
      D_In   => new_res_high_sum,      -- [in  std_logic_vector(0 to C_SIZE-1)]
      WD_Out => old_res_high_sum,      -- [out std_logic_vector(0 to C_SIZE-1)]
      RD_Out => Result_high_sum_out);  -- [out std_logic_vector(0 to C_SIZE-1)]

  FSL_M_Data <= result_low_sum_out when count(4) = '0' else Result_high_sum_out;
    
  Write_To_FSL : process (Clk, Reset) is
  begin  -- process Write_To_FSL
    if Reset = '1' then                 -- asynchronous reset (active high)
      FSL_M_Write_i <= '0';
      writing_done <= '0';
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      FSL_M_Write_i <= '0';
      if (state = writing) and (FSL_M_Full = '0') and (writing_done = '0') then
        if (Count_out_32 = "00000") and (Count_Out_16 = "0000") then
          FSL_M_Write_i <= '1';
          if (count = "11111") then
            writing_done <= '1';
          end if;
        end if;
      elsif (state = idle) then
        writing_done <= '0';
      end if;
    end if;
  end process Write_To_FSL;
    
  Write_To_FSL_count : process (Clk, Reset) is
  begin  -- process Write_To_FSL_count
    if Reset = '1' then                 -- asynchronous reset (active high)
      count       <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (FSL_M_Write_i = '1' and FSL_M_Full = '0') then
        count <= std_logic_vector(unsigned(count) + 1);        
      end if;
    end if;
  end process Write_To_FSL_count;

  FSL_M_Write <= FSL_M_Write_i and not FSL_M_Full;
  
  FSL_M_Control <= '0';
  FSL_M_Clk     <= Clk;
end architecture IMP;
