-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- fsl_dct32.vhd - Entity and architecture
--
--
-------------------------------------------------------------------------------
-- Filename:        fsl_imdct.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              fsl_imdct.vhd
--
-------------------------------------------------------------------------------
-- Author:          
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--     2003-11-12    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity fsl_imdct is
  port (
    Clk           : in  std_logic;
    Reset         : in  std_logic;
    FSL_S_Clk     : out std_logic;
    FSL_S_Read    : out std_logic;
    FSL_S_Data    : in  std_logic_vector(0 to 31);
    FSL_S_Control : in  std_logic;
    FSL_S_Exists  : in  std_logic;

    FSL_M_Clk     : out std_logic;
    FSL_M_Write   : out std_logic;
    FSL_M_Data    : out std_logic_vector(0 to 31);
    FSL_M_Control : out std_logic;
    FSL_M_Full    : in  std_logic
    );
end entity fsl_imdct;

library unisim;
use unisim.all;

library fsl_imdct_v1_00_a;
use fsl_imdct_v1_00_a.do_the_mult;
use fsl_imdct_v1_00_a.dual_register;

architecture IMP of fsl_imdct is

  component dual_register is
    generic (
      C_SIZE : integer);
    port (
      Clk    : in  std_logic;
      RAdr   : in  std_logic_vector(0 to 3);
      WAdr   : in  std_logic_vector(0 to 3);
      WE     : in  std_logic;
      D_In   : in  std_logic_vector(0 to C_SIZE-1);
      WD_Out : out std_logic_vector(0 to C_SIZE-1);
      RD_Out : out std_logic_vector(0 to C_SIZE-1));
  end component dual_register;

  component do_the_mult is
    port (
      Clk         : in  std_logic;
      Reset       : in  std_logic;
      CE          : in  std_logic;
      Valid_Input : in  std_logic;
      Op1         : in  std_logic_vector(0 to 31);
      Op2         : in  std_logic_vector(0 to 31);
      Valid_Res   : out std_logic;
      Result      : out std_logic_vector(0 to 31));
  end component do_the_mult;

  type ROM_TYPE is array (natural range 0 to 647) of std_logic_vector(0 to 31);

  constant Rom_Table : ROM_TYPE :=
    (0   => X"0acf37ad", 1 => X"09bd7ca0", 2 => X"0898c779", 3 => X"07635284",
     4   => X"061f78aa", 5 => X"04cfb0e2", 6 => X"03768962", 7 => X"0216a2a2",
     8   => X"00b2aa3e", 9 => X"ff4d55c2", 10 => X"fde95d5e", 11 => X"fc89769e",
     12  => X"fb304f1e", 13 => X"f9e08756", 14 => X"f89cad7c", 15 => X"f7673887",
     16  => X"f6428360", 17 => X"f530c853", 18 => X"f4341cae", 19 => X"f34e6cba",
     20  => X"f28177f9", 21 => X"f1cecdbb", 22 => X"f137ca18", 23 => X"f0bd934b",
     24  => X"f0611770", 25 => X"f0230ab7", 26 => X"f003e603", 27 => X"f003e603",
     28  => X"f0230ab7", 29 => X"f0611770", 30 => X"f0bd934b", 31 => X"f137ca18",
     32  => X"f1cecdbb", 33 => X"f28177f9", 34 => X"f34e6cba", 35 => X"f4341cae",
     36  => X"f34e6cba", 37 => X"f137ca18", 38 => X"f0230ab7", 39 => X"f0230ab7",
     40  => X"f137ca18", 41 => X"f34e6cba", 42 => X"f6428360", 43 => X"f9e08756",
     44  => X"fde95d5e", 45 => X"0216a2a2", 46 => X"061f78aa", 47 => X"09bd7ca0",
     48  => X"0cb19346", 49 => X"0ec835e8", 50 => X"0fdcf549", 51 => X"0fdcf549",
     52  => X"0ec835e8", 53 => X"0cb19346", 54 => X"09bd7ca0", 55 => X"061f78aa",
     56  => X"0216a2a2", 57 => X"fde95d5e", 58 => X"f9e08756", 59 => X"f6428360",
     60  => X"f34e6cba", 61 => X"f137ca18", 62 => X"f0230ab7", 63 => X"f0230ab7",
     64  => X"f137ca18", 65 => X"f34e6cba", 66 => X"f6428360", 67 => X"f9e08756",
     68  => X"fde95d5e", 69 => X"0216a2a2", 70 => X"061f78aa", 71 => X"09bd7ca0",
     72  => X"f7673887", 73 => X"fde95d5e", 74 => X"04cfb0e2", 75 => X"0acf37ad",
     76  => X"0ec835e8", 77 => X"0ffc19fd", 78 => X"0e313245", 79 => X"09bd7ca0",
     80  => X"03768962", 81 => X"fc89769e", 82 => X"f6428360", 83 => X"f1cecdbb",
     84  => X"f003e603", 85 => X"f137ca18", 86 => X"f530c853", 87 => X"fb304f1e",
     88  => X"0216a2a2", 89 => X"0898c779", 90 => X"0d7e8807", 91 => X"0fdcf549",
     92  => X"0f426cb5", 93 => X"0bcbe352", 94 => X"061f78aa", 95 => X"ff4d55c2",
     96  => X"f89cad7c", 97 => X"f34e6cba", 98 => X"f0611770", 99 => X"f0611770",
     100 => X"f34e6cba", 101 => X"f89cad7c", 102 => X"ff4d55c2", 103 => X"061f78aa",
     104 => X"0bcbe352", 105 => X"0f426cb5", 106 => X"0fdcf549", 107 => X"0d7e8807",
     108 => X"0e313245", 109 => X"0fdcf549", 110 => X"0bcbe352", 111 => X"03768962",
     112 => X"f9e08756", 113 => X"f28177f9", 114 => X"f003e603", 115 => X"f34e6cba",
     116 => X"fb304f1e", 117 => X"04cfb0e2", 118 => X"0cb19346", 119 => X"0ffc19fd",
     120 => X"0d7e8807", 121 => X"061f78aa", 122 => X"fc89769e", 123 => X"f4341cae",
     124 => X"f0230ab7", 125 => X"f1cecdbb", 126 => X"f89cad7c", 127 => X"0216a2a2",
     128 => X"0acf37ad", 129 => X"0f9ee890", 130 => X"0ec835e8", 131 => X"0898c779",
     132 => X"ff4d55c2", 133 => X"f6428360", 134 => X"f0bd934b", 135 => X"f0bd934b",
     136 => X"f6428360", 137 => X"ff4d55c2", 138 => X"0898c779", 139 => X"0ec835e8",
     140 => X"0f9ee890", 141 => X"0acf37ad", 142 => X"0216a2a2", 143 => X"f89cad7c",
     144 => X"061f78aa", 145 => X"f9e08756", 146 => X"f137ca18", 147 => X"f137ca18",
     148 => X"f9e08756", 149 => X"061f78aa", 150 => X"0ec835e8", 151 => X"0ec835e8",
     152 => X"061f78aa", 153 => X"f9e08756", 154 => X"f137ca18", 155 => X"f137ca18",
     156 => X"f9e08756", 157 => X"061f78aa", 158 => X"0ec835e8", 159 => X"0ec835e8",
     160 => X"061f78aa", 161 => X"f9e08756", 162 => X"f137ca18", 163 => X"f137ca18",
     164 => X"f9e08756", 165 => X"061f78aa", 166 => X"0ec835e8", 167 => X"0ec835e8",
     168 => X"061f78aa", 169 => X"f9e08756", 170 => X"f137ca18", 171 => X"f137ca18",
     172 => X"f9e08756", 173 => X"061f78aa", 174 => X"0ec835e8", 175 => X"0ec835e8",
     176 => X"061f78aa", 177 => X"f9e08756", 178 => X"f137ca18", 179 => X"f137ca18",
     180 => X"f0bd934b", 181 => X"f34e6cba", 182 => X"00b2aa3e", 183 => X"0d7e8807",
     184 => X"0ec835e8", 185 => X"03768962", 186 => X"f530c853", 187 => X"f0230ab7",
     188 => X"f89cad7c", 189 => X"07635284", 190 => X"0fdcf549", 191 => X"0acf37ad",
     192 => X"fc89769e", 193 => X"f137ca18", 194 => X"f28177f9", 195 => X"ff4d55c2",
     196 => X"0cb19346", 197 => X"0f426cb5", 198 => X"04cfb0e2", 199 => X"f6428360",
     200 => X"f003e603", 201 => X"f7673887", 202 => X"061f78aa", 203 => X"0f9ee890",
     204 => X"0bcbe352", 205 => X"fde95d5e", 206 => X"f1cecdbb", 207 => X"f1cecdbb",
     208 => X"fde95d5e", 209 => X"0bcbe352", 210 => X"0f9ee890", 211 => X"061f78aa",
     212 => X"f7673887", 213 => X"f003e603", 214 => X"f6428360", 215 => X"04cfb0e2",
     216 => X"fc89769e", 217 => X"0cb19346", 218 => X"0e313245", 219 => X"ff4d55c2",
     220 => X"f137ca18", 221 => X"f4341cae", 222 => X"04cfb0e2", 223 => X"0fdcf549",
     224 => X"0898c779", 225 => X"f7673887", 226 => X"f0230ab7", 227 => X"fb304f1e",
     228 => X"0bcbe352", 229 => X"0ec835e8", 230 => X"00b2aa3e", 231 => X"f1cecdbb",
     232 => X"f34e6cba", 233 => X"03768962", 234 => X"0f9ee890", 235 => X"09bd7ca0",
     236 => X"f89cad7c", 237 => X"f003e603", 238 => X"f9e08756", 239 => X"0acf37ad",
     240 => X"0f426cb5", 241 => X"0216a2a2", 242 => X"f28177f9", 243 => X"f28177f9",
     244 => X"0216a2a2", 245 => X"0f426cb5", 246 => X"0acf37ad", 247 => X"f9e08756",
     248 => X"f003e603", 249 => X"f89cad7c", 250 => X"09bd7ca0", 251 => X"0f9ee890",
     252 => X"0fdcf549", 253 => X"061f78aa", 254 => X"f34e6cba", 255 => X"f34e6cba",
     256 => X"061f78aa", 257 => X"0fdcf549", 258 => X"0216a2a2", 259 => X"f137ca18",
     260 => X"f6428360", 261 => X"09bd7ca0", 262 => X"0ec835e8", 263 => X"fde95d5e",
     264 => X"f0230ab7", 265 => X"f9e08756", 266 => X"0cb19346", 267 => X"0cb19346",
     268 => X"f9e08756", 269 => X"f0230ab7", 270 => X"fde95d5e", 271 => X"0ec835e8",
     272 => X"09bd7ca0", 273 => X"f6428360", 274 => X"f137ca18", 275 => X"0216a2a2",
     276 => X"0fdcf549", 277 => X"061f78aa", 278 => X"f34e6cba", 279 => X"f34e6cba",
     280 => X"061f78aa", 281 => X"0fdcf549", 282 => X"0216a2a2", 283 => X"f137ca18",
     284 => X"f6428360", 285 => X"09bd7ca0", 286 => X"0ec835e8", 287 => X"fde95d5e",
     288 => X"00b2aa3e", 289 => X"f0230ab7", 290 => X"fc89769e", 291 => X"0f426cb5",
     292 => X"061f78aa", 293 => X"f1cecdbb", 294 => X"f7673887", 295 => X"0cb19346",
     296 => X"0acf37ad", 297 => X"f530c853", 298 => X"f34e6cba", 299 => X"0898c779",
     300 => X"0e313245", 301 => X"f9e08756", 302 => X"f0bd934b", 303 => X"03768962",
     304 => X"0fdcf549", 305 => X"ff4d55c2", 306 => X"f003e603", 307 => X"fde95d5e",
     308 => X"0f9ee890", 309 => X"04cfb0e2", 310 => X"f137ca18", 311 => X"f89cad7c",
     312 => X"0d7e8807", 313 => X"09bd7ca0", 314 => X"f4341cae", 315 => X"f4341cae",
     316 => X"09bd7ca0", 317 => X"0d7e8807", 318 => X"f89cad7c", 319 => X"f137ca18",
     320 => X"04cfb0e2", 321 => X"0f9ee890", 322 => X"fde95d5e", 323 => X"f003e603",
     324 => X"f003e603", 325 => X"0216a2a2", 326 => X"0f9ee890", 327 => X"fb304f1e",
     328 => X"f137ca18", 329 => X"07635284", 330 => X"0d7e8807", 331 => X"f6428360",
     332 => X"f4341cae", 333 => X"0bcbe352", 334 => X"09bd7ca0", 335 => X"f28177f9",
     336 => X"f89cad7c", 337 => X"0ec835e8", 338 => X"04cfb0e2", 339 => X"f0611770",
     340 => X"fde95d5e", 341 => X"0ffc19fd", 342 => X"ff4d55c2", 343 => X"f0230ab7",
     344 => X"03768962", 345 => X"0f426cb5", 346 => X"f9e08756", 347 => X"f1cecdbb",
     348 => X"0898c779", 349 => X"0cb19346", 350 => X"f530c853", 351 => X"f530c853",
     352 => X"0cb19346", 353 => X"0898c779", 354 => X"f1cecdbb", 355 => X"f9e08756",
     356 => X"0f426cb5", 357 => X"03768962", 358 => X"f0230ab7", 359 => X"ff4d55c2",
     360 => X"0216a2a2", 361 => X"0ec835e8", 362 => X"f6428360", 363 => X"f6428360",
     364 => X"0ec835e8", 365 => X"0216a2a2", 366 => X"f0230ab7", 367 => X"061f78aa",
     368 => X"0cb19346", 369 => X"f34e6cba", 370 => X"f9e08756", 371 => X"0fdcf549",
     372 => X"fde95d5e", 373 => X"f137ca18", 374 => X"09bd7ca0", 375 => X"09bd7ca0",
     376 => X"f137ca18", 377 => X"fde95d5e", 378 => X"0fdcf549", 379 => X"f9e08756",
     380 => X"f34e6cba", 381 => X"0cb19346", 382 => X"061f78aa", 383 => X"f0230ab7",
     384 => X"0216a2a2", 385 => X"0ec835e8", 386 => X"f6428360", 387 => X"f6428360",
     388 => X"0ec835e8", 389 => X"0216a2a2", 390 => X"f0230ab7", 391 => X"061f78aa",
     392 => X"0cb19346", 393 => X"f34e6cba", 394 => X"f9e08756", 395 => X"0fdcf549",
     396 => X"0f9ee890", 397 => X"f6428360", 398 => X"f89cad7c", 399 => X"0ffc19fd",
     400 => X"f9e08756", 401 => X"f530c853", 402 => X"0f426cb5", 403 => X"fde95d5e",
     404 => X"f28177f9", 405 => X"0d7e8807", 406 => X"0216a2a2", 407 => X"f0bd934b",
     408 => X"0acf37ad", 409 => X"061f78aa", 410 => X"f003e603", 411 => X"07635284",
     412 => X"09bd7ca0", 413 => X"f0611770", 414 => X"03768962", 415 => X"0cb19346",
     416 => X"f1cecdbb", 417 => X"ff4d55c2", 418 => X"0ec835e8", 419 => X"f4341cae",
     420 => X"fb304f1e", 421 => X"0fdcf549", 422 => X"f7673887", 423 => X"f7673887",
     424 => X"0fdcf549", 425 => X"fb304f1e", 426 => X"f4341cae", 427 => X"0ec835e8",
     428 => X"ff4d55c2", 429 => X"f1cecdbb", 430 => X"0cb19346", 431 => X"03768962",
     432 => X"fb304f1e", 433 => X"f6428360", 434 => X"0ffc19fd", 435 => X"f7673887",
     436 => X"f9e08756", 437 => X"0f9ee890", 438 => X"f4341cae", 439 => X"fde95d5e",
     440 => X"0e313245", 441 => X"f1cecdbb", 442 => X"0216a2a2", 443 => X"0bcbe352",
     444 => X"f0611770", 445 => X"061f78aa", 446 => X"0898c779", 447 => X"f003e603",
     448 => X"09bd7ca0", 449 => X"04cfb0e2", 450 => X"f0bd934b", 451 => X"0cb19346",
     452 => X"00b2aa3e", 453 => X"f28177f9", 454 => X"0ec835e8", 455 => X"fc89769e",
     456 => X"f530c853", 457 => X"0fdcf549", 458 => X"f89cad7c", 459 => X"f89cad7c",
     460 => X"0fdcf549", 461 => X"f530c853", 462 => X"fc89769e", 463 => X"0ec835e8",
     464 => X"f28177f9", 465 => X"00b2aa3e", 466 => X"0cb19346", 467 => X"f0bd934b",
     468 => X"f137ca18", 469 => X"0ec835e8", 470 => X"f9e08756", 471 => X"f9e08756",
     472 => X"0ec835e8", 473 => X"f137ca18", 474 => X"061f78aa", 475 => X"061f78aa",
     476 => X"f137ca18", 477 => X"0ec835e8", 478 => X"f9e08756", 479 => X"f9e08756",
     480 => X"0ec835e8", 481 => X"f137ca18", 482 => X"061f78aa", 483 => X"061f78aa",
     484 => X"f137ca18", 485 => X"0ec835e8", 486 => X"f9e08756", 487 => X"f9e08756",
     488 => X"0ec835e8", 489 => X"f137ca18", 490 => X"061f78aa", 491 => X"061f78aa",
     492 => X"f137ca18", 493 => X"0ec835e8", 494 => X"f9e08756", 495 => X"f9e08756",
     496 => X"0ec835e8", 497 => X"f137ca18", 498 => X"061f78aa", 499 => X"061f78aa",
     500 => X"f137ca18", 501 => X"0ec835e8", 502 => X"f9e08756", 503 => X"f9e08756",
     504 => X"07635284", 505 => X"0216a2a2", 506 => X"f530c853", 507 => X"0f9ee890",
     508 => X"f137ca18", 509 => X"0898c779", 510 => X"00b2aa3e", 511 => X"f6428360",
     512 => X"0f426cb5", 513 => X"f0bd934b", 514 => X"09bd7ca0", 515 => X"ff4d55c2",
     516 => X"f7673887", 517 => X"0ec835e8", 518 => X"f0611770", 519 => X"0acf37ad",
     520 => X"fde95d5e", 521 => X"f89cad7c", 522 => X"0e313245", 523 => X"f0230ab7",
     524 => X"0bcbe352", 525 => X"fc89769e", 526 => X"f9e08756", 527 => X"0d7e8807",
     528 => X"f003e603", 529 => X"0cb19346", 530 => X"fb304f1e", 531 => X"fb304f1e",
     532 => X"0cb19346", 533 => X"f003e603", 534 => X"0d7e8807", 535 => X"f9e08756",
     536 => X"fc89769e", 537 => X"0bcbe352", 538 => X"f0230ab7", 539 => X"0e313245",
     540 => X"0d7e8807", 541 => X"f0230ab7", 542 => X"0f426cb5", 543 => X"f4341cae",
     544 => X"061f78aa", 545 => X"00b2aa3e", 546 => X"f89cad7c", 547 => X"0cb19346",
     548 => X"f0611770", 549 => X"0f9ee890", 550 => X"f34e6cba", 551 => X"07635284",
     552 => X"ff4d55c2", 553 => X"f9e08756", 554 => X"0bcbe352", 555 => X"f0bd934b",
     556 => X"0fdcf549", 557 => X"f28177f9", 558 => X"0898c779", 559 => X"fde95d5e",
     560 => X"fb304f1e", 561 => X"0acf37ad", 562 => X"f137ca18", 563 => X"0ffc19fd",
     564 => X"f1cecdbb", 565 => X"09bd7ca0", 566 => X"fc89769e", 567 => X"fc89769e",
     568 => X"09bd7ca0", 569 => X"f1cecdbb", 570 => X"0ffc19fd", 571 => X"f137ca18",
     572 => X"0acf37ad", 573 => X"fb304f1e", 574 => X"fde95d5e", 575 => X"0898c779",
     576 => X"f6428360", 577 => X"061f78aa", 578 => X"fde95d5e", 579 => X"fde95d5e",
     580 => X"061f78aa", 581 => X"f6428360", 582 => X"0cb19346", 583 => X"f137ca18",
     584 => X"0fdcf549", 585 => X"f0230ab7", 586 => X"0ec835e8", 587 => X"f34e6cba",
     588 => X"09bd7ca0", 589 => X"f9e08756", 590 => X"0216a2a2", 591 => X"0216a2a2",
     592 => X"f9e08756", 593 => X"09bd7ca0", 594 => X"f34e6cba", 595 => X"0ec835e8",
     596 => X"f0230ab7", 597 => X"0fdcf549", 598 => X"f137ca18", 599 => X"0cb19346",
     600 => X"f6428360", 601 => X"061f78aa", 602 => X"fde95d5e", 603 => X"fde95d5e",
     604 => X"061f78aa", 605 => X"f6428360", 606 => X"0cb19346", 607 => X"f137ca18",
     608 => X"0fdcf549", 609 => X"f0230ab7", 610 => X"0ec835e8", 611 => X"f34e6cba",
     612 => X"f4341cae", 613 => X"0cb19346", 614 => X"f28177f9", 615 => X"0e313245",
     616 => X"f137ca18", 617 => X"0f426cb5", 618 => X"f0611770", 619 => X"0fdcf549",
     620 => X"f003e603", 621 => X"0ffc19fd", 622 => X"f0230ab7", 623 => X"0f9ee890",
     624 => X"f0bd934b", 625 => X"0ec835e8", 626 => X"f1cecdbb", 627 => X"0d7e8807",
     628 => X"f34e6cba", 629 => X"0bcbe352", 630 => X"f530c853", 631 => X"09bd7ca0",
     632 => X"f7673887", 633 => X"07635284", 634 => X"f9e08756", 635 => X"04cfb0e2",
     636 => X"fc89769e", 637 => X"0216a2a2", 638 => X"ff4d55c2", 639 => X"ff4d55c2",
     640 => X"0216a2a2", 641 => X"fc89769e", 642 => X"04cfb0e2", 643 => X"f9e08756",
     644 => X"07635284", 645 => X"f7673887", 646 => X"09bd7ca0", 647 => X"f530c853"
     );

  type   STATE_TYPE is (idle, executing, writing);
  signal state : STATE_TYPE;

  signal writing_done : std_logic;

  signal CE          : std_logic;
  signal Valid_Input : std_logic;
  signal Op1         : std_logic_vector(0 to 31);
  signal Op2         : std_logic_vector(0 to 31);
  signal Op2_high    : std_logic_vector(0 to 31);
  signal Valid_Res   : std_logic;
  signal Result      : std_logic_vector(0 to 31);

  signal table_addr : std_logic_vector(0 to 9);

  signal Count_In    : std_logic_vector(0 to 9);
  signal Count_In_18 : std_logic_vector(0 to 4);
  signal Count_In_36 : std_logic_vector(0 to 5);

  signal Count_Out_18 : std_logic_vector(0 to 4);
  signal Count_out_36 : std_logic_vector(0 to 5);

  signal Valid_Res_part1 : std_logic;
  signal Valid_Res_part2 : std_logic;
  signal Valid_Res_part3 : std_logic;

  signal count         : std_logic_vector(0 to 5);
  signal FSL_M_Write_i : std_logic;

  signal new_res_sum       : std_logic_vector(0 to 31);
  signal old_res_sum       : std_logic_vector(0 to 31);
  signal old_res_sum_part1 : std_logic_vector(0 to 31);
  signal old_res_sum_part2 : std_logic_vector(0 to 31);
  signal old_res_sum_part3 : std_logic_vector(0 to 31);

  signal result_sum_out       : std_logic_vector(0 to 31);
  signal result_sum_out_part1 : std_logic_vector(0 to 31);
  signal result_sum_out_part2 : std_logic_vector(0 to 31);
  signal result_sum_out_part3 : std_logic_vector(0 to 31);

begin  -- architecture IMP

  -----------------------------------------------------------------------------
  -- The constant table
  -----------------------------------------------------------------------------
  Constant_Table_Low : process (Clk) is
  begin  -- process Constant_Table_Low
    if Clk'event and Clk = '1' then     -- rising clock edge
      op2 <= Rom_Table(to_integer(unsigned(table_addr)));
    end if;
  end process Constant_Table_Low;

  -----------------------------------------------------------------------------
  -- Handles the state of the core
  -----------------------------------------------------------------------------
  State_Machine : process (Clk, Reset) is
  begin  -- process State_Machine
    if Reset = '1' then                 -- asynchronous reset (active high)
      state <= idle;
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      case state is
        when Idle =>
          if (FSL_S_Exists = '1') then
            state <= executing;
          end if;
        when executing =>
          if (Count_In = "1010000111") then  -- 647
            state <= writing;
          end if;
        when writing =>
          if (writing_done = '1') then
            state <= idle;
          end if;
        when others => null;
      end case;
    end if;
  end process State_Machine;

  -----------------------------------------------------------------------------
  -- Handles the input side
  -----------------------------------------------------------------------------
  Count_In_Handler : process (Clk, Reset) is
  begin  -- process Count_In_Handler
    if Reset = '1' then                  -- asynchronous reset (active high)
      Count_In    <= (others => '0');
      Count_In_18 <= (others => '0');
      Count_In_36 <= (others => '0');
    elsif Clk'event and Clk = '1' then   -- rising clock edge
      if (writing_done = '1') then
        Count_In    <= (others => '0');
        Count_In_18 <= (others => '0');
        Count_In_36 <= (others => '0');
      elsif (CE = '1') then
        if (count_in /= "1010000111") then
          Count_In <= std_logic_vector(unsigned(Count_In) + 1);          
        end if;
        if (Count_In_36 = "100011") then
          Count_In_36 <= (others => '0');
          if (Count_In_18 = "10001") then  -- 17
            Count_In_18 <= std_logic_vector(unsigned(Count_In_18) + 1);
          end if;
        else
          Count_In_36 <= std_logic_vector(unsigned(Count_In_36) + 1);
        end if;
      end if;
    end if;
  end process Count_In_Handler;

  table_addr <= Count_In;

  Get_Op1 : process (Clk, Reset) is
  begin  -- process Get_Op1
    if Reset = '1' then                 -- asynchronous reset (active high)
      FSL_S_Read <= '0';
      Op1        <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      FSL_S_Read <= '0';
      if (state = executing) and (Count_In_36 = "000000") and (FSL_S_Exists = '1') then
        FSL_S_Read <= '1';
        Op1        <= FSL_S_Data;
      end if;
    end if;
  end process Get_Op1;

  Valid_Input_Handle : process (Clk, Reset) is
  begin  -- process Valid_Input_Handle
    if Reset = '1' then                 -- asynchronous reset (active high)
      Valid_Input <= '0';
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      Valid_Input <= '0';
      if (state = executing) then
        Valid_Input <= CE;
      end if;
    end if;
  end process Valid_Input_Handle;

  CE_Calc : process (Count_In_36, FSL_M_Full, FSL_S_Exists, state) is
  begin  -- process CE_Calc
    CE <= '0';
    if (state = executing) then
      CE <= '1';
      if (Count_In_36 = "000000" and FSL_S_Exists = '0') then
        CE <= '0';
      end if;
    end if;
    if (state = writing) then
      CE <= '1';
      if (FSL_M_Full = '1') then
        CE <= '0';
      end if;
    end if;
  end process CE_Calc;

  -----------------------------------------------------------------------------
  -- The mult module
  -----------------------------------------------------------------------------
  do_the_mult_I : do_the_mult
    port map (
      Clk         => Clk,               -- [in  std_logic]
      Reset       => Reset,             -- [in  std_logic]
      CE          => CE,                -- [in  std_logic]
      Valid_Input => Valid_Input,       -- [in  std_logic]
      Op1         => Op1,               -- [in  std_logic_vector(0 to 31)]
      Op2         => Op2,               -- [in  std_logic_vector(0 to 31)]
      Valid_Res   => Valid_Res,         -- [out std_logic]
      Result      => Result);           -- [out std_logic_vector(0 to 31)]

  -----------------------------------------------------------------------------
  -- Do the summation
  -----------------------------------------------------------------------------
  Count_36_Out : process (Clk, Reset) is
  begin  -- process Count_32_Out
    if Reset = '1' then                    -- asynchronous reset (active high)
      Count_Out_36 <= (others => '0');
    elsif Clk'event and Clk = '1' then     -- rising clock edge
      if (state = idle) then
        Count_Out_36 <= (others => '0');
      elsif (Valid_Res = '1') then
        if (Count_out_36 = "100011") then  -- 35
          Count_out_36 <= (others => '0');
        else
          Count_Out_36 <= std_logic_vector(unsigned(Count_Out_36) + 1);
        end if;
      end if;
    end if;
  end process Count_36_Out;

  Count_18_Out : process (Clk, Reset) is
  begin  -- process Count_18_Out
    if Reset = '1' then                 -- asynchronous reset (active high)
      Count_Out_18 <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (state = idle) then
        Count_Out_18 <= (others => '0');
      elsif (Valid_Res = '1') then
        if (Count_Out_36 = "100011") then
          Count_Out_18 <= std_logic_vector(unsigned(Count_Out_18) + 1);
        end if;
      end if;
    end if;
  end process Count_18_Out;


  Valid_Res_part1 <= Valid_Res when Count_Out_36(0 to 1) = "00" else '0';
  Valid_Res_part2 <= Valid_Res when Count_Out_36(0 to 1) = "01" else '0';
  Valid_Res_part3 <= Valid_Res when Count_Out_36(0 to 1) = "10" else '0';

  dual_register_part1 : dual_register
    generic map (
      C_SIZE => 32)                     -- [integer]
    port map (
      Clk    => Clk,                    -- [in  std_logic]
      RAdr   => Count(2 to 5),          -- [in  std_logic_vector(0 to 3)]
      WAdr   => Count_out_36(2 to 5),   -- [in  std_logic_vector(0 to 3)]
      WE     => Valid_Res_part1,        -- [in  std_logic]
      D_In   => new_res_sum,        -- [in  std_logic_vector(0 to C_SIZE-1)]
      WD_Out => old_res_sum_part1,  -- [out std_logic_vector(0 to C_SIZE-1)]
      RD_Out => Result_sum_out_part1);  -- [out std_logic_vector(0 to C_SIZE-1)]

  dual_register_part2 : dual_register
    generic map (
      C_SIZE => 32)                     -- [integer]
    port map (
      Clk    => Clk,                    -- [in  std_logic]
      RAdr   => Count(2 to 5),          -- [in  std_logic_vector(0 to 3)]
      WAdr   => Count_out_36(2 to 5),   -- [in  std_logic_vector(0 to 3)]
      WE     => Valid_Res_part2,        -- [in  std_logic]
      D_In   => new_res_sum,        -- [in  std_logic_vector(0 to C_SIZE-1)]
      WD_Out => old_res_sum_part2,  -- [out std_logic_vector(0 to C_SIZE-1)]
      RD_Out => Result_sum_out_part2);  -- [out std_logic_vector(0 to C_SIZE-1)]

  dual_register_part3 : dual_register
    generic map (
      C_SIZE => 32)                     -- [integer]
    port map (
      Clk    => Clk,                    -- [in  std_logic]
      RAdr   => Count(2 to 5),          -- [in  std_logic_vector(0 to 3)]
      WAdr   => Count_out_36(2 to 5),   -- [in  std_logic_vector(0 to 3)]
      WE     => Valid_Res_part3,        -- [in  std_logic]
      D_In   => new_res_sum,        -- [in  std_logic_vector(0 to C_SIZE-1)]
      WD_Out => old_res_sum_part3,  -- [out std_logic_vector(0 to C_SIZE-1)]
      RD_Out => Result_sum_out_part3);  -- [out std_logic_vector(0 to C_SIZE-1)]

  Old_value_Mux : process (Count_out_36, old_res_sum_part1, old_res_sum_part2,
                           old_res_sum_part3) is
  begin  -- process Old_value_Mux
    case Count_out_36(0 to 1) is
      when "00"   => old_res_sum <= old_res_sum_part1;
      when "01"   => old_res_sum <= old_res_sum_part2;
      when "10"   => old_res_sum <= old_res_sum_part3;
      when others => old_res_sum <= old_res_sum_part1;
    end case;
  end process Old_value_Mux;

  new_res_sum <= Result when Count_out_18 = "00000" else std_logic_vector(unsigned(old_res_sum) + unsigned(Result));


  FSL_M_Data_Mux: process (Result_sum_out_part1, Result_sum_out_part2,
                           Result_sum_out_part3, count) is
  begin  -- process FSL_M_Data_Mux
    case count(0 to 1) is
      when "00"  => FSL_M_Data <= Result_sum_out_part1;
      when "01"  => FSL_M_Data <= Result_sum_out_part2;
      when "10"  => FSL_M_Data <= Result_sum_out_part3;
      when others => FSL_M_Data <= Result_sum_out_part1;
    end case;
  end process FSL_M_Data_Mux;

  Write_To_FSL : process (Clk, Reset) is
  begin  -- process Write_To_FSL
    if Reset = '1' then                 -- asynchronous reset (active high)
      FSL_M_Write_i <= '0';
      writing_done  <= '0';
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      FSL_M_Write_i <= '0';
      if (state = writing) and (FSL_M_Full = '0') and (writing_done = '0') then
        if (Count_out_36 = "000000") and (Count_Out_18 = "10010") then
          FSL_M_Write_i <= '1';
          if (count = "100011") then
            writing_done <= '1';
          end if;
        end if;
      elsif (state = idle) then
        writing_done <= '0';
      end if;
    end if;
  end process Write_To_FSL;

  Write_To_FSL_count : process (Clk, Reset) is
  begin  -- process Write_To_FSL_count
    if Reset = '1' then                 -- asynchronous reset (active high)
      count <= (others => '0');
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (state = idle) then
        count <= (others => '0');        
      elsif (FSL_M_Write_i = '1' and FSL_M_Full = '0') then
        count <= std_logic_vector(unsigned(count) + 1);
      end if;
    end if;
  end process Write_To_FSL_count;

  FSL_M_Write <= FSL_M_Write_i and not FSL_M_Full;

  FSL_M_Control <= '0';
  FSL_M_Clk     <= Clk;
end architecture IMP;
