-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- fsl_mp3_mul.vhd - Entity and architecture
--
--
-------------------------------------------------------------------------------
-- Filename:        fsl_mp3_mul.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              fsl_mp3_mul.vhd
--
-------------------------------------------------------------------------------
-- Author:          
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--     2003-11-17    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity fsl_mp3_mul is
  generic (
    SHIFT_VAL     : integer  := 28  -- should be greater than 16
  );
  port (
    Clk           : in  std_logic;
    Reset         : in  std_logic;
    FSL_S_Clk     : out std_logic;
    FSL_S_Read    : out std_logic;
    FSL_S_Data    : in  std_logic_vector(0 to 31);
    FSL_S_Control : in  std_logic;
    FSL_S_Exists  : in  std_logic;

    FSL_M_Clk     : out std_logic;
    FSL_M_Write   : out std_logic;
    FSL_M_Data    : out std_logic_vector(0 to 31);
    FSL_M_Control : out std_logic;
    FSL_M_Full    : in  std_logic
    );
end entity fsl_mp3_mul;

library unisim;
use unisim.all;

library fsl_mp3_mul_v1_00_a;
use fsl_mp3_mul_v1_00_a.do_the_mult;

architecture IMP of fsl_mp3_mul is

  component do_the_mult is
    generic (
      SHIFT_VAL   : integer := 28
    );
    port (
      Clk         : in  std_logic;
      Reset       : in  std_logic;
      CE          : in  std_logic;
      Valid_Input : in  std_logic;
      Op1         : in  std_logic_vector(0 to 31);
      Op2         : in  std_logic_vector(0 to 31);
      Valid_Res   : out std_logic;
      Result      : out std_logic_vector(0 to 31));
  end component do_the_mult;

  signal writing_done : std_logic;

  signal CE          : std_logic;
  signal Valid_Input : std_logic;
  signal Op1_Valid   : std_logic;
  signal Op1         : std_logic_vector(0 to 31);
  signal Op2         : std_logic_vector(0 to 31);
  signal Valid_Res   : std_logic;
  signal Result      : std_logic_vector(0 to 31);

  signal FSL_M_Write_i : std_logic;

begin  -- architecture IMP

  Get_Op : process (Clk, Reset) is
  begin  -- process Get_Op1
    if Reset = '1' then                 -- asynchronous reset (active high)
      Op1        <= (others => '0');
      Op1_Valid  <= '0';
    elsif Clk'event and Clk = '1' then  -- rising clock edge
      if (FSL_S_Exists = '1' and FSL_M_Full = '0') then
        Op1_Valid <= not Op1_Valid;
        if (Op1_Valid = '0') then
          Op1        <= FSL_S_Data;
        end if;
      end if;
    end if;
  end process Get_Op;

  FSL_S_Read <= FSL_S_Exists and not FSL_M_Full;
  Valid_Input <= FSL_S_Exists and not FSL_M_Full and Op1_Valid;
  
  op2 <= FSL_S_Data;

  CE <= not FSL_M_Full;
  
  -----------------------------------------------------------------------------
  -- The mult module
  -----------------------------------------------------------------------------
  do_the_mult_I : do_the_mult
    generic map (
      SHIFT_VAL   => SHIFT_VAL
    )		 
    port map (
      Clk         => Clk,               -- [in  std_logic]
      Reset       => Reset,             -- [in  std_logic]
      CE          => CE,                -- [in  std_logic]
      Valid_Input => Valid_Input,       -- [in  std_logic]
      Op1         => Op1,               -- [in  std_logic_vector(0 to 31)]
      Op2         => Op2,               -- [in  std_logic_vector(0 to 31)]
      Valid_Res   => Valid_Res,         -- [out std_logic]
      Result      => Result);           -- [out std_logic_vector(0 to 31)]

  FSL_M_Write <= Valid_Res and CE;
  FSL_M_Data  <= Result;

  FSL_M_Control <= '0';
  FSL_M_Clk     <= Clk;

end architecture IMP;
