-------------------------------------------------------------------------------
-- $Id: opb_zbt_controller.vhd,v 1.5 2002/03/29 22:25:16 goran Exp $
-------------------------------------------------------------------------------
-- opb_zbt_controller.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        opb_zbt_controller.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              opb_zbt_controller.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision: 1.5 $
-- Date:            $Date: 2002/03/29 22:25:16 $
--
-- History:
--   goran  2001-06-12    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

library common_v1_00_a;
use common_v1_00_a.pselect;

library unisim;
use unisim.all;

entity Opb_zbt_controller is
  generic (
    C_BASEADDR      : std_logic_vector(0 to 31) := X"FF700000";
    C_HIGHADDR      : std_logic_vector(0 to 31) := X"FF77FFFF";
    C_ZBT_ADDR_SIZE : integer                   := 17;
    -- If C_EXTERNAL_DLL is true (1) then
    -- ZBT_Clk is driven outside this module
    -- and ZBT_Clk_FBOut is also driven outside this module
    C_EXTERNAL_DLL  : integer                   := 0
    );
  port (
    OPB_Clk : in std_logic;
    OPB_Rst : in std_logic;

    -- OPB signals
    OPB_ABus    : in std_logic_vector(0 to 31);
    OPB_BE      : in std_logic_vector(0 to 3);
    OPB_RNW     : in std_logic;
    OPB_select  : in std_logic;
    OPB_seqAddr : in std_logic;
    OPB_DBus    : in std_logic_vector(0 to 31);

    ZBT_DBus    : out std_logic_vector(0 to 31);
    ZBT_errAck  : out std_logic;
    ZBT_retry   : out std_logic;
    ZBT_toutSup : out std_logic;
    ZBT_xferAck : out std_logic;

    -- ZBT Memory signals
    ZBT_Clk_FB    : in  std_logic;
    ZBT_Clk_FBOut : out std_logic;
    ZBT_Clk       : out std_logic;
    ZBT_OE_N      : out std_logic;
    ZBT_ADV_LD_N  : out std_logic;
    ZBT_LBO_N     : out std_logic;
    ZBT_CE1_N     : out std_logic;
    ZBT_CE2_N     : out std_logic;
    ZBT_CE2       : out std_logic;
    ZBT_RW_N      : out std_logic;
    ZBT_CKE_N     : out std_logic;
    ZBT_A         : out std_logic_vector(0 to C_ZBT_ADDR_SIZE-1);
    ZBT_BW_N      : out std_logic_vector(1 to 4);
    ZBT_IO_I      : in  std_logic_vector(0 to 31);
    ZBT_IO_O      : out std_logic_vector(0 to 31);
    ZBT_IO_T      : out std_logic;
    ZBT_IOP_I     : in  std_logic_vector(1 to 4);
    ZBT_IOP_O     : out std_logic_vector(1 to 4);
    ZBT_IOP_T     : out std_logic
    );

end entity Opb_zbt_controller;

architecture IMP of Opb_zbt_controller is

  component Opb_zbt_controller_Core is
    generic (
      C_ZBT_ADDR_SIZE : integer;
      C_EXTERNAL_DLL  : integer
      );
    port (
      Clk   : in std_logic;
      Reset : in std_logic;

      ZBT_CS : in std_logic;

      -- OPB signals
      OPB_ABus    : in std_logic_vector(0 to C_ZBT_ADDR_SIZE-1);
      OPB_BE      : in std_logic_vector(0 to 3);
      OPB_RNW     : in std_logic;
      OPB_seqAddr : in std_logic;
      OPB_DBus    : in std_logic_vector(0 to 31);

      ZBT_DBus    : out std_logic_vector(0 to 31);
      ZBT_xferAck : out std_logic;

      -- ZBT Memory signals
      ZBT_Clk_FB    : in  std_logic;
      ZBT_Clk_FBOut : out std_logic;
      ZBT_Clk       : out std_logic;
      ZBT_OE_N      : out std_logic;
      ZBT_ADV_LD_N  : out std_logic;
      ZBT_LBO_N     : out std_logic;
      ZBT_CE1_N     : out std_logic;
      ZBT_CE2_N     : out std_logic;
      ZBT_CE2       : out std_logic;
      ZBT_RW_N      : out std_logic;
      ZBT_CKE_N     : out std_logic;
      ZBT_A         : out std_logic_vector(0 to C_ZBT_ADDR_SIZE-1);
      ZBT_BW_N      : out std_logic_vector(1 to 4);
      ZBT_IO_I      : in  std_logic_vector(0 to 31);
      ZBT_IO_O      : out std_logic_vector(0 to 31);
      ZBT_IO_T      : out std_logic;
      ZBT_IOP_I     : in  std_logic_vector(1 to 4);
      ZBT_IOP_O     : out std_logic_vector(1 to 4);
      ZBT_IOP_T     : out std_logic
      );
  end component Opb_zbt_controller_Core;

  component pselect is
    generic (
      C_AB  : integer;
      C_AW  : integer;
      C_BAR : std_logic_vector);
    port (
      A      : in  std_logic_vector(0 to C_AW-1);
      AValid : in  std_logic;
      ps     : out std_logic);
  end component pselect;

  constant C_OPB_AWIDTH : natural := 32;
  
  function Addr_Bits (x, y : std_logic_vector(0 to C_OPB_AWIDTH-1)) return integer is
    variable addr_nor : std_logic_vector(0 to C_OPB_AWIDTH-1);
  begin
    addr_nor := x xor y;
    for i in 0 to C_OPB_AWIDTH-1 loop
      if addr_nor(i) = '1' then return i;
      end if;
    end loop;
    return(C_OPB_AWIDTH);
  end function Addr_Bits;

  constant C_AB : integer := Addr_Bits(C_HIGHADDR, C_BASEADDR);

  signal zbt_cs_I        : std_logic;
  signal zbt_cs_Previous : std_logic;
  signal zbt_cs          : std_logic;

  signal xferAck_i : std_logic;
  signal cs_rst     : std_logic;
  
begin  -- architecture IMP

  -----------------------------------------------------------------------------
  -- Detecting an access to the ZBT memory
  -----------------------------------------------------------------------------
  pselect_I : pselect
    generic map (
      C_AB   => C_AB,                   -- [integer]
      C_AW   => OPB_ABus'length,        -- [integer]
      C_BAR  => C_BASEADDR)             -- [std_logic_vector]
    port map (
      A      => OPB_ABus,               -- [in  std_logic_vector(0 to C_AW-1)]
      AValid => OPB_select,             -- [in  std_logic]
      ps     => zbt_cs_I);              -- [out std_logic]

  cs_rst <= OPB_Rst or xferAck_i;
  
  zbt_cs_I_DFF : process (OPB_Clk) is
  begin  -- process zbt_cs_I_DFF
    if OPB_Clk'event and OPB_Clk = '1' then  -- rising clock edge
      if cs_Rst = '1' then               -- asynchronous reset (active high)
        zbt_cs_Previous <= '0';
      else
        zbt_cs_Previous <= zbt_cs_I;
      end if;
    end if;
  end process zbt_cs_I_DFF;

  zbt_cs <= zbt_cs_I and not zbt_cs_Previous;

  ZBT_errAck  <= '0';
  ZBT_retry   <= '0';
  ZBT_toutSup <= '0';

  -----------------------------------------------------------------------------
  -- Instanciating the core
  -----------------------------------------------------------------------------
  Opb_zbt_controller_Core_I : Opb_zbt_controller_Core
    generic map (
      C_ZBT_ADDR_SIZE => C_ZBT_ADDR_SIZE,                   -- [integer]
      C_EXTERNAL_DLL  => C_EXTERNAL_DLL                     -- [integer]
      )
    port map (
      Clk           => OPB_Clk,         -- [in  std_logic]
      Reset         => OPB_Rst,         -- [in  std_logic]
      ZBT_CS        => ZBT_CS,          -- [in  std_logic]
      -- OPB signals
      OPB_ABus      => OPB_ABus(30-C_ZBT_ADDR_SIZE to 29),  -- [in  std_logic_vector(0 to C_ZBT_ADDR_SIZE-1)]
      OPB_BE        => OPB_BE,          -- [in  std_logic_vector(0 to 3)]
      OPB_RNW       => OPB_RNW,         -- [in  std_logic]
      OPB_seqAddr   => OPB_seqAddr,     -- [in  std_logic]
      OPB_DBus      => OPB_DBus,        -- [in  std_logic_vector(0 to 31)]
      ZBT_DBus      => ZBT_DBus,        -- [out std_logic_vector(0 to 31)]
      ZBT_xferAck   => xferAck_i,       -- [out std_logic]
      -- ZBT Memory signals
      ZBT_Clk_FB    => ZBT_Clk_FB,      -- [in  std_logic]
      ZBT_Clk_FBOut => ZBT_Clk_FBOut,   -- [in  std_logic]
      ZBT_Clk       => ZBT_Clk,         -- [out std_logic]
      ZBT_OE_N      => ZBT_OE_N,        -- [out std_logic]
      ZBT_ADV_LD_N  => ZBT_ADV_LD_N,    -- [out std_logic]
      ZBT_LBO_N     => ZBT_LBO_N,       -- [out std_logic]
      ZBT_CE1_N     => ZBT_CE1_N,       -- [out std_logic]
      ZBT_CE2_N     => ZBT_CE2_N,       -- [out std_logic]
      ZBT_CE2       => ZBT_CE2,         -- [out std_logic]
      ZBT_RW_N      => ZBT_RW_N,        -- [out std_logic]
      ZBT_CKE_N     => ZBT_CKE_N,       -- [out std_logic]
      ZBT_A         => ZBT_A,  -- [out std_logic_vector(0 to C_ZBT_ADDR_SIZE-1)]
      ZBT_BW_N      => ZBT_BW_N,        -- [out std_logic_vector(1 to 4)]
      ZBT_IO_I      => ZBT_IO_I,        -- [in  std_logic_vector(0 to 31)]
      ZBT_IO_O      => ZBT_IO_O,        -- [out std_logic_vector(0 to 31)]
      ZBT_IO_T      => ZBT_IO_T,        -- [out std_logic]
      ZBT_IOP_I     => ZBT_IOP_I,       -- [in  std_logic_vector(1 to 4)]
      ZBT_IOP_O     => ZBT_IOP_O,       -- [out std_logic_vector(1 to 4)]
      ZBT_IOP_T     => ZBT_IOP_T);      -- [out std_logic]

  ZBT_xferAck <= xferAck_i;
  
end architecture IMP;
